
-----------------------------------------------------------------------------
-- |
-- Maintainer  :  dtlapa@yahoo.com, flavioxavier@gmail.com, hpacheco@gmail.com
-- Stability   :  experimental
-- Portability :  portable
--
-- See
--
--     PostGreSQL Abstract Syntax Tree Data Types ('ASTPostgreSQL')
-- 
-- by
--
--     Digo Lapa, Flavio Ferreira and Hugo Pacheco
--
-----------------------------------------------------------------------------

module Data.SQL.ASTPostgreSQL where

import Text.XML.HaXml.OneOfN

-- PostgreSQL Abstract Syntax Tree Data Types (ASTPostgreSQL)

-- * Data Types

-- ** 'ASTPostgreSQL'

data ASTPostgreSQL = AST Statements
	deriving Show

type Statements = [Statement]

data Statement
	= AlterDatabaseStmt DatabaseName OptWith AlterdbOptList
	| AlterDatabaseSetStmt DatabaseName (Either SetRest Statement)
	| AlterDomainStmt AnyName (OneOf5 AlterColumnDefault () () TableConstraint (Name,OptDropBehavior))
	| AlterFunctionStmt FunctionWithArgTypes AlterFuncOptList OptRestrict
	| AlterGroupStmt RoleId AddDrop NameList
	| AlterObjectSchemaStmt AlterObjectSchemaStatement
	| AlterOwnerStmt AlterOwnerStatement
	| AlterSeqStmt QualifiedName OptSeqList
	| AlterTableStmt RelationExpr (Either AlterTableCmds AlterRelCmds)
	| AlterRoleSetStmt RoleId (Either SetRest Statement)
	| AlterRoleStmt RoleId OptWith OptRoleList
	| AlterUserSetStmt RoleId (Either SetRest Statement)
	| AlterUserStmt RoleId OptWith OptRoleList
	| AnalyzeStmt AnalyzeKeyword OptVerbose (Maybe (QualifiedName,OptNameList))
	| CheckPointStmt
	| ClosePortalStmt Name
	| ClusterStmt (Maybe (Maybe IndexName,QualifiedName))
	| CommentStmt CommentOn
	| ConstraintsSetStmt ConstraintsSetList ConstraintsSetMode
	| CopyStmt OptBinary QualifiedName OptColumnList OptOids CopyFrom CopyFileName CopyDelimiter OptWith CopyOptList
	| CreateAsStmt OptTemp QualifiedName OptCreateAs WithOidsAs Statement
	| CreateAssertStmt Name AExpr ConstraintAttributeSpec
	| CreateCastStmt Typename Typename (Maybe FunctionWithArgTypes) CastContext
	| CreateConversionStmt OptDefault AnyName Sconst Sconst AnyName
	| CreateDomainStmt AnyName OptAs Typename ColQualList
	| CreateFunctionStmt OptOrReplace FuncName FuncArgs (Maybe FuncReturn) CreateFuncOptList OptDefinition
	| CreateGroupStmt RoleId OptWith OptRoleList
	| CreateOpClassStmt AnyName OptDefault Typename AccessMethod OpClassItemList
	| CreatePLangStmt OptTrusted OptProcedural ColIdOrSconst (Maybe (HandlerName,OptValidator,OptLancompiler))
	| CreateSchemaStmt (Maybe OptSchemaName) RoleId OptSchemaEltList
	| CreateSeqStmt OptTemp QualifiedName OptSeqList
	| CreateStmt OptTemp QualifiedName (Maybe QualifiedName) OptTableElementList (Maybe OptInherit) OptWithOids OnCommitOption OptTableSpace
	| CreateTableSpaceStmt Name OptTableSpaceOwner Sconst
	| CreateTrigStmt Name (Maybe TriggerActionTime) TriggerEvents QualifiedName (Either TriggerForSpec(OptConstrFromTable,ConstraintAttributeSpec)) FuncName TriggerFuncArgs
	| CreateRoleStmt RoleId OptWith OptRoleList
	| CreateUserStmt RoleId OptWith OptRoleList
	| CreatedbStmt DatabaseName OptWith CreatedbOptList
	| DeallocateStmt (Either Name Name)
	| DeclareCursorStmt Name CursorOptions OptHold Statement
	| DefineStmt (OneOf3 (FuncName,Definition) (AnyOperator,Definition) (AnyName,Either Definition TableFuncElementList))
	| DeleteStmt RelationExpr UsingClause WhereClause
	| DropAssertStmt Name OptDropBehavior
	| DropCastStmt Typename Typename OptDropBehavior
	| DropGroupStmt NameList
	| DropOpClassStmt AnyName AccessMethod OptDropBehavior
	| DropPLangStmt OptProcedural ColIdOrSconst OptDropBehavior
	| DropRuleStmt Name QualifiedName OptDropBehavior
	| DropStmt DropType AnyNameList OptDropBehavior
	| DropTableSpaceStmt Name
	| DropTrigStmt Name QualifiedName OptDropBehavior
	| DropRoleStmt NameList
	| DropUserStmt NameList
	| DropdbStmt DatabaseName
	| ExecuteStmt ExecuteStatement
	| ExplainStmt OptAnalyze OptVerbose ExplainableStmt
	| FetchStmt (Either Fetch Move) (Maybe (FetchDirection,FromIn)) Name
	| GrantStmt Privileges PrivilegeTarget GranteeList OptGrantGrantOption
	| GrantRoleStmt PrivilegeList NameList OptGrantAdminOption OptGrantedBy
	| IndexStmt IndexOptUnique IndexName QualifiedName AccessMethodClause IndexParams OptTableSpace WhereClause
	| InsertStmt QualifiedName InsertRest
	| ListenStmt QualifiedName
	| LoadStmt FileName
	| LockStmt OptTable QualifiedNameList OptLock OptNowait
	| NotifyStmt QualifiedName
	| PrepareStmt Name PrepTypeClause PreparableStmt
	| ReindexStmt (Either (ReindexType,QualifiedName) (Either System Database,Name)) OptForce
	| RemoveAggrStmt FuncName AggrArgType OptDropBehavior
	| RemoveFuncStmt FuncName FuncArgs OptDropBehavior
	| RemoveOperStmt AnyOperator OperArgTypes OptDropBehavior
	| RenameStmt RenameStatement
	| RevokeStmt (Either (On,Privileges,PrivilegeTarget,GranteeList,OptDropBehavior) (For,Privileges,PrivilegeTarget,GranteeList,OptDropBehavior))
	| RevokeRoleStmt (Either (PrivilegeList,NameList,OptGrantedBy,OptDropBehavior) ((PrivilegeList,NameList,OptGrantedBy,OptDropBehavior)))
	| RuleStmt OptOrReplace Name Event QualifiedName WhereClause OptInstead RuleActionList
	| SelectStmt (Either SelectNoParens SelectWithParens)
	| TransactionStmt TransactionStatement
	| TruncateStmt OptTable QualifiedNameList
	| UnlistenStmt (Either QualifiedName Star)
	| UpdateStmt RelationExpr UpdateTargetList FromClause WhereClause
	| VacuumStmt OptFull OptFreeze OptVerbose (Maybe (Either QualifiedName Statement))
	| VariableResetStmt (OneOf5 VarName (Time,Zone) (Transaction,Isolation,Level) (Session,Authorization) All)
	| VariableSetStmt (Maybe (Either Local Session)) SetRest
	| VariableShowStmt (OneOf5 VarName (Time,Zone) (Transaction,Isolation,Level) (Session,Authorization) All)
	| ViewStmt (Maybe Replace) OptTemp QualifiedName OptColumnList Statement
	deriving Show

type AddDrop = Either Add Drop

type OptSchemaName = Maybe ColId

type OptSchemaEltList = [SchemaStmt]

type SchemaStmt = Statement

data SetRest
	= SetTo VarName VarListOrDefault
	| SetEqual VarName VarListOrDefault
	| SetTimeZone ZoneValue
	| SetTransaction (Either TransactionModeList TransactionModeList)
	| SetName OptEncoding
	| SetRole (Either ColId Sconst)
	| SetSessionAuthorization (Either ColIdOrSconst Default)
	deriving Show

type VarName = MyList ColId

type VarListOrDefault = Either VarList Default

type VarList = MyList VarValue

type VarValue = OneOf3 OptBoolean ColIdOrSconst NumericOnly

data IsoLevel
	= IsoReadUncommited
	| IsoReadCommited
	| IsoRepeatableRead
	| IsoSerializable
	deriving Show

data OptBoolean
	= OptBoolTrue
	| OptBoolFalse
	| OptBoolOn
	| OptBoolOff
	deriving Show

type OptWith = Maybe With

type OptRoleList = [OptRoleElem]

data OptRoleElem
	= RolePassword Sconst
	| RoleEncryptedPassword Sconst
	| RoleUnencryptedPassword Sconst
	| RoleSuperUser
	| RoleNoSuperUser
	| RoleInherit
	| RoleNoInherit
	| RoleCreateDB
	| RoleNoCreateDB
	| RoleCreateRole
	| RoleNoCreateRole
	| RoleCreateUser
	| RoleNoCreateUser
	| RoleLogin
	| RoleNoLogin
	| RoleConnectionLimit SignedIconst
	| RoleValidUntil Sconst
	| RoleUser NameList
	| RoleSysId Iconst
	| RoleAdmin NameList
	| RoleRole NameList
	| RoleInRole NameList
	| RoleInGroup NameList
	deriving Show

data ZoneValue
	= ZoneSconst Sconst
	| ZoneIdent
	| ZoneConstInterval ConstInterval (Maybe Iconst) Sconst OptInterval
	| ZoneNum NumericOnly
	| ZoneDefault
	| ZoneLocal
	deriving Show

type OptEncoding = Maybe (Either Sconst Default)

type ColIdOrSconst = Either ColId Sconst


type ConstraintsSetList = Either All NameList

type ConstraintsSetMode = Either Deferred Immediate


type AlterTableCmds = MyList AlterTableCmd

data AlterTableCmd
	= AlterTableAdd (Either (OptColumn,ColumnDef) TableConstraint)
	| AlterTableAlter OptColumn ColId (OneOf4 AlterColumnDefault (Drop,Not,Null) (Set,OneOf3 (Not,Null) (Statistics,IntegerOnly) (Storage,ColId)) (Type,Typename,AlterUsing))
	| AlterTableDrop (Either (OptColumn,ColId) (Constraint,Name)) OptDropBehavior
	| AlterTableCreateToastTable
	| AlterTableClusterOn Name
	| AlterTableSetWithout (Either Oids Cluster)
	| AlterTableEnableTrigger (OneOf3 Name All User)
	| AlterTableDisableTrigger (OneOf3 Name All User)
	| AlterTableRelCmd AlterRelCmd
	deriving Show

type AlterRelCmds = MyList AlterRelCmd

data AlterRelCmd
	= AlterRelCmdOwnerTo RoleId
	| AlterRelCmdSetTablespace Name
	deriving Show

data AlterColumnDefault
	= AlterColumnSetDefault AExpr
	| AlterColumnDropDefault
	deriving Show

type OptDropBehavior = Maybe (Either Cascade Restrict)
	
data AlterUsing = AlterUse (Maybe AExpr)
	deriving Show
	
type CopyFrom = Either From To

type CopyFileName = OneOf3 Sconst Stdin Stdout
	
type CopyOptList = [CopyOptItem]

data CopyOptItem
	= CpItemBinary
	| CpItemOids
	| CpItemDelimiter OptAs Sconst
	| CpItemNull OptAs Sconst
	| CpItemCsv
	| CpItemHeader
	| CpItemQuote OptAs Sconst
	| CpItemEscape OptAs Sconst
	| CpItemForceQuote ColumnList
	| CpItemForceNotNull ColumnList
	deriving Show

type OptBinary = Maybe Binary

type OptOids = Maybe (With,Oids)

type CopyDelimiter = Maybe (OptUsing,Delimiters,Sconst)

type OptUsing = Maybe Using

type OptTemp = Maybe OptTempAux

data OptTempAux
	= TmpTemporary
	| TmpTemp
	| TmpLocalTemporary
	| TmpLocalTemp
	| TmpGlobalTemporary
	| TmpGlobalTemp
	deriving Show

type OptTableElementList = Maybe TableElementList

type TableElementList = MyList TableElement

type TableElement = OneOf3 ColumnDef TableLikeClause TableConstraint

data ColumnDef = CDef ColId Typename ColQualList
	deriving Show

type ColQualList = [ColConstraint]

type ColConstraint = OneOf3 (Constraint,Name,ColConstraintElem) ColConstraintElem ConstraintAttr

data ColConstraintElem
	= CstrNotNull
	| CstrNull
	| CstrUnique OptConsTableSpace
	| CstrPrimaryKey OptConsTableSpace
	| CstrCheck AExpr
	| CstrDefault BExpr
	| CstrReferences QualifiedName OptColumnList KeyMatch KeyActions
	deriving Show

data ConstraintAttr
	= CstrAttrDeferrable
	| CstrAttrNotDeferrable
	| CstrAttrInitiallyDeferred
	| CstrAttrInitiallyImmediate
	deriving Show

data TableLikeClause = TLike QualifiedName LikeIncludingDefaults
	deriving Show

type LikeIncludingDefaults = Maybe LikeIncludingDefaultsAux

data LikeIncludingDefaultsAux
	= LikeIncludingDefaults
	| LikeExcludingDefaults
	deriving Show

data TableConstraint
	= TConstraint (Maybe Name) ConstraintElem
	deriving Show

data ConstraintElem
	= CstrElemCheck AExpr
	| CstrElemUnique ColumnList OptConsTableSpace
	| CstrElemPrimaryKey ColumnList OptConsTableSpace
	| CstrElemForeignKey ColumnList QualifiedName OptColumnList KeyMatch KeyActions ConstraintAttributeSpec
	deriving Show

data OptColumnList = OptColList (Maybe ColumnList)
	deriving Show

type ColumnList = MyList ColumnElem

type ColumnElem = ColId

type KeyMatch = Maybe KeyMatchAux

data KeyMatchAux
	= MatchFull
	| MatchPartial
	| MatchSimple
	deriving Show

type KeyActions = Maybe (OneOf3 KeyUpdate KeyDelete (KeyUpdate,KeyDelete))

data KeyUpdate = OnUpdate KeyAction
	deriving Show

data KeyDelete = OnDelete KeyAction
	deriving Show

data KeyAction
	= KeyNoAction
	| KeyRestrict
	| KeyCascade
	| KeySetNull
	| KeySetDefault
	deriving Show

data OptInherit = OptInherits (Maybe (QualifiedNameList))
	deriving Show

data OptWithOids
	= WithOids
	| WithoutOids
	| NoOids
	deriving Show

type OnCommitOption = Maybe OnCommitOptionAux


data OnCommitOptionAux
	= OnCommitDrop
	| OnCommitDeleteRows
	| OnCommitPreserveRows
	deriving Show

data OptTableSpace = OptTableSpaceConst (Maybe Name)
	deriving Show

data OptConsTableSpace = OptConsTableSpaceConst (Maybe Name)
	deriving Show

data WithOidsAs
	= WOA_WithOidsAs
	| WOA_WithoutOidsAs
	| WOA_As
	deriving Show

data OptCreateAs = OptCreateAsConst (Maybe CreateAsList)
	deriving Show

type CreateAsList = MyList CreateAsElement

type CreateAsElement = ColId

type OptSeqList = [OptSeqElem]

data OptSeqElem
	= SeqElemCache NumericOnly
	| SeqElemCycle
	| SeqElemNoCycle
	| SeqElemIncrement OptBy NumericOnly
	| SeqElemMaxValue NumericOnly
	| SeqElemMinValue NumericOnly
	| SeqElemNoMaxValue
	| SeqElemNoMinValue
	| SeqElemStart OptWith NumericOnly
	| SeqElemRestart OptWith NumericOnly
	deriving Show

type NumericOnly = Either FloatOnly IntegerOnly

data FloatOnly
	= FloatConst Fconst
	| MinusFloatConst Fconst
	deriving Show

type IntegerOnly = SignedIconst

type OptTrusted = Maybe Trusted

type HandlerName = Either Name (Name,Attrs)


data OptValidator = OptVal (Maybe HandlerName)
	deriving Show

data OptLancompiler = OptLanComp (Maybe Sconst)
	deriving Show

type OptProcedural = Maybe Procedural

data OptTableSpaceOwner = OptTSOwner (Maybe Name)
	deriving Show

type TriggerActionTime = Either Before After

data TriggerEvents
	= OneTrigger TriggerOneEvent
	| TwoTriggers TriggerOneEvent TriggerOneEvent
	| ThreeTriggers TriggerOneEvent TriggerOneEvent TriggerOneEvent
	deriving Show

type TriggerOneEvent = OneOf3 Insert Delete Update

data TriggerForSpec = TriggerForSpecConst (Maybe (TriggerForOpt,TriggerForType))
	deriving Show

type TriggerForOpt = Maybe Each

type TriggerForType = Either RowString StatementString

data StatementString = STATEMENT
	deriving Show

type TriggerFuncArgs = MyList TriggerFuncArg

type Ident = String
type Iconst = Int
type Fconst = String
type Sconst = String
type Bconst = String
type Xconst = String

type TriggerFuncArg = OneOf6 Iconst Fconst Sconst Bconst Xconst ColId

data OptConstrFromTable = OptConstrFromTableConst (Maybe QualifiedName)
	deriving Show

type ConstraintAttributeSpec = (Maybe ConstraintDeferrabilitySpec,Maybe ConstraintTimeSpec)

type ConstraintDeferrabilitySpec = Either (Not,Deferrable) Deferrable

data ConstraintTimeSpec = CnstrTimeSpec (Either Immediate Deferred)
	deriving Show

data Definition = DefList [DefElem]
	deriving Show

data DefElem
	= DefEqual ColLabel DefArg
	| DefLabel ColLabel
	deriving Show

data DefArg
	= DefArgFunc FuncType
	| DefArgOp QualAllOp
	| DefArgNum NumericOnly
	| DefArgS Sconst
	deriving Show

type OpClassItemList = MyList OpClassItem

data OpClassItem
	= OpClassItemOperator Iconst AnyOperator (Maybe OperArgTypes) OptRecheck
	| OpClassItemFunction Iconst FuncName FuncArgs
	| OpClassItemStorage Typename
	deriving Show

type OptDefault = Maybe Default

type OptRecheck = Maybe Recheck

data DropType
	= DTTable
	| DTSequence
	| DTView
	| DTIndex
	| DTType
	| DTDomain
	| DTConversion
	| DTSchema
	deriving Show

data AnyNameList = AnyNameLst [AnyName]
	deriving Show

type AnyName = (ColId,Maybe Attrs)

data Attrs = AttrsCont [AttrName]
	deriving Show

data CommentOn
	= CmtOn CommentType AnyName CommentText
	| CmtOnAggregate FuncName AggrArgType CommentText
	| CmtOnFunction FuncName FuncArgs CommentText
	| CmtOnOperator AnyOperator OperArgTypes CommentText
	| CmtOnConstraint Name AnyName CommentText
	| CmtOnRule Name (Maybe AnyName) CommentText
	| CmtOnTrigger Name AnyName CommentText
	| CmtOnOperatorClass AnyName AccessMethod CommentText
	| CmtOnLarge NumericOnly CommentText
	| CmtOnCast Typename Typename CommentText
	| CmtOnLanguage OptProcedural AnyName CommentText
	deriving Show

type CommentType = OneOf10 Column Database Schema Index Sequence Table Domain Type View Conversion

type CommentText = Either Sconst Null

type FetchDirection = Maybe (OneOf10 Next Prior First Last (Absolute,SignedIconst) (Relative,SignedIconst) SignedIconst All (Forward,Maybe (Either SignedIconst All)) (Backward,Maybe (Either SignedIconst All)))

type FromIn = Either From In

data Privileges = Priv (Either PrivilegeList (Maybe PrivilegesString))
	deriving Show

type PrivilegeList = MyList Privilege

data Privilege
	= PrivSelect
	| PrivReferences
	| PrivCreate
	| PrivColId ColId
	deriving Show

data PrivilegeTarget
	= PrivName QualifiedNameList
	| PrivTable QualifiedNameList
	| PrivFunction FunctionWithArgTypesList
	| PrivDatabase NameList
	| PrivLanguage NameList
	| PrivSchema NameList
	| PrivTablespace NameList
	deriving Show

type GranteeList = MyList Grantee

type Grantee = (Maybe Group,RoleId)

data OptGrantGrantOption
	= WithGrantOption
	| NoGrant
	deriving Show

data FunctionWithArgTypesList = FuncLst [FunctionWithArgTypes]
	deriving Show

data FunctionWithArgTypes = FuncArgType FuncName FuncArgs
	deriving Show

data OptGrantAdminOption
	= WithAdminOption
	| NoGrantAdmin
	deriving Show

data OptGrantedBy
	= GrantedBy RoleId
	| NoGrantedBy
	deriving Show

type IndexOptUnique = Maybe Unique

type AccessMethodClause = Maybe (Using,AccessMethod)

data IndexParams = IndexParamsConst [IndexElem]
	deriving Show

data IndexElem
	= IElem ColId OptClass
	| IFuncElem FuncExpr OptClass
	| IExprElem AExpr OptClass
	deriving Show

type OptClass = Maybe ((Maybe Using),AnyName)

data OptOrReplace
	= OrReplace
	| NoOrReplace
	deriving Show

data FuncArgs = FArgs (Maybe FuncArgsList)
	deriving Show

data FuncArgsList = FuncArgsLst [FuncArg]
	deriving Show

type FuncArg = (Maybe ArgClass,Maybe ParamName,Maybe FuncType)

type ArgClass = (Maybe In,Maybe Out)

type ParamName = FunctionName

type FuncReturn = FuncType

data FuncType
	= FuncTypename Typename
	| FuncAttrs TypeName Attrs
	| FuncSetOf TypeName Attrs
	deriving Show

type CreateFuncOptList = [CreateFuncOptItem]

data CommonFuncOptItem
	= CalledOnNullInput
	| ReturnsNullOnNummInput
	| Strict
	| Immutable
	| Stable
	| Volatile
	| ExternalSecurityDefiner
	| ExternalSecurityInvoker
	| SecurityDefiner
	| SecurityInvoker
	deriving Show

data CreateFuncOptItem
	= FuncItemAs FuncAs
	| FuncItemLanguage ColIdOrSconst
	| FuncItemCommon CommonFuncOptItem
	deriving Show

data FuncAs
	= SingleFuncAs Sconst
	| BinFuncAs Sconst Sconst
		deriving Show

type OptDefinition = Maybe (With,Definition)

type AlterFuncOptList = [CommonFuncOptItem]

type OptRestrict = Maybe Restrict

data Star = STAR
	deriving Show

type AggrArgType = Either Typename Star

data OperArgTypes
	= SingleArg Typename
	| BinArg Typename Typename
	| LeftUnaryArg Typename
	| RightUnaryArg Typename
	deriving Show

data AnyOperator
	= AnyOpAll AllOp
	| AnyOpCol ColId AnyOperator
	deriving Show

data CastContext
	= AsImplicit
	| AsAssignment
	| NoCastContext
	deriving Show

type ReindexType = Either Index Table

type OptForce = Maybe Force

data RenameStatement
	= RenAggregate FuncName AggrArgType Name
	| RenConversion AnyName Name
	| RenDatabase DatabaseName DatabaseName
	| RenFunction FuncName FuncArgs Name
	| RenGroup RoleId RoleId
	| RenLanguage Name Name
	| RenOperator AnyName AccessMethod Name
	| RenSchema Name Name
	| RenTable RelationExpr Name
	| RenIndex RelationExpr Name
	| RenTableRename RelationExpr OptColumn Name Name
	| RenTrigger Name RelationExpr Name
	| RenRole RoleId RoleId
	| RenUser RoleId RoleId
	| RenTablespace Name Name
	deriving Show

type OptColumn = Maybe Column

data AlterObjectSchemaStatement
	= AltAggregate FuncName AggrArgType Name
	| AltDomain AnyName Name
	| AltFunction FuncName FuncArgs Name
	| AltSequence RelationExpr Name
	| AltTable RelationExpr Name
	| AltType AnyName Name
	deriving Show

data AlterOwnerStatement
	= AltOwnerAggregate FuncName AggrArgType RoleId
	| AltOwnerConversion AnyName RoleId
	| AltOwnerDatabase DatabaseName RoleId
	| AltOwnerDomain AnyName RoleId
	| AltOwnerFunction FuncName FuncArgs RoleId
	| AltOwnerOperator AnyOperator OperArgTypes RoleId
	| AltOwnerOperatorClass AnyName AccessMethod RoleId
	| AltOwnerSchema Name RoleId
	| AltOwnerType AnyName RoleId
	| AltOwnerTablespace Name RoleId
	deriving Show

data RuleActionList
	= RuleActNothing
	| RuleActStmt Statement
	| RuleActMulti RuleActionMulti
	deriving Show

type RuleActionMulti = MyList RuleActionStmtOrEmpty

type RuleActionStmt = Statement


type RuleActionStmtOrEmpty = Maybe RuleActionStmt

type Event = OneOf4 Select Update Delete Insert

type OptInstead = Maybe (Either Instead Also)

data TransactionStatement
	= TransAbort OptTransaction
	| TransBegin OptTransaction TransactionModeListOrEmpty
	| TransStart TransactionModeListOrEmpty
	| TransCommit OptTransaction
	| TransEnd OptTransaction
	| TransRollback OptTransaction
	| TransSavepoint ColId
	| TransRelease (Maybe Savepoint) ColId
	| TransRollbackSave OptTransaction (Maybe Savepoint) ColId
	| TransPrepare Sconst
	| TransCommitPreparred Sconst
	| TransRollbackPrepared Sconst
	deriving Show

type OptTransaction = Maybe (Either Work Transaction)

data TransactionModeItem
	= TransactionItemIsolationLevel IsoLevel
	| TransactionItemReadOnly
	| TransactionItemReadWrite
	deriving Show

type TransactionModeList = [TransactionModeItem]

type TransactionModeListOrEmpty = Maybe TransactionModeList

type CreatedbOptList = [CreatedbOptItem]

data CreatedbOptItem
	= DBItemTablespace OptEqual (Either Name Default)
	| DBItemLocation OptEqual (Either Sconst Default)
	| DBItemTemplate OptEqual (Either Name Default)
	| DBItemEncoding OptEqual (OneOf3 Sconst Iconst Default)
	| DBItemConnection OptEqual SignedIconst
	| DbItemOwner OptEqual (Either Name Default)
	deriving Show

type OptEqual = Maybe Equal


type AlterdbOptList = [AlterdbOptItem]

data AlterdbOptItem = AlterdbItemConst OptEqual SignedIconst
	deriving Show

type OptAs = Maybe As

type AnalyzeKeyword = Either Analyse Analyze

type OptVerbose = Maybe Verbose

type OptFull = Maybe Full

type OptFreeze = Maybe Freeze

data OptNameList = OptNameLst (Maybe NameList)
	deriving Show

type ExplainableStmt = Statement

type OptAnalyze = Maybe AnalyzeKeyword

data PrepTypeClause = PrepTypeClauseConst (Maybe PrepTypeList)
	deriving Show

type PrepTypeList = MyList Typename

type PreparableStmt = Statement

data ExecuteStatement
	= ExecStmtExecute Name ExecuteParamClause
	| ExecStmtCreate OptTemp QualifiedName OptCreateAs Name ExecuteParamClause
	deriving Show

data ExecuteParamClause = ExecClause (Maybe ExprList)
	deriving Show

data InsertRest
	= InsValues InsertTargetList
	| InsDefaultValues
	| InsSelect Statement
	| InsColValues InsertColumnList InsertTargetList
	| InsCol InsertColumnList Statement
	deriving Show

type InsertColumnList = MyList InsertColumnItem

data InsertColumnItem = InsColItem ColId OptIndirection
	deriving Show

data UsingClause = UseClause (Maybe FromList)
	deriving Show

data OptLock = OptLck (Maybe LockType)
	deriving Show

data LockType
	= LckAccessShare
	| LckRowShare
	| LckRowExclusive
	| LckShareUptateExclusive
	| LckShare
	| LckShareRowExclusive
	| LckExclusive
	| LckAccessExclusive
	deriving Show

type OptNowait = Maybe Nowait

data CursorOptions = CursorOpt (Maybe (CursorOptions,OneOf4 (No,Scroll) Scroll Binary Insensitive))
	deriving Show

data OptHold
	= NoHold
	| WithHold
	| WithoutHold
	deriving Show

data SelectWithParens
	= SlctNo SelectNoParens
	| SlctWith SelectWithParens
	deriving Show

data SelectNoParens
	= SlctSimple SimpleSelect
	| SlctBin SelectClause SortClause
	| SlctMulti1 SelectClause OptSortClause ForLockingClause OptSelectLimit
	| SlctMulti2 SelectClause OptSortClause SelectLimit OptForLockingClause
	deriving Show

type SelectClause = Either SimpleSelect SelectWithParens

data SimpleSelect
	= SimpSlct OptDistinct TargetList IntoClause FromClause WhereClause GroupClause HavingClause
	| UnionSlct SelectClause OptAll SelectClause
	| IntersectSlct SelectClause OptAll SelectClause
	| ExceptSlct SelectClause OptAll SelectClause
	deriving Show

type IntoClause = Maybe (Into,OptTempTableName)

data OptTempTableName
	= TNTemporary OptTable QualifiedName
	| TNTemp OptTable QualifiedName
	| TNLocalTemporary OptTable QualifiedName
	| TNLocalTemp OptTable QualifiedName
	| TNGlobalTemporary OptTable QualifiedName
	| TNGlobalTemp OptTable QualifiedName
	| TNTable QualifiedName
	| TN QualifiedName
	deriving Show

type OptTable = Maybe Table

type OptAll = Maybe (Either All Distinct)

data OptDistinct
	= DisctDistinct
	| DisctDistinctOn ExprList
	| DisctAll
	| NoDisct
	deriving Show

type OptSortClause = Maybe SortClause

data SortClause = SortCl SortByList
	deriving Show

type SortByList = MyList SortBy

data SortBy
	= SortByUsing AExpr QualAllOp
	| SortByAsc AExpr
	| SortByDesc AExpr
	| SortByExp AExpr
	deriving Show

data SelectLimit
	= SlctLimitOffset SelectLimitValue SelectOffsetValue
	| SlctLimit SelectLimitValue
	| SlctOffset SelectOffsetValue
	deriving Show

type OptSelectLimit = Maybe SelectLimit

type SelectLimitValue = Either AExpr All

type SelectOffsetValue = AExpr

data GroupClause = GrpClause (Maybe ExprList)
	deriving Show

data HavingClause = HavClause (Maybe AExpr)
	deriving Show

data ForLockingClause
	= ForLckUpdate LockedRelsList OptNowait
	| ForLckShare LockedRelsList OptNowait
	| ForLckReadOnly
	deriving Show

type OptForLockingClause = Maybe ForLockingClause

data LockedRelsList = LockedRelsLst (Maybe NameList)
	deriving Show

data FromClause = FromClauseConst (Maybe FromList)
	deriving Show

data MyList a = MyLst [a]
	deriving Show

type FromList = MyList TableRef

data TableRef
	= TableRefExpr RelationExpr (Maybe AliasClause)
	| TableRefFunc FuncTable (Maybe AliasClause)
	| TableRefFunc2 FuncTable (OneOf3 As ColId (As,ColId)) TableFuncElementList
	| TableRefSelect SelectWithParens (Maybe AliasClause)
	| TableRefJoin JoinedTable (Maybe AliasClause)
	deriving Show

data JoinedTable
	= JoinedTableJoined JoinedTable
	| JoinedTableCrossJoin TableRef TableRef
	| JoinedTableUnionjoin TableRef TableRef
	| JoinedTableJoin TableRef (Maybe JoinType) TableRef JoinQual
	| JoinedTableNatural TableRef (Maybe JoinType) TableRef
	deriving Show

data AliasClause = AliasClauseConst (Maybe As) ColId (Maybe NameList)
	deriving Show

type JoinType = Either ((OneOf3 Full LeftString RightString),JoinOuter) Inner

type JoinOuter = Maybe Outer

data JoinQual = JoinQualConst (Either NameList AExpr)
	deriving Show

data RelationExpr = RelationExprConst (Maybe Only) QualifiedName (Maybe Star)
	deriving Show

type FuncTable = FuncExpr

data WhereClause = WhereClauseConst (Maybe AExpr)
	deriving Show

type TableFuncElementList = MyList TableFuncElement

type TableFuncElement = (ColId,Typename)

data Typename
	= TypenameConst (Maybe Setof) SimpleTypename (Either OptArrayBounds Iconst)
	deriving Show

data OptArrayBounds
	= SingleArray OptArrayBounds
	| IconstArray OptArrayBounds Iconst
	| EmptyArray
	deriving Show

data SimpleTypename
	= SimpTypeGeneric GenericType
	| SimpTypeNum Numeric
	| SimpTypeBit Bit
	| SimpTypeChar Character
	| SimpTypeDate ConstDatetime
	| SimpTypeInterval ConstInterval (Maybe Iconst) OptInterval
	| SimpType TypeName Attrs
	deriving Show

data ConstTypename
	= ConstTypeGeneric GenericType
	| ConstTypeNum Numeric
	| ConstTypeBit ConstBit
	| ConstTypeChar ConstCharacter
	| ConstTypeDate ConstDatetime
	deriving Show

type GenericType = TypeName

data Numeric
	= NumInt
	| NumInteger
	| NumSmallInt
	| NumBigInt
	| NumReal
	| NumFloat OptFloat
	| NumDouble
	| NumDecimal OptDecimal
	| NumNumeric OptNumeric
	| NumBool
	deriving Show

data OptFloat = OptFloatConst (Maybe Iconst)
	deriving Show

data OptNumeric = OptNumericConst (Maybe (Iconst,Maybe Iconst))
	deriving Show

type OptDecimal = OptNumeric

type Bit = Either BitWithLength BitWithoutLength

type ConstBit = Bit

type BitWithLength = BitDefinition

type BitWithoutLength = BitDefinition

data BitDefinition = BitDef OptVarying (Maybe Iconst)
	deriving Show

type Character = CharDefinition

type CharacterWithLength = CharDefinition

type CharacterWithoutLength = CharDefinition

type ConstCharacter = Character

data CharDefinition = CharDef Character2 (Maybe Iconst) OptCharset
	deriving Show

data Character2
	= Char2Character OptVarying
	| Char2VarChar
	| Char2National OptVarying
	deriving Show

type OptVarying = Maybe Varying

data OptCharset = OptCharConst (Maybe ColId)
	deriving Show

data ConstDatetime = ConstTimestamp (Maybe Iconst) OptTimezone
	deriving Show

type ConstInterval = Interval

data OptTimezone
	= WithTimeZone
	| WithoutTimeZone
	| NoTimeZoneDecl
	deriving Show

data OptInterval
	= IntervalYear
	| IntervalMonth
	| IntervalDay
	| IntervalHour
	| IntervalMinute
	| IntervalSecond
	| IntervalYearToMonth
	| IntervalDayToHour
	| IntervalDayToMinute
	| IntervalDayToSecond
	| IntervalHourToMinute
	| IntrevalHourToSecond
	| IntervalMinuteToSecond
	| NoInterval
	deriving Show

data AExpr
	= AExpCExp CExpr
	| AExpTypecast AExpr Typename
	| AExpTZ AExpr AExpr
	| AExpPositive AExpr
	| AExpNegative AExpr
	| AExpMath AExpr MathOp AExpr
	| AExpQual (Maybe AExpr) QualOp AExpr
	| AExpAnd AExpr AExpr
	| AExpOr AExpr AExpr
	| AExpNot AExpr
	| AExpLike AExpr AExpr (Maybe AExpr)
	| AExpNotLike AExpr AExpr (Maybe AExpr)
	| AExpILike AExpr AExpr (Maybe AExpr)
	| AExpNotILike AExpr AExpr (Maybe AExpr)
	| AExpSimilar AExpr AExpr (Maybe AExpr)
	| AExpNotSimilar AExpr AExpr (Maybe AExpr)
	| AExpIsNull AExpr
	| AExpNotNull AExpr
	| AExpIsNotNull AExpr
	| AExpOverlaps Row Row
	| AExpIsTrue AExpr
	| AExpIsNotTrue AExpr
	| AExpIsFalse AExpr
	| AExpIsNotFalse AExpr
	| AExpIsUnknown AExpr
	| AExpIsNotUnknown AExpr
	| AExpIsDistinct AExpr AExpr
	| AExpIsOf AExpr TypeList
	| AExpIsNotOf AExpr TypeList
	| AExpBetween AExpr OptAsymmetric BExpr BExpr
	| AExpNotBetween AExpr OptAsymmetric BExpr BExpr
	| AExpBetweenSymmetric AExpr BExpr BExpr
	| AExpNotBetweenSymmetric AExpr BExpr BExpr
	| AExpIn AExpr InExpr
	| AExprNotIn AExpr InExpr
	| AExpSub AExpr SubQueryOp SubType (Either SelectWithParens AExpr)
	| AExpUnique SelectWithParens
	deriving Show
	
data BExpr
	= BExpCExp CExpr
	| BExpTypecast BExpr Typename
	| BExpPositive BExpr
	| BExpNegative BExpr
	| BExpMath BExpr MathOp BExpr
	| BExpQual (Maybe BExpr) QualOp BExpr
	| BExpIsDistinct BExpr BExpr
	| BExpIsOf BExpr TypeList
	| BExpIsNotOf BExpr TypeList
	deriving Show

data CExpr
	= CExpCol Columnref
	| CExpAExpC AExprConst
	| CExpParam OptIndirection
	| CExpAExp AExpr OptIndirection
	| CExpCase CaseExpr
	| CExpFunc FuncExpr
	| CExpSelect SelectWithParens
	| CExpExists SelectWithParens
	| CExpArray (Either SelectWithParens ArrayExpr)
	| CExpRow Row
	deriving Show

data FuncExpr
	= FuncExpFunc FuncName (Maybe (OneOf4 ExprList ExprList ExprList Star))
	| FuncExpCurrentDate
	| FuncExpCurrentTime (Maybe Iconst)
	| FuncExpCurrentTimestamp (Maybe Iconst)
	| FuncExpLocalTime (Maybe Iconst)
	| FuncExpLocalTimestamp (Maybe Iconst)
	| FuncExpCurrentRole
	| FuncExpCurrentUser
	| FuncExpSessionUser
	| FuncExpUser
	| FuncExpCast AExpr Typename
	| FuncExpExtract ExtractList
	| FuncExpOverlay OverlayList
	| FuncExpPosition PositionList
	| FuncExpSubstring SubstrList
	| FuncExpTreat AExpr Typename
	| FuncExpTrim (OneOf4 TrimList TrimList TrimList TrimList)
	| FuncExpConvert (Either (AExpr,AnyName) ExprList)
	| FuncExpNullif AExpr AExpr
	| FuncExpCoalesce ExprList
	| FuncExpGreatest ExprList
	| FuncExpLeast ExprList
	deriving Show

data RowString = ROW
	deriving Show

data Row
	= RowExpr ExprList
	| RowSingle
	| RowExprs ExprList AExpr
	deriving Show

type SubType = OneOf3 Any Some All

type AllOp = Either Op MathOp

type MathOp = OneOf9 Plus Minus Star Slash Percentage Hat Smaller Greater Equal

data Plus = PLUS
	deriving Show

data Minus = MINUS
	deriving Show

data Slash = SLASH
	deriving Show

data Percentage = PERCENTAGE
	deriving Show

data Hat = HAT
	deriving Show

data Smaller = SMALLER
	deriving Show

data Greater = GREATER
	deriving Show

data Equal = EQUAL
	deriving Show

data QualOp = QualOper (Either Op AnyOperator)
	deriving Show

data QualAllOp = QualAllOper (Either AllOp AnyOperator)
	deriving Show

data SubQueryOp
	= SubQAll AllOp
	| SubQOp AnyOperator
	| SubQLike
	| SubQNotLike
	| SubQIlike
	| SubQNotIlike
	deriving Show

type ExprList = MyList AExpr

data ExtractList
	= ExtractLst ExtractArg AExpr
	| ExtractEmpty
	deriving Show

type TypeList = MyList Typename

type ArrayExprList = MyList ArrayExpr

data ArrayExpr = ArrayExp (Either ExprList ArrayExprList)
	deriving Show

type ExtractArg = OneOf8 Ident Year Month Day Hour Minute Second Sconst

data OverlayList = OverLayLst AExpr OverlayPlacing SubstrFrom (Maybe SubstrFor)
	deriving Show

data OverlayPlacing = OverlayPlace AExpr
	deriving Show

data PositionList
	= PosLst BExpr BExpr
	| PosLstEmpty
	deriving Show

data SubstrList
	= SStrFromFor AExpr SubstrFrom SubstrFor
	| SStrFrom AExpr SubstrFrom
	| SStrFor AExpr SubstrFor
	| SStrLst ExprList
	| SStrEmpty
	deriving Show

data SubstrFrom = SubstringFrom AExpr
	deriving Show

data SubstrFor = SubstringFor AExpr
	deriving Show

type TrimList = (Maybe (Maybe AExpr,From),ExprList)

data InExpr
	= InExpSelect SelectWithParens
	| InExpLst ExprList
	deriving Show

data CaseExpr = CaseExp CaseArg WhenClauseList CaseDefault
	deriving Show

type WhenClauseList = [WhenClause]

data WhenClause = WhenClauseConst AExpr AExpr
	deriving Show

data CaseDefault = CaseDef (Maybe AExpr)
	deriving Show

type CaseArg = Maybe AExpr

data Columnref
	= ColRefRel RelationName
	| ColRefRelInd RelationName Indirection
	deriving Show

data IndirectionEl
	= IndAttr AttrName
	| IndStar
	| IndExp AExpr
	| IndBinExp AExpr AExpr
	deriving Show

type Indirection = [IndirectionEl]

type OptIndirection = [IndirectionEl]

type OptAsymmetric = Maybe Asymmetric

type TargetList = MyList TargetEl

data TargetEl
	= TargetElAs AExpr ColLabel
	| TargetElExp AExpr
	| TargetElStar
	deriving Show

type UpdateTargetList = MyList UpdateTargetEl

data UpdateTargetEl = UpdtTargetEl ColId OptIndirection (Either AExpr Default)
	deriving Show

type InsertTargetList = MyList InsertTargetEl

type InsertTargetEl = Either AExpr Default

data Default = DEFAULT
	deriving Show

type RelationName = Either SpecialRuleRelation ColId


type QualifiedNameList = MyList QualifiedName

data QualifiedName
	= QualNameRel RelationName
	| QualNameRelInd RelationName Indirection
	deriving Show

type NameList = MyList Name

type Name = ColId

type DatabaseName = ColId

type AccessMethod = ColId

type AttrName = ColLabel

type IndexName = ColId

type FileName = Sconst

data FuncName
	= FuncNameFunc FunctionName
	| FuncNameRelInd RelationName Indirection
	deriving Show

data AExprConst
	= AExpCI Iconst
	| AExpCF Fconst
	| AExpCS Sconst
	| AExpCB Bconst
	| AExpCX Xconst
	| AExpCType ConstTypename Sconst
	| AExpCInt ConstInterval (Maybe Iconst) Sconst OptInterval
	| AExpCTrue
	| AExpCFalse
	| AExpCNull
	deriving Show

type RoleId = ColId

type SignedIconst = (Maybe Minus,Iconst)

type ColId = OneOf3 Ident UnreservedKeyword ColNameKeyword

type TypeName = Either Ident UnreservedKeyword

type FunctionName = OneOf3 Ident UnreservedKeyword FuncNameKeyword

type ColLabel = OneOf5 Ident UnreservedKeyword ColNameKeyword FuncNameKeyword ReservedKeyword

data UnreservedKeyword
	= UnreservedKeywordABORT 
	| UnreservedKeywordABSOLUTE 
	| UnreservedKeywordACCESS 
	| UnreservedKeywordACTION 
	| UnreservedKeywordADD 
	| UnreservedKeywordADMIN 
	| UnreservedKeywordAFTER 
	| UnreservedKeywordAGGREGATE 
	| UnreservedKeywordALSO 
	| UnreservedKeywordALTER 
	| UnreservedKeywordASSERTION 
	| UnreservedKeywordASSIGNMENT 
	| UnreservedKeywordAT 
	| UnreservedKeywordBACKWARD 
	| UnreservedKeywordBEFORE 
	| UnreservedKeywordBEGIN 
	| UnreservedKeywordBY 
	| UnreservedKeywordCACHE 
	| UnreservedKeywordCALLED 
	| UnreservedKeywordCASCADE 
	| UnreservedKeywordCHAIN 
	| UnreservedKeywordCHARACTERISTICS 
	| UnreservedKeywordCHECKPOINT 
	| UnreservedKeywordCLASS 
	| UnreservedKeywordCLOSE 
	| UnreservedKeywordCLUSTER 
	| UnreservedKeywordCOMMENT 
	| UnreservedKeywordCOMMIT 
	| UnreservedKeywordCOMMITTED 
	| UnreservedKeywordCONNECTION 
	| UnreservedKeywordCONSTRAINTS 
	| UnreservedKeywordCONVERSION 
	| UnreservedKeywordCOPY 
	| UnreservedKeywordCREATEDB 
	| UnreservedKeywordCREATEROLE 
	| UnreservedKeywordCREATEUSER 
	| UnreservedKeywordCSV 
	| UnreservedKeywordCURSOR 
	| UnreservedKeywordCYCLE 
	| UnreservedKeywordDATABASE 
	| UnreservedKeywordDAY 
	| UnreservedKeywordDEALLOCATE 
	| UnreservedKeywordDECLARE 
	| UnreservedKeywordDEFAULTS 
	| UnreservedKeywordDEFERRED 
	| UnreservedKeywordDEFINER 
	| UnreservedKeywordDELETE 
	| UnreservedKeywordDELIMITER 
	| UnreservedKeywordDELIMITERS 
	| UnreservedKeywordDISABLE 
	| UnreservedKeywordDOMAIN 
	| UnreservedKeywordDOUBLE 
	| UnreservedKeywordDROP 
	| UnreservedKeywordEACH 
	| UnreservedKeywordENABLE 
	| UnreservedKeywordENCODING 
	| UnreservedKeywordENCRYPTED 
	| UnreservedKeywordESCAPE 
	| UnreservedKeywordEXCLUDING 
	| UnreservedKeywordEXCLUSIVE 
	| UnreservedKeywordEXECUTE 
	| UnreservedKeywordEXPLAIN 
	| UnreservedKeywordEXTERNAL 
	| UnreservedKeywordFETCH 
	| UnreservedKeywordFIRST 
	| UnreservedKeywordFORCE 
	| UnreservedKeywordFORWARD 
	| UnreservedKeywordFUNCTION 
	| UnreservedKeywordGLOBAL 
	| UnreservedKeywordGRANTED 
	| UnreservedKeywordHANDLER 
	| UnreservedKeywordHEADER 
	| UnreservedKeywordHOLD 
	| UnreservedKeywordHOUR 
	| UnreservedKeywordIMMEDIATE 
	| UnreservedKeywordIMMUTABLE 
	| UnreservedKeywordIMPLICIT 
	| UnreservedKeywordINCLUDING 
	| UnreservedKeywordINCREMENT 
	| UnreservedKeywordINDEX 
	| UnreservedKeywordINHERIT 
	| UnreservedKeywordINHERITS 
	| UnreservedKeywordINPUT 
	| UnreservedKeywordINSENSITIVE 
	| UnreservedKeywordINSERT 
	| UnreservedKeywordINSTEAD 
	| UnreservedKeywordINVOKER 
	| UnreservedKeywordISOLATION 
	| UnreservedKeywordKEY
	| UnreservedKeywordLANCOMPILER 
	| UnreservedKeywordLANGUAGE 
	| UnreservedKeywordLARGE 
	| UnreservedKeywordLAST 
	| UnreservedKeywordLEVEL 
	| UnreservedKeywordLISTEN 
	| UnreservedKeywordLOAD 
	| UnreservedKeywordLOCAL 
	| UnreservedKeywordLOCATION 
	| UnreservedKeywordLOCK 
	| UnreservedKeywordLOGIN 
	| UnreservedKeywordMATCH 
	| UnreservedKeywordMAXVALUE 
	| UnreservedKeywordMINUTE 
	| UnreservedKeywordMINVALUE 
	| UnreservedKeywordMODE 
	| UnreservedKeywordMONTH 
	| UnreservedKeywordMOVE 
	| UnreservedKeywordNAMES 
	| UnreservedKeywordNEXT 
	| UnreservedKeywordNO 
	| UnreservedKeywordNOCREATEDB 
	| UnreservedKeywordNOCREATEROLE 
	| UnreservedKeywordNOCREATEUSER 
	| UnreservedKeywordNOINHERIT 
	| UnreservedKeywordNOLOGIN 
	| UnreservedKeywordNOSUPERUSER 
	| UnreservedKeywordNOTHING 
	| UnreservedKeywordNOTIFY 
	| UnreservedKeywordNOWAIT 
	| UnreservedKeywordOBJECT 
	| UnreservedKeywordOF 
	| UnreservedKeywordOIDS 
	| UnreservedKeywordOPERATOR 
	| UnreservedKeywordOPTION 
	| UnreservedKeywordOWNER 
	| UnreservedKeywordPARTIAL 
	| UnreservedKeywordPASSWORD 
	| UnreservedKeywordPREPARE 
	| UnreservedKeywordPREPARED 
	| UnreservedKeywordPRESERVE 
	| UnreservedKeywordPRIOR 
	| UnreservedKeywordPRIVILEGES 
	| UnreservedKeywordPROCEDURAL 
	| UnreservedKeywordPROCEDURE 
	| UnreservedKeywordQUOTE 
	| UnreservedKeywordREAD 
	| UnreservedKeywordRECHECK 
	| UnreservedKeywordREINDEX 
	| UnreservedKeywordRELATIVE 
	| UnreservedKeywordRELEASE 
	| UnreservedKeywordRENAME 
	| UnreservedKeywordREPEATABLE 
	| UnreservedKeywordREPLACE 
	| UnreservedKeywordRESET 
	| UnreservedKeywordRESTART 
	| UnreservedKeywordRESTRICT 
	| UnreservedKeywordRETURNS 
	| UnreservedKeywordREVOKE 
	| UnreservedKeywordROLE 
	| UnreservedKeywordROLLBACK 
	| UnreservedKeywordROWS 
	| UnreservedKeywordRULE 
	| UnreservedKeywordSAVEPOINT 
	| UnreservedKeywordSCHEMA 
	| UnreservedKeywordSCROLL 
	| UnreservedKeywordSECOND 
	| UnreservedKeywordSECURITY 
	| UnreservedKeywordSEQUENCE 
	| UnreservedKeywordSERIALIZABLE 
	| UnreservedKeywordSESSION 
	| UnreservedKeywordSET 
	| UnreservedKeywordSHARE 
	| UnreservedKeywordSHOW 
	| UnreservedKeywordSIMPLE 
	| UnreservedKeywordSTABLE 
	| UnreservedKeywordSTART 
	| UnreservedKeywordSTATEMENT 
	| UnreservedKeywordSTATISTICS 
	| UnreservedKeywordSTDIN 
	| UnreservedKeywordSTDOUT 
	| UnreservedKeywordSTORAGE 
	| UnreservedKeywordSUPERUSER 
	| UnreservedKeywordSYSID 
	| UnreservedKeywordSYSTEM 
	| UnreservedKeywordSTRICT 
	| UnreservedKeywordTABLESPACE 
	| UnreservedKeywordTEMP 
	| UnreservedKeywordTEMPLATE 
	| UnreservedKeywordTEMPORARY 
	| UnreservedKeywordTOAST 
	| UnreservedKeywordTRANSACTION 
	| UnreservedKeywordTRIGGER 
	| UnreservedKeywordTRUNCATE 
	| UnreservedKeywordTRUSTED 
	| UnreservedKeywordTYPE 
	| UnreservedKeywordUNCOMMITTED 
	| UnreservedKeywordUNENCRYPTED 
	| UnreservedKeywordUNKNOWN
	| UnreservedKeywordUNLISTEN
	| UnreservedKeywordUNTIL
	| UnreservedKeywordUPDATE
	| UnreservedKeywordVACUUM
	| UnreservedKeywordVALID
	| UnreservedKeywordVALIDATOR
	| UnreservedKeywordVALUES
	| UnreservedKeywordVARYING
	| UnreservedKeywordVIEW
	| UnreservedKeywordVOLATILE
	| UnreservedKeywordWITH
	| UnreservedKeywordWITHOUT
	| UnreservedKeywordWORK
	| UnreservedKeywordWRITE
	| UnreservedKeywordYEAR
	| UnreservedKeywordZONE
	deriving Show

data ColNameKeyword
	= ColNameKeywordBIGINT 
	| ColNameKeywordBIT 
	| ColNameKeywordBOOLEAN
	| ColNameKeywordCHAR
	| ColNameKeywordCHARACTER 
	| ColNameKeywordCOALESCE 
	| ColNameKeywordCONVERT 
	| ColNameKeywordDEC 
	| ColNameKeywordDECIMAL
	| ColNameKeywordEXISTS 
	| ColNameKeywordEXTRACT 
	| ColNameKeywordFLOAT
	| ColNameKeywordGREATEST 
	| ColNameKeywordINOUT 
	| ColNameKeywordINT
	| ColNameKeywordINTEGER 
	| ColNameKeywordINTERVAL 
	| ColNameKeywordLEAST 
	| ColNameKeywordNATIONAL 
	| ColNameKeywordNCHAR 
	| ColNameKeywordNONE 
	| ColNameKeywordNULLIF 
	| ColNameKeywordNUMERIC 
	| ColNameKeywordOUT
	| ColNameKeywordOVERLAY 
	| ColNameKeywordPOSITION 
	| ColNameKeywordPRECISION 
	| ColNameKeywordREAL 
	| ColNameKeywordROW 
	| ColNameKeywordSETOF 
	| ColNameKeywordSMALLINT 
	| ColNameKeywordSUBSTRING 
	| ColNameKeywordTIME 
	| ColNameKeywordTIMESTAMP 
	| ColNameKeywordTREAT 
	| ColNameKeywordTRIM 
	| ColNameKeywordVARCHAR 
	deriving Show

data FuncNameKeyword
	= FuncNameAUTHORIZATION 
	| FuncNameBETWEEN 
	| FuncNameBINARY 
	| FuncNameCROSS 
	| FuncNameFREEZE 
	| FuncNameFULL 
	| FuncNameILIKE 
	| FuncNameINNER
	| FuncNameIS 
	| FuncNameISNULL 
	| FuncNameJOIN 
	| FuncNameLEFT 
	| FuncNameLIKE 
	| FuncNameNATURAL 
	| FuncNameNOTNULL 
	| FuncNameOUTER
	| FuncNameOVERLAPS 
	| FuncNameRIGHT 
	| FuncNameSIMILAR 
	| FuncNameVERBOSE 
	deriving Show

data ReservedKeyword
	= ReservedKeywordALL
	| ReservedKeywordANALYSE
	| ReservedKeywordANALYZE
	| ReservedKeywordAND
	| ReservedKeywordANY
	| ReservedKeywordARRAY
	| ReservedKeywordAS
	| ReservedKeywordASC
	| ReservedKeywordASYMMETRIC
	| ReservedKeywordBOTH
	| ReservedKeywordCASE
	| ReservedKeywordCAST
	| ReservedKeywordCHECK
	| ReservedKeywordCOLLATE
	| ReservedKeywordCOLUMN
	| ReservedKeywordCONSTRAINT
	| ReservedKeywordCREATE
	| ReservedKeywordCURRENT_DATE
	| ReservedKeywordCURRENT_ROLE
	| ReservedKeywordCURRENT_TIME
	| ReservedKeywordCURRENT_TIMESTAMP
	| ReservedKeywordCURRENT_USER
	| ReservedKeywordDEFAULT
	| ReservedKeywordDEFERRABLE
	| ReservedKeywordDESC
	| ReservedKeywordDISTINCT
	| ReservedKeywordDO
	| ReservedKeywordELSE
	| ReservedKeywordEND
	| ReservedKeywordEXCEPT
	| ReservedKeywordFALSE
	| ReservedKeywordFOR
	| ReservedKeywordFOREIGN
	| ReservedKeywordFROM
	| ReservedKeywordGRANT
	| ReservedKeywordGROUP
	| ReservedKeywordHAVING
	| ReservedKeywordIN
	| ReservedKeywordINITIALLY
	| ReservedKeywordINTERSECT
	| ReservedKeywordINTO
	| ReservedKeywordLEADING
	| ReservedKeywordLIMIT
	| ReservedKeywordLOCALTIME
	| ReservedKeywordLOCALTIMESTAMP
	| ReservedKeywordNEW
	| ReservedKeywordNOT
	| ReservedKeywordNULL
	| ReservedKeywordOFF
	| ReservedKeywordOFFSET
	| ReservedKeywordOLD
	| ReservedKeywordON
	| ReservedKeywordONLY
	| ReservedKeywordOR
	| ReservedKeywordORDER
	| ReservedKeywordPLACING
	| ReservedKeywordPRIMARY
	| ReservedKeywordREFERENCES
	| ReservedKeywordSELECT
	| ReservedKeywordSESSION_USER
	| ReservedKeywordSOME
	| ReservedKeywordSYMMETRIC
	| ReservedKeywordTABLE
	| ReservedKeywordTHEN
	| ReservedKeywordTO
	| ReservedKeywordTRAILING
	| ReservedKeywordTRUE
	| ReservedKeywordUNION
	| ReservedKeywordUNIQUE
	| ReservedKeywordUSER
	| ReservedKeywordUSING
	| ReservedKeywordWHEN
	| ReservedKeywordWHERE
	deriving Show

data Asymmetric = ASYMMETRIC
	deriving Show

data From = FROM
	deriving Show

data Second = SECOND
	deriving Show

data Minute = MINUTE
	deriving Show

data Hour = HOUR
	deriving Show

data Day = DAY
	deriving Show

data Month = MONTH
	deriving Show

data Year = YEAR
	deriving Show

type Op = String

data All = ALL
	deriving Show

data Some = SOME
	deriving Show

data Any = ANY
	deriving Show

data Interval = INTERVAL
	deriving Show

data Varying = VARYING
	deriving Show

data Setof = SETOF
	deriving Show

data Only = ONLY
	deriving Show

data Outer = OUTER
	deriving Show

data Inner = INNER
	deriving Show

data LeftString = LEFT
	deriving Show

data RightString = RIGHT
	deriving Show

data Full = FULL
	deriving Show

data As = AS
	deriving Show

data Replace = REPLACE
	deriving Show

data On = ON
	deriving Show

data For = FOR
	deriving Show

data With = WITH
	deriving Show

data Distinct = DISTINCT
	deriving Show

data Table = TABLE
	deriving Show

data Into = INTO
	deriving Show

data Insensitive = INSENSITIVE
	deriving Show

data Binary = BINARY
	deriving Show

data No = NO
	deriving Show

data Scroll = SCROLL
	deriving Show

data Nowait = NOWAIT
	deriving Show

data Freeze = FREEZE
	deriving Show

data Verbose = VERBOSE
	deriving Show

data Analyse = ANALYSE
	deriving Show

data Analyze = ANALYZE
	deriving Show

data Translation = TRANSLATION
	deriving Show

data Transaction = TRANSACTION
	deriving Show

data Work = WORK
	deriving Show

data Savepoint = SAVEPOINT
	deriving Show

data Also = ALSO
	deriving Show

data Instead = INSTEAD
	deriving Show

data Insert = INSERT
	deriving Show

data Delete = DELETE
	deriving Show

data Update = UPDATE
	deriving Show
	
data Select = SELECT
	deriving Show

data Column = COLUMN
	deriving Show

data Force = FORCE
	deriving Show

data Index = INDEX
	deriving Show

data Restrict = RESTRICT
	deriving Show

data In = IN
	deriving Show

data Out = OUT
	deriving Show

data Using = USING
	deriving Show

data Unique = UNIQUE
	deriving Show

data Group = GROUP
	deriving Show

data PrivilegesString = PRIVILEGES
	deriving Show

data Trusted = TRUSTED
	deriving Show

data Procedural = PROCEDURAL
	deriving Show

data Backward = BACKWARD
	deriving Show

data Forward = FORWARD
	deriving Show

data Relative = RELATIVE
	deriving Show

data Absolute = ABSOLUTE
	deriving Show

data Last = LAST
	deriving Show

data First = FIRST
	deriving Show
	
data Prior = PRIOR
	deriving Show

data Next = NEXT
	deriving Show

data Null = NULL
	deriving Show

data Conversion = CONVERSION
	deriving Show

data View = VIEW
	deriving Show

data Type = TYPE
	deriving Show

data Domain = DOMAIN
	deriving Show

data Sequence = SEQUENCE
	deriving Show

data Schema = SCHEMA
	deriving Show

data Database = DATABASE
	deriving Show

data Recheck = RECHECK
	deriving Show

data Deferred = DEFERRED
	deriving Show

data Deferrable = DEFERRABLE
	deriving Show

data Immediate = IMMEDIATE
	deriving Show

data Not = NOT
	deriving Show

data Each = EACH
	deriving Show

data After = AFTER
	deriving Show

data Before = BEFORE
	deriving Show

type OptBy = Maybe By

data By = BY
	deriving Show

data Constraint = CONSTRAINT
	deriving Show

data Delimiters = DELIMITERS
	deriving Show

data Oids = OIDS
	deriving Show

data Stdin = STDIN
	deriving Show

data Stdout = STDOUT
	deriving Show

data To = TO
	deriving Show

data Cascade = CASCADE
	deriving Show

data User = USER
	deriving Show

data Cluster = CLUSTER
	deriving Show

data Storage = STORAGE
	deriving Show

data Statistics = STATISTICS
	deriving Show

data Set = SET
	deriving Show

data Drop = DROP
	deriving Show

data Session = SESSION
	deriving Show

data Local = LOCAL
	deriving Show

data Authorization = AUTHORIZATION
	deriving Show

data Level = LEVEL
	deriving Show

data Isolation = ISOLATION
	deriving Show

data Zone = ZONE
	deriving Show

data Time = TIME
	deriving Show

data System = SYSTEM
	deriving Show

data Move = MOVE
	deriving Show

data Fetch = FETCH
	deriving Show

data Add = ADD
	deriving Show

type SpecialRuleRelation = Either Old New

data Old = OLD
	deriving Show

data New = NEW
	deriving Show

-- * Functions

-- | returns the list encapsulated in 'MyList'
tkMyList (MyLst x) = x	

-- * Classes

-- ** 'SQLShow'

-- 'SQLShow' class defines if a type is showable in SQL

class SQLShow a where
    sshow :: a -> String

-- 'SQLShow' class instances

instance SQLShow AlterUsing where
	sshow (AlterUse (Just x)) = "USING " ++ sshow x
	sshow (AlterUse Nothing) = ""

instance SQLShow RowString where
	sshow ROW = "ROW"

instance SQLShow StatementString where
	sshow STATEMENT = "STATEMENT"

instance SQLShow Int where
	sshow x = show x

instance SQLShow No where
	sshow NO = "NO"

instance SQLShow OverlayList where
	sshow (OverLayLst x y z mb) = sshow x ++ ' ' : sshow y ++ ' ' : sshow z ++ ' ' : sshow mb

instance SQLShow CharDefinition where
	sshow (CharDef x (Just y) z) = sshow x ++ '(' : sshow y ++ ')' : sshow z
	sshow (CharDef x Nothing z) = sshow x ++ ' ' : sshow z

instance (SQLShow a,SQLShow b,SQLShow c,SQLShow d,SQLShow e,SQLShow f) => SQLShow (OneOf6 a b c d e f) where
	sshow (OneOf6 x) = sshow x
	sshow (TwoOf6 x) = sshow x
	sshow (ThreeOf6 x) = sshow x
	sshow (FourOf6 x) = sshow x
	sshow (FiveOf6 x) = sshow x
	sshow (SixOf6 x) = sshow x

instance (SQLShow a,SQLShow b,SQLShow c,SQLShow d,SQLShow e,SQLShow f,SQLShow g,SQLShow h,SQLShow i,SQLShow j) => SQLShow (OneOf10 a b c d e f g h i j) where
	sshow (OneOf10 x) = sshow x
	sshow (TwoOf10 x) = sshow x
	sshow (ThreeOf10 x) = sshow x
	sshow (FourOf10 x) = sshow x
	sshow (FiveOf10 x) = sshow x
	sshow (SixOf10 x) = sshow x
	sshow (SevenOf10 x) = sshow x
	sshow (EightOf10 x) = sshow x
	sshow (NineOf10 x) = sshow x
	sshow (TenOf10 x) = sshow x

instance (SQLShow a,SQLShow b,SQLShow c,SQLShow d,SQLShow e,SQLShow f,SQLShow g,SQLShow h,SQLShow i) => SQLShow (OneOf9 a b c d e f g h i) where
	sshow (OneOf9 x) = sshow x
	sshow (TwoOf9 x) = sshow x
	sshow (ThreeOf9 x) = sshow x
	sshow (FourOf9 x) = sshow x
	sshow (FiveOf9 x) = sshow x
	sshow (SixOf9 x) = sshow x
	sshow (SevenOf9 x) = sshow x
	sshow (EightOf9 x) = sshow x
	sshow (NineOf9 x) = sshow x

instance (SQLShow a,SQLShow b,SQLShow c,SQLShow d,SQLShow e,SQLShow f,SQLShow g,SQLShow h) => SQLShow (OneOf8 a b c d e f g h) where
	sshow (OneOf8 x) = sshow x
	sshow (TwoOf8 x) = sshow x
	sshow (ThreeOf8 x) = sshow x
	sshow (FourOf8 x) = sshow x
	sshow (FiveOf8 x) = sshow x
	sshow (SixOf8 x) = sshow x
	sshow (SevenOf8 x) = sshow x
	sshow (EightOf8 x) = sshow x

instance (SQLShow a,SQLShow b,SQLShow c,SQLShow d) => SQLShow (OneOf4 a b c d) where
	sshow (OneOf4 x) = sshow x
	sshow (TwoOf4 x) = sshow x
	sshow (ThreeOf4 x) = sshow x
	sshow (FourOf4 x) = sshow x

instance (SQLShow a,SQLShow b,SQLShow c,SQLShow d,SQLShow e) => SQLShow (OneOf5  a b c d e) where
	sshow (OneOf5 x) = sshow x
	sshow (TwoOf5 x) = sshow x
	sshow (ThreeOf5 x) = sshow x
	sshow (FourOf5 x) = sshow x
	sshow (FiveOf5 x) = sshow x

instance SQLShow CursorOptions where
	sshow (CursorOpt mb) = sshow mb

instance SQLShow Add where
	sshow ADD = "ADD"

instance SQLShow Fetch where
	sshow FETCH = "FETCH"

instance SQLShow Move where
	sshow MOVE = "MOVE"

instance SQLShow System where
	sshow SYSTEM = "SYSTEM"

instance SQLShow Time where
	sshow TIME = "TIME"

instance SQLShow Zone where
	sshow ZONE = "ZONE"

instance SQLShow Isolation where
	sshow ISOLATION = " ISOLATION"

instance SQLShow Level where
	sshow LEVEL = "LEVEL"

instance SQLShow Authorization where
	sshow AUTHORIZATION = "AUTHORIZATION"

instance SQLShow Local where
	sshow LOCAL = "LOCAL"

instance SQLShow Session where
	sshow SESSION = "SESSION"

instance SQLShow Drop where
	sshow DROP = "DROP"

instance SQLShow Set where
	sshow SET = "SET"

instance SQLShow Statistics where
	sshow STATISTICS = "STATISTICS"

instance SQLShow Storage where
	sshow STORAGE = "STORAGE"

instance SQLShow Cluster where
	sshow CLUSTER = "CLUSTER"

instance SQLShow User where
	sshow USER = "USER"

instance SQLShow Cascade where
	sshow CASCADE = "CASCADE"

instance SQLShow To where
	sshow TO = "TO"

instance SQLShow Stdout where
	sshow STDOUT = "STDOUT"

instance SQLShow Stdin where
	sshow STDIN = "STDIN"

instance SQLShow Oids where
	sshow OIDS = "OIDS"

instance SQLShow Delimiters where
	sshow DELIMITERS = "DELIMITERS"

instance SQLShow Constraint where
	sshow CONSTRAINT = "CONSTRAINT"

instance SQLShow ColConstraintElem where
	sshow CstrNotNull = "NOT NULL"
	sshow CstrNull = "NULL"
	sshow (CstrUnique x) = "UNIQUE " ++ sshow x
	sshow (CstrPrimaryKey x) = "PRIMARY KEY " ++ sshow x
	sshow (CstrCheck x) = "CHECK (" ++ sshow x ++ ")"
	sshow (CstrDefault x) = "DEFAULT " ++ sshow x
	sshow (CstrReferences x y z w) = "REFERENCES " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z ++ ' ' : sshow w

instance SQLShow By where
	sshow BY = "BY"

instance SQLShow Before where
	sshow BEFORE = "BEFORE"

instance SQLShow After where
	sshow AFTER = "AFTER"

instance SQLShow Each where
	sshow EACH = "EACH"

instance SQLShow Not where
	sshow NOT = "NOT"

instance SQLShow Deferrable where
	sshow DEFERRABLE = "DEFERRABLE"

instance SQLShow Immediate where
	sshow IMMEDIATE = "IMMEDIATE"

instance SQLShow Deferred where
	sshow DEFERRED = "DEFERRED"

instance SQLShow Recheck where
	sshow RECHECK = "RECHECK"

instance SQLShow Database where
	sshow DATABASE = "DATABASE"

instance SQLShow Schema where
	sshow SCHEMA = "SCHEMA"

instance SQLShow Sequence where
	sshow SEQUENCE = "SEQUENCE"

instance SQLShow Domain where
	sshow DOMAIN = "DOMAIN"

instance SQLShow Type where
	sshow TYPE = "TYPE"

instance SQLShow View where
	sshow VIEW = "VIEW"

instance SQLShow Conversion where
	sshow CONVERSION = "CONVERSION"

instance SQLShow Null where
	sshow NULL = "NULL"

instance SQLShow Next where
	sshow NEXT = "NEXT"

instance SQLShow Prior where
	sshow PRIOR = "PRIOR"
	
instance SQLShow First where
	sshow FIRST = "FIRST"

instance SQLShow Last where
	sshow LAST = "LAST"

instance SQLShow Absolute where
	sshow ABSOLUTE = "ABSOLUTE"

instance SQLShow Relative where
	sshow RELATIVE = "RELATIVE"

instance SQLShow Forward where
	sshow FORWARD = "FORWARD"

instance SQLShow Backward where
	sshow BACKWARD = "BACKWARD"

instance SQLShow Procedural where
	sshow PROCEDURAL = "PROCEDURAL"

instance SQLShow Trusted where
	sshow TRUSTED = "TRUSTED"

instance SQLShow PrivilegesString where
	sshow PRIVILEGES = "PRIVILEGES"

instance SQLShow Group where
	sshow GROUP = "GROUP"

instance SQLShow Unique where
	sshow UNIQUE = "UNIQUE"

instance SQLShow Using where
	sshow USING = "USING"

instance SQLShow In where
	sshow IN = "IN"

instance SQLShow Out where
	sshow OUT = "OUT"

instance SQLShow Restrict where
	sshow RESTRICT = "RESTRICT"

instance SQLShow Index where
	sshow INDEX = "INDEX"

instance SQLShow Force where
	sshow FORCE = "FORCE"

instance SQLShow Column where
	sshow COLUMN = "COLUMN"

instance SQLShow Select where
	sshow SELECT = "SELECT"

instance SQLShow Update where
	sshow UPDATE = "UPDATE"

instance SQLShow Delete where
	sshow DELETE = "DELETE"

instance SQLShow Insert where
	sshow INSERT = "INSERT"

instance SQLShow Instead where
	sshow INSTEAD = "INSTEAD"

instance SQLShow Also where
	sshow ALSO = "ALSO"

instance SQLShow Savepoint where
	sshow SAVEPOINT = "SAVEPOINT"

instance SQLShow Work where
	sshow WORK = "WORK"

instance SQLShow Transaction where
	sshow TRANSACTION = "TRANSACTION"

instance SQLShow TransactionModeItem where
	sshow (TransactionItemIsolationLevel x) = "ISOLATION LEVEL " ++ sshow x
	sshow TransactionItemReadOnly = "READ ONLY"
	sshow TransactionItemReadWrite = "READ WRITE"

instance SQLShow Analyze where
	sshow ANALYZE = "ANALYZE"

instance SQLShow Analyse where
	sshow ANALYSE = "ANALYSE"

instance SQLShow Verbose where
	sshow VERBOSE = "VERBOSE"

instance SQLShow Freeze where
	sshow FREEZE = "FREEZE"

instance SQLShow Nowait where
	sshow NOWAIT = "NOWAIT"

instance SQLShow Scroll where
	sshow SCROLL = "SCROLL"

instance SQLShow Binary where
	sshow BINARY = "BINARY"

instance SQLShow Insensitive where
	sshow INSENSITIVE = "INSENSITIVE"

instance SQLShow Into where
	sshow INTO = "INTO"

instance SQLShow Table where
	sshow TABLE = "TABLE"

instance SQLShow Distinct where
	sshow DISTINCT = "DISTINCT"

instance SQLShow As where
	sshow AS = "AS"

instance SQLShow Full where
	sshow FULL = "FULL"

instance SQLShow RightString where
	sshow RIGHT = "RIGHT"

instance SQLShow LeftString where
	sshow LEFT = "LEFT"

instance SQLShow Inner where
	sshow INNER = "INNER"

instance SQLShow Outer where
	sshow OUTER = "OUTER"

instance SQLShow Only where
	sshow ONLY = "ONLY"

instance SQLShow Setof where
	sshow SETOF = "SETOF"

instance SQLShow Varying where
	sshow VARYING = "VARYING"

instance SQLShow Interval where
	sshow INTERVAL = "INTERVAL"

instance SQLShow Any where
	sshow ANY = "ANY"

instance SQLShow Some where
	sshow SOME = "SOME"

instance SQLShow All where
	sshow ALL = "ALL"

instance SQLShow Year where
	sshow YEAR = "YEAR"

instance SQLShow Month where
	sshow MONTH = "MONTH"

instance SQLShow Day where
	sshow DAY = "DAY"

instance SQLShow Hour where
	sshow HOUR = "HOUR"

instance SQLShow Minute where
	sshow MINUTE = "MINUTE"

instance SQLShow Second where
	sshow SECOND = "SECOND"

instance SQLShow From where
	sshow FROM = "FROM"

instance SQLShow Asymmetric where
	sshow ASYMMETRIC = "ASYMMETRIC"

instance SQLShow Default where
	sshow DEFAULT = "DEFAULT"

instance SQLShow ReservedKeyword where
	sshow ReservedKeywordALL = "ALL"
	sshow ReservedKeywordANALYSE = "ANALYSE"
	sshow ReservedKeywordANALYZE = "ANALYZE"
	sshow ReservedKeywordAND = "AND"
	sshow ReservedKeywordANY = "ANY"
	sshow ReservedKeywordARRAY = "ARRAY"
	sshow ReservedKeywordAS = "AS"
	sshow ReservedKeywordASC = "ASC"
	sshow ReservedKeywordASYMMETRIC = "ASYMMETRIC"
	sshow ReservedKeywordBOTH = "BOTH"
	sshow ReservedKeywordCASE = "CASE"
	sshow ReservedKeywordCAST = "CAST"
	sshow ReservedKeywordCHECK = "CHECK"
	sshow ReservedKeywordCOLLATE = "COLLATE"
	sshow ReservedKeywordCOLUMN = "COLUMN"
	sshow ReservedKeywordCONSTRAINT = "CONSTRAINT"
	sshow ReservedKeywordCREATE = "CREATE"
	sshow ReservedKeywordCURRENT_DATE = "CURRENT_DATE"
	sshow ReservedKeywordCURRENT_ROLE = "CURRENT_ROLE"
	sshow ReservedKeywordCURRENT_TIME = "CURRENT_TIME"
	sshow ReservedKeywordCURRENT_TIMESTAMP = "CURRENT_TIMESTAMP"
	sshow ReservedKeywordCURRENT_USER = "CURRENT_USER"
	sshow ReservedKeywordDEFAULT = "DEFAULT"
	sshow ReservedKeywordDEFERRABLE = "DEFERRABLE"
	sshow ReservedKeywordDESC = "DESC"
	sshow ReservedKeywordDISTINCT = "DISTINCT"
	sshow ReservedKeywordDO = "DO"
	sshow ReservedKeywordELSE = "ELSE"
	sshow ReservedKeywordEND = "END"
	sshow ReservedKeywordEXCEPT = "EXCEPT"
	sshow ReservedKeywordFALSE = "FALSE"
	sshow ReservedKeywordFOR = "FOR"
	sshow ReservedKeywordFOREIGN = "FOREIGN"
	sshow ReservedKeywordFROM = "FROM"
	sshow ReservedKeywordGRANT = "GRANT"
	sshow ReservedKeywordGROUP = "GROUP"
	sshow ReservedKeywordHAVING = "HAVING"
	sshow ReservedKeywordIN = "IN"
	sshow ReservedKeywordINITIALLY = "INITIALLY"
	sshow ReservedKeywordINTERSECT = "INTERSECT"
	sshow ReservedKeywordINTO = "INTO"
	sshow ReservedKeywordLEADING = "LEADING"
	sshow ReservedKeywordLIMIT = "LIMIT"
	sshow ReservedKeywordLOCALTIME = "LOCALTIME"
	sshow ReservedKeywordLOCALTIMESTAMP = "LOCALTIMESTAMP"
	sshow ReservedKeywordNEW = "NEW"
	sshow ReservedKeywordNOT = "NOT"
	sshow ReservedKeywordNULL = "NULL"
	sshow ReservedKeywordOFF = "OFF"
	sshow ReservedKeywordOFFSET = "OFFSET"
	sshow ReservedKeywordOLD = "OLD"
	sshow ReservedKeywordON = "ON"
	sshow ReservedKeywordONLY = "ONLY"
	sshow ReservedKeywordOR = "OR"
	sshow ReservedKeywordORDER = "ORDER"
	sshow ReservedKeywordPLACING = "PLACING"
	sshow ReservedKeywordPRIMARY = "PRIMARY"
	sshow ReservedKeywordREFERENCES = "REFERENCES"
	sshow ReservedKeywordSELECT = "SELECT"
	sshow ReservedKeywordSESSION_USER = "SESSION_USER"
	sshow ReservedKeywordSOME = "SOME"
	sshow ReservedKeywordSYMMETRIC = "SYMMETRIC"
	sshow ReservedKeywordTABLE = "TABLE"
	sshow ReservedKeywordTHEN = "THEN"
	sshow ReservedKeywordTO = "TO"
	sshow ReservedKeywordTRAILING = "RAILING"
	sshow ReservedKeywordTRUE = "TRUE"
	sshow ReservedKeywordUNION = "UNION"
	sshow ReservedKeywordUNIQUE = "UNIQUE"
	sshow ReservedKeywordUSER = "USER"
	sshow ReservedKeywordUSING = "USING"
	sshow ReservedKeywordWHEN = "WHEN"
	sshow ReservedKeywordWHERE = "WHERE"

instance SQLShow New where
	sshow NEW = "NEW"

instance SQLShow Old where
	sshow OLD = "OLD"

instance SQLShow FuncNameKeyword where
	sshow FuncNameAUTHORIZATION = "AUTHORIZATION"
	sshow FuncNameBETWEEN = "BETWEEN"
	sshow FuncNameBINARY = "BINARY"
	sshow FuncNameCROSS = "CROSS"
	sshow FuncNameFREEZE = "FREEZE"
	sshow FuncNameFULL = "FULL"
	sshow FuncNameILIKE = "ILIKE"
	sshow FuncNameINNER = "INNER"
	sshow FuncNameIS = "IS"
	sshow FuncNameISNULL = "ISNULL"
	sshow FuncNameJOIN = "JOIN"
	sshow FuncNameLEFT = "LEFT"
	sshow FuncNameLIKE = "LIKE"
	sshow FuncNameNATURAL = "NATURAL"
	sshow FuncNameNOTNULL = "NOTNULL"
	sshow FuncNameOUTER = "OUTER"
	sshow FuncNameOVERLAPS = "OVERLAPS"
	sshow FuncNameRIGHT = "RIGHT"
	sshow FuncNameSIMILAR = "SIMILAR"
	sshow FuncNameVERBOSE = "VERBOSE"

instance SQLShow ColNameKeyword where
	sshow ColNameKeywordBIGINT = "BIGINT"
	sshow ColNameKeywordBIT = "BIT"
	sshow ColNameKeywordBOOLEAN = "BOOLEAN"
	sshow ColNameKeywordCHAR = "CHAR"
	sshow ColNameKeywordCHARACTER  = "CHARACTER"
	sshow ColNameKeywordCOALESCE = "COALESCE"
	sshow ColNameKeywordCONVERT = "CONVERT"
	sshow ColNameKeywordDEC = "DEC"
	sshow ColNameKeywordDECIMAL= "DECIMAL"
	sshow ColNameKeywordEXISTS = "EXISTS"
	sshow ColNameKeywordEXTRACT = "EXTRACT"
	sshow ColNameKeywordFLOAT = "FLOAT"
	sshow ColNameKeywordGREATEST = "GREATEST"
	sshow ColNameKeywordINOUT = "INOUT"
	sshow ColNameKeywordINT= "INT"
	sshow ColNameKeywordINTEGER = "INTEGER"
	sshow ColNameKeywordINTERVAL = "INTERVAL"
	sshow ColNameKeywordLEAST = "LEAST"
	sshow ColNameKeywordNATIONAL = "NATIONAL"
	sshow ColNameKeywordNCHAR = "NCHAR"
	sshow ColNameKeywordNONE = "NONE"
	sshow ColNameKeywordNULLIF = "NULLIF"
	sshow ColNameKeywordNUMERIC = "NUMERIC"
	sshow ColNameKeywordOUT = "OUT"
	sshow ColNameKeywordOVERLAY = "OVERLAY"
	sshow ColNameKeywordPOSITION = "POSITION"
	sshow ColNameKeywordPRECISION = "PRECISION"
	sshow ColNameKeywordREAL = "REAL"
	sshow ColNameKeywordROW = "ROW"
	sshow ColNameKeywordSETOF = "SETOF"
	sshow ColNameKeywordSMALLINT = "SMALLINT"
	sshow ColNameKeywordSUBSTRING = "SUBSTRING"
	sshow ColNameKeywordTIME = "TIME"
	sshow ColNameKeywordTIMESTAMP = "TIMESTAMP"
	sshow ColNameKeywordTREAT = "TREAT"
	sshow ColNameKeywordTRIM = "TRIM"
	sshow ColNameKeywordVARCHAR = "VARCHAR"

instance SQLShow UnreservedKeyword where
	sshow UnreservedKeywordABORT = "ABORT" 
	sshow UnreservedKeywordABSOLUTE = "ABSOLUTE" 
	sshow UnreservedKeywordACCESS = "ACCESS" 
	sshow UnreservedKeywordACTION = "ACTION" 
	sshow UnreservedKeywordADD = "ADD" 
	sshow UnreservedKeywordADMIN = "ADMIN" 
	sshow UnreservedKeywordAFTER = "AFTER" 
	sshow UnreservedKeywordAGGREGATE = "AGGREGATE" 
	sshow UnreservedKeywordALSO = "ALSO" 
	sshow UnreservedKeywordALTER = "ALTER" 
	sshow UnreservedKeywordASSERTION = "ASSERTION" 
	sshow UnreservedKeywordASSIGNMENT = "ASSIGNMENT" 
	sshow UnreservedKeywordAT = "AT" 
	sshow UnreservedKeywordBACKWARD = "BACKWARD" 
	sshow UnreservedKeywordBEFORE = "BEFORE" 
	sshow UnreservedKeywordBEGIN = "BEGIN" 
	sshow UnreservedKeywordBY = "BY" 
	sshow UnreservedKeywordCACHE = "CACHE" 
	sshow UnreservedKeywordCALLED = "CALLED" 
	sshow UnreservedKeywordCASCADE = "CASCADE" 
	sshow UnreservedKeywordCHAIN = "CHAIN" 
	sshow UnreservedKeywordCHARACTERISTICS = "CHARACTERISTICS" 
	sshow UnreservedKeywordCHECKPOINT = "CHECKPOINT" 
	sshow UnreservedKeywordCLASS = "CLASS" 
	sshow UnreservedKeywordCLOSE = "CLOSE" 
	sshow UnreservedKeywordCLUSTER = "CLUSTER" 
	sshow UnreservedKeywordCOMMENT = "COMMENT" 
	sshow UnreservedKeywordCOMMIT = "COMMIT" 
	sshow UnreservedKeywordCOMMITTED = "COMMITED" 
	sshow UnreservedKeywordCONNECTION = "CONNECTION" 
	sshow UnreservedKeywordCONSTRAINTS = "CONSTRAINTS" 
	sshow UnreservedKeywordCONVERSION = "CONVERSION" 
	sshow UnreservedKeywordCOPY = "COPY" 
	sshow UnreservedKeywordCREATEDB = "CREATEDB" 
	sshow UnreservedKeywordCREATEROLE = "CREATEROLE" 
	sshow UnreservedKeywordCREATEUSER = "CREATEUSER" 
	sshow UnreservedKeywordCSV = "CSV" 
	sshow UnreservedKeywordCURSOR = "CURSOR" 
	sshow UnreservedKeywordCYCLE = "CYCLE" 
	sshow UnreservedKeywordDATABASE = "DATABASE" 
	sshow UnreservedKeywordDAY = "DAY" 
	sshow UnreservedKeywordDEALLOCATE = "DEALLOCATE" 
	sshow UnreservedKeywordDECLARE = "DECLARE" 
	sshow UnreservedKeywordDEFAULTS = "DEFAULTS" 
	sshow UnreservedKeywordDEFERRED = "DEFERRED" 
	sshow UnreservedKeywordDEFINER = "DEFINER" 
	sshow UnreservedKeywordDELETE = "DELETE" 
	sshow UnreservedKeywordDELIMITER = "DELIMITER" 
	sshow UnreservedKeywordDELIMITERS = "DELIMITERS" 
	sshow UnreservedKeywordDISABLE = "DISABLE" 
	sshow UnreservedKeywordDOMAIN = "DOMAIN" 
	sshow UnreservedKeywordDOUBLE = "DOUBLE" 
	sshow UnreservedKeywordDROP = "DROP" 
	sshow UnreservedKeywordEACH = "EACH" 
	sshow UnreservedKeywordENABLE = "ENABLE" 
	sshow UnreservedKeywordENCODING = "ENCODING" 
	sshow UnreservedKeywordENCRYPTED = "ENCRYPTED" 
	sshow UnreservedKeywordESCAPE = "ESCAPE" 
	sshow UnreservedKeywordEXCLUDING = "EXCLUDING" 
	sshow UnreservedKeywordEXCLUSIVE = "EXCLUSIVE" 
	sshow UnreservedKeywordEXECUTE = "EXECUTE" 
	sshow UnreservedKeywordEXPLAIN = "EXPLAIN" 
	sshow UnreservedKeywordEXTERNAL = "EXTERNAL" 
	sshow UnreservedKeywordFETCH = "FETCH" 
	sshow UnreservedKeywordFIRST = "FIRST" 
	sshow UnreservedKeywordFORCE = "FORCE" 
	sshow UnreservedKeywordFORWARD = "FORWARD" 
	sshow UnreservedKeywordFUNCTION = "FUNCTION" 
	sshow UnreservedKeywordGLOBAL = "GLOBAL" 
	sshow UnreservedKeywordGRANTED = "GRANTED" 
	sshow UnreservedKeywordHANDLER = "HANDLER" 
	sshow UnreservedKeywordHEADER = "HEADER" 
	sshow UnreservedKeywordHOLD = "HOLD" 
	sshow UnreservedKeywordHOUR = "HOUR" 
	sshow UnreservedKeywordIMMEDIATE = "IMMEDIATE" 
	sshow UnreservedKeywordIMMUTABLE = "IMMUTABLE" 
	sshow UnreservedKeywordIMPLICIT = "IMPLICIT" 
	sshow UnreservedKeywordINCLUDING = "INCLUDING" 
	sshow UnreservedKeywordINCREMENT = "INCREMENT" 
	sshow UnreservedKeywordINDEX = "INDEX" 
	sshow UnreservedKeywordINHERIT = "INHERIT" 
	sshow UnreservedKeywordINHERITS = "INHERITS" 
	sshow UnreservedKeywordINPUT = "INPUT" 
	sshow UnreservedKeywordINSENSITIVE = "INSENSITIVE" 
	sshow UnreservedKeywordINSERT = "INSERT" 
	sshow UnreservedKeywordINSTEAD = "INSTEAD" 
	sshow UnreservedKeywordINVOKER = "INVOKER" 
	sshow UnreservedKeywordISOLATION = "ISOLATION" 
	sshow UnreservedKeywordKEY = "KEY" 
	sshow UnreservedKeywordLANCOMPILER = "LANCOMPILER" 
	sshow UnreservedKeywordLANGUAGE = "LANGUAGE" 
	sshow UnreservedKeywordLARGE = "LARGE" 
	sshow UnreservedKeywordLAST = "LAST" 
	sshow UnreservedKeywordLEVEL = "LEVLE" 
	sshow UnreservedKeywordLISTEN = "LISTEN" 
	sshow UnreservedKeywordLOAD = "LOAD" 
	sshow UnreservedKeywordLOCAL = "LOCAL" 
	sshow UnreservedKeywordLOCATION = "LOCATION" 
	sshow UnreservedKeywordLOCK = "LOCK" 
	sshow UnreservedKeywordLOGIN = "LOGIN" 
	sshow UnreservedKeywordMATCH = "MATCH" 
	sshow UnreservedKeywordMAXVALUE = "MAXVALUE" 
	sshow UnreservedKeywordMINUTE = "MINUTE" 
	sshow UnreservedKeywordMINVALUE = "MINVALUE" 
	sshow UnreservedKeywordMODE = "MODE" 
	sshow UnreservedKeywordMONTH = "MONTH" 
	sshow UnreservedKeywordMOVE = "MOVE" 
	sshow UnreservedKeywordNAMES = "NAMES" 
	sshow UnreservedKeywordNEXT = "NEXT" 
	sshow UnreservedKeywordNO = "NO" 
	sshow UnreservedKeywordNOCREATEDB = "NOCREATEDB" 
	sshow UnreservedKeywordNOCREATEROLE = "NOCREATEROLE" 
	sshow UnreservedKeywordNOCREATEUSER = "NOCREATEUSER" 
	sshow UnreservedKeywordNOINHERIT = "NOINHERIT" 
	sshow UnreservedKeywordNOLOGIN = "NOLOGIN" 
	sshow UnreservedKeywordNOSUPERUSER = "NOSUPERUSER" 
	sshow UnreservedKeywordNOTHING = "NOTHING" 
	sshow UnreservedKeywordNOTIFY = "NOTIFY" 
	sshow UnreservedKeywordNOWAIT = "NOWAIT" 
	sshow UnreservedKeywordOBJECT = "OBJECT" 
	sshow UnreservedKeywordOF = "OF" 
	sshow UnreservedKeywordOIDS = "OIDS" 
	sshow UnreservedKeywordOPERATOR = "OPERATOR" 
	sshow UnreservedKeywordOPTION = "OPTION" 
	sshow UnreservedKeywordOWNER = "OWNER" 
	sshow UnreservedKeywordPARTIAL = "PARTIAL" 
	sshow UnreservedKeywordPASSWORD = "PASSWORD" 
	sshow UnreservedKeywordPREPARE = "PREPARE" 
	sshow UnreservedKeywordPREPARED = "PREPARED" 
	sshow UnreservedKeywordPRESERVE = "PRESERVE" 
	sshow UnreservedKeywordPRIOR = "PRIOR" 
	sshow UnreservedKeywordPRIVILEGES = "PRIVILEGES" 
	sshow UnreservedKeywordPROCEDURAL = "PROCEDURAL" 
	sshow UnreservedKeywordPROCEDURE = "PROCEDURE" 
	sshow UnreservedKeywordQUOTE = "QUOTE" 
	sshow UnreservedKeywordREAD = "READ" 
	sshow UnreservedKeywordRECHECK = "RECHECK" 
	sshow UnreservedKeywordREINDEX = "REINDEX" 
	sshow UnreservedKeywordRELATIVE = "RELATIVE" 
	sshow UnreservedKeywordRELEASE = "RELEASE" 
	sshow UnreservedKeywordRENAME = "RENAME" 
	sshow UnreservedKeywordREPEATABLE = "REPEATABLE" 
	sshow UnreservedKeywordREPLACE = "REPLACE" 
	sshow UnreservedKeywordRESET = "RESET" 
	sshow UnreservedKeywordRESTART = "RESTART" 
	sshow UnreservedKeywordRESTRICT = "RESTRICT" 
	sshow UnreservedKeywordRETURNS = "RETURNS" 
	sshow UnreservedKeywordREVOKE = "REVOKE" 
	sshow UnreservedKeywordROLE = "ROLE" 
	sshow UnreservedKeywordROLLBACK = "ROLLBACK" 
	sshow UnreservedKeywordROWS = "ROWS" 
	sshow UnreservedKeywordRULE = "RULE" 
	sshow UnreservedKeywordSAVEPOINT = "SAVEPOINT" 
	sshow UnreservedKeywordSCHEMA = "SCHEMA" 
	sshow UnreservedKeywordSCROLL = "SCROLL" 
	sshow UnreservedKeywordSECOND = "SECOND" 
	sshow UnreservedKeywordSECURITY = "SECURITY" 
	sshow UnreservedKeywordSEQUENCE = "SEQUENCE" 
	sshow UnreservedKeywordSERIALIZABLE = "SERIALIZABLE" 
	sshow UnreservedKeywordSESSION = "SESSION" 
	sshow UnreservedKeywordSET = "SET" 
	sshow UnreservedKeywordSHARE = "SHARE" 
	sshow UnreservedKeywordSHOW = "SHOW" 
	sshow UnreservedKeywordSIMPLE = "SIMPLE" 
	sshow UnreservedKeywordSTABLE = "STABLE" 
	sshow UnreservedKeywordSTART = "START" 
	sshow UnreservedKeywordSTATEMENT = "STATEMENT" 
	sshow UnreservedKeywordSTATISTICS = "STATISTICS" 
	sshow UnreservedKeywordSTDIN = "STDIN" 
	sshow UnreservedKeywordSTDOUT = "STDOUT" 
	sshow UnreservedKeywordSTORAGE = "STORAGE" 
	sshow UnreservedKeywordSUPERUSER = "SUPERUSER" 
	sshow UnreservedKeywordSYSID = "SYSID" 
	sshow UnreservedKeywordSYSTEM = "SYSTEM" 
	sshow UnreservedKeywordSTRICT = "STRICT" 
	sshow UnreservedKeywordTABLESPACE = "TABLESPACE" 
	sshow UnreservedKeywordTEMP = "TEMP" 
	sshow UnreservedKeywordTEMPLATE = "TEMPLATE" 
	sshow UnreservedKeywordTEMPORARY = "TEMPORARY" 
	sshow UnreservedKeywordTOAST = "TOAST" 
	sshow UnreservedKeywordTRANSACTION = "TRANSACTION" 
	sshow UnreservedKeywordTRIGGER = "TRIGGER" 
	sshow UnreservedKeywordTRUNCATE = "TRUNCATE" 
	sshow UnreservedKeywordTRUSTED = "TRUSTED" 
	sshow UnreservedKeywordTYPE = "TYPE" 
	sshow UnreservedKeywordUNCOMMITTED = "UNCOMMITED" 
	sshow UnreservedKeywordUNENCRYPTED = "UNENCRYPTED" 
	sshow UnreservedKeywordUNKNOWN = "UNKNOWN"
	sshow UnreservedKeywordUNLISTEN = "UNLISTEN"
	sshow UnreservedKeywordUNTIL = "UNTIL"
	sshow UnreservedKeywordUPDATE = "UPDATE"
	sshow UnreservedKeywordVACUUM = "VACUUM"
	sshow UnreservedKeywordVALID = "VALID"
	sshow UnreservedKeywordVALIDATOR = "VALIDATOR"
	sshow UnreservedKeywordVALUES = "VALUES"
	sshow UnreservedKeywordVARYING = "VARYING"
	sshow UnreservedKeywordVIEW = "VIEW"
	sshow UnreservedKeywordVOLATILE = "VOLATILE"
	sshow UnreservedKeywordWITH = "WITH"
	sshow UnreservedKeywordWITHOUT = "WITHOUT"
	sshow UnreservedKeywordWORK = "WORK"
	sshow UnreservedKeywordWRITE = "WRITE"
	sshow UnreservedKeywordYEAR = "YEAR"
	sshow UnreservedKeywordZONE = "ZONE"

instance SQLShow AExprConst where
	sshow (AExpCI x) = sshow x
	sshow (AExpCF x) = sshow x
	sshow (AExpCS x) = sshow x
	sshow (AExpCB x) = sshow x
	sshow (AExpCX x) = sshow x
	sshow (AExpCType x y) = sshow x ++ ' ' : sshow y
	sshow (AExpCInt x (Just y) z w) = sshow x ++ " (" ++ sshow y ++ ") " ++ sshow z ++ ' ' : sshow w
	sshow (AExpCInt x Nothing z w) = sshow x ++ ' ' : sshow z ++ ' ' : sshow w
	sshow AExpCTrue = "TRUE"
	sshow AExpCFalse = "FALSE"
	sshow AExpCNull = "NULL"

instance SQLShow FuncName where
	sshow (FuncNameFunc x) = sshow x
	sshow (FuncNameRelInd x y) = sshow x ++ ' ' : sshow y

instance SQLShow QualifiedName where
	sshow (QualNameRel x) = sshow x
	sshow (QualNameRelInd x y) = sshow x ++ ' ' : sshow y

instance SQLShow UpdateTargetEl where
	sshow (UpdtTargetEl x y mb) = sshow x ++ ' ' : sshow y ++ '=' : sshow mb

instance SQLShow TargetEl where
	sshow (TargetElAs x y) = sshow x ++ " AS " ++ sshow y
	sshow (TargetElExp x) = sshow x
	sshow TargetElStar = "*"

instance SQLShow IndirectionEl where
	sshow (IndAttr x) = '.' : sshow x
	sshow IndStar = ".*"
	sshow (IndExp x) = '[' : sshow x ++ "]"
	sshow (IndBinExp x y) = '[' : sshow x ++ ':' : sshow y ++ "]"

instance SQLShow Columnref where
	sshow (ColRefRel x) = sshow x
	sshow (ColRefRelInd x y) = sshow x ++ ' ' : sshow y

instance SQLShow CaseDefault where
	sshow (CaseDef (Just x)) = "ELSE " ++ sshow x
	sshow (CaseDef Nothing) = ""

instance SQLShow WhenClause where
	sshow (WhenClauseConst x y) = "WHEN " ++ sshow x ++ " THEN " ++ sshow y

instance SQLShow CaseExpr where
	sshow (CaseExp x y z) = "CASE " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z ++ " END"

instance SQLShow InExpr where
	sshow (InExpSelect x) = sshow x
	sshow (InExpLst x) = sshow x

instance SQLShow SubstrFor where
	sshow (SubstringFor x) = "FOR " ++ sshow x

instance SQLShow SubstrFrom where
	sshow (SubstringFrom x) = "FROM " ++ sshow x

instance SQLShow SubstrList where
	sshow (SStrFromFor x y z) = sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (SStrFrom x y) = sshow x ++ ' ' : sshow y
	sshow (SStrFor x y) = sshow x ++ ' ' : sshow y
	sshow (SStrLst x) = sshow x
	sshow SStrEmpty = ""

instance SQLShow PositionList where
	sshow (PosLst x y) = sshow x ++ " IN " ++ sshow y
	sshow PosLstEmpty = ""

instance SQLShow OverlayPlacing where
	sshow (OverlayPlace x) = "PLACING " ++ sshow x

instance SQLShow ArrayExpr where
	sshow (ArrayExp either) = '[' : sshow either ++ "]"

instance SQLShow ExtractList where
	sshow (ExtractLst x y) = sshow x ++ " FROM " ++ sshow y
	sshow ExtractEmpty = ""

instance SQLShow SubQueryOp where
	sshow (SubQAll x) = sshow x
	sshow (SubQOp x) = "OPERATOR (" ++ sshow x ++ ")"
	sshow SubQLike = "LIKE"
	sshow SubQNotLike = "NOT LIKE"
	sshow SubQIlike = "ILIKE"
	sshow SubQNotIlike = "NOT ILIKE"

instance SQLShow QualAllOp where
	sshow (QualAllOper (Left x)) = sshow x
	sshow (QualAllOper (Right x)) = "OPERATOR (" ++ sshow x ++ ")"

instance SQLShow QualOp where
	sshow (QualOper (Left x)) = sshow x
	sshow (QualOper (Right x)) = "OPERATOR (" ++ sshow x ++ ")"

instance SQLShow Slash where
	sshow SLASH = "/"

instance SQLShow Percentage where
	sshow PERCENTAGE = "%"

instance SQLShow Hat where
	sshow HAT = "^"

instance SQLShow Smaller where
	sshow SMALLER = "<"

instance SQLShow Greater where
	sshow GREATER = ">"

instance SQLShow Equal where
	sshow EQUAL = "="

instance SQLShow Plus where
	sshow PLUS = "+"
	
instance SQLShow Minus where
	sshow MINUS = "-"

instance SQLShow Row where
	sshow (RowExpr x) = "ROW (" ++ sshow x ++ ")"
	sshow RowSingle = "ROW ()"
	sshow (RowExprs x y) = '(' : sshow x ++ ',' : sshow y ++ ")"

instance SQLShow FuncExpr where
	sshow (FuncExpFunc x Nothing) = sshow x ++ "()"
	sshow (FuncExpFunc x (Just (OneOf4 y))) = sshow x ++ " (" ++ sshow y ++ ")"
	sshow (FuncExpFunc x (Just (TwoOf4 y))) = sshow x ++ " (ALL " ++ sshow y ++ ")"
	sshow (FuncExpFunc x (Just (ThreeOf4 y))) = sshow x ++ " (DISTINCT " ++ sshow y ++ ")"
	sshow (FuncExpFunc x (Just (FourOf4 y))) = sshow x ++ " (*)"
	sshow FuncExpCurrentDate = "CURRENT_DATE"
	sshow (FuncExpCurrentTime Nothing) = "CURRENT_TIME"
	sshow (FuncExpCurrentTime (Just x)) = "CURRENT_TIME (" ++ sshow x ++ ")"
	sshow (FuncExpCurrentTimestamp Nothing) = "CURRENT_TIMESTAMP"
	sshow (FuncExpCurrentTimestamp (Just x)) = "CURRENT_TIMESTAMP (" ++ sshow x ++ ")"
	sshow (FuncExpLocalTime Nothing) = "LOCALTIME"
	sshow (FuncExpLocalTime (Just x)) = "LOCALTIME (" ++ sshow x ++ ")"
	sshow (FuncExpLocalTimestamp Nothing) = "LOCALTIMESTAMP"
	sshow (FuncExpLocalTimestamp (Just x)) = "LOCALTIMESTAMP (" ++ sshow x ++ ")"
	sshow FuncExpCurrentRole = "CURRENT_ROLE"
	sshow FuncExpCurrentUser = "CURRENT_USER"
	sshow FuncExpSessionUser = "SESSION_USER"
	sshow FuncExpUser = "USER"
	sshow (FuncExpCast x y) = "CAST (" ++ sshow x ++ " AS " ++ sshow y ++ ")"
	sshow (FuncExpExtract x) = "EXTRACT (" ++ sshow x ++ ")"
	sshow (FuncExpOverlay x) = "OVERLAY (" ++ sshow x ++ ")"
	sshow (FuncExpPosition x) = "POSITION (" ++ sshow x ++ ")"
	sshow (FuncExpSubstring x) = "SUBSTRING (" ++ sshow x ++ ")"
	sshow (FuncExpTreat x y) = "TREAT (" ++ sshow x ++ " AS " ++ sshow y ++ ")"
	sshow (FuncExpTrim (OneOf4 x)) = "TRIM (BOTH " ++ sshow x ++ ")"
	sshow (FuncExpTrim (TwoOf4 x)) = "TRIM (LEADING " ++ sshow x ++ ")"
	sshow (FuncExpTrim (ThreeOf4 x)) = "TRIM (TRAILING " ++ sshow x ++ ")"
	sshow (FuncExpTrim (FourOf4 x)) = "TRIM (" ++ sshow x ++ ")"
	sshow (FuncExpConvert (Left (x,y))) = "CONVERT (" ++ sshow x ++ " USING " ++ sshow y ++ ")"
	sshow (FuncExpConvert (Right x)) = "CONVERT (" ++ sshow x ++ ")"
	sshow (FuncExpNullif x y) = "NULLIF (" ++ sshow x ++ ',' : sshow y ++ ")"
	sshow (FuncExpCoalesce x) = "COALESCE " ++ sshow x ++ ")"
	sshow (FuncExpGreatest x) = "GREATEST (" ++ sshow x ++ ")"
	sshow (FuncExpLeast x) = "LEAST (" ++ sshow x ++ ")"

instance SQLShow CExpr where
	sshow (CExpCol x) = sshow x
	sshow (CExpAExpC x) = sshow x
	sshow (CExpParam x) = "PARAM " ++ sshow x
	sshow (CExpAExp x y) = '(' : sshow x ++ ')' : sshow y
	sshow (CExpCase x) = sshow x
	sshow (CExpFunc x) = sshow x
	sshow (CExpSelect x) = sshow x
	sshow (CExpExists x) = "EXISTS " ++ sshow x
	sshow (CExpArray either) = "ARRAY " ++ sshow either
	sshow (CExpRow x) = sshow x

instance SQLShow BExpr where
	sshow (BExpCExp x) = sshow x
	sshow (BExpTypecast x y) = sshow x ++ " TYPECAST " ++ sshow y
	sshow (BExpPositive x) = '+' : sshow x
	sshow (BExpNegative x) = '-' : sshow x
	sshow (BExpMath x y z) = sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (BExpQual mb x y) = sshow mb ++ ' ' : sshow x ++ ' ' : sshow y
	sshow (BExpIsDistinct x y) = sshow x ++ " IS DISTINCT FROM " ++ sshow y
	sshow (BExpIsOf x y) = sshow x ++ " IS OF (" ++ sshow y ++ ")"
	sshow (BExpIsNotOf x y) = sshow x ++ " IS NOT OF (" ++ sshow y ++ ")"

instance SQLShow AExpr where
	sshow (AExpCExp x) = sshow x
	sshow (AExpTypecast x y) = sshow x ++ " TYPECAST " ++ sshow y
	sshow (AExpTZ x y) = sshow x ++ " AT TIME ZONE " ++ sshow y
	sshow (AExpPositive x) = '+' : sshow x
	sshow (AExpNegative x) = '-' : sshow x
	sshow (AExpMath x y z) = sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (AExpQual mb x y) = sshow mb ++ ' ' : sshow x ++ ' ' : sshow y
	sshow (AExpAnd x y) = sshow x ++ " AND " ++ sshow y
	sshow (AExpOr x y) = sshow x ++ " OR " ++ sshow y
	sshow (AExpNot x) = "NOT " ++ sshow x
	sshow (AExpLike x y (Just z)) = sshow x ++ " LIKE " ++ sshow y ++ " ESCAPE " ++ sshow z
	sshow (AExpLike x y Nothing) = sshow x ++ " LIKE " ++ sshow y
	sshow (AExpNotLike x y (Just z)) = sshow x ++ " NOT LIKE " ++ sshow y ++ " ESCAPE " ++ sshow z
	sshow (AExpNotLike x y Nothing) = sshow x ++ " NOT LIKE " ++ sshow y
	sshow (AExpILike x y (Just z)) = sshow x ++ " ILIKE " ++ sshow y ++ " ESCAPE " ++ sshow z
	sshow (AExpILike x y Nothing) = sshow x ++ " ILIKE " ++ sshow y
	sshow (AExpNotILike x y (Just z)) = sshow x ++ " NOT ILIKE " ++ sshow y ++ " ESCAPE " ++ sshow z
	sshow (AExpNotILike x y Nothing) = sshow x ++ " NOT ILIKE " ++ sshow y
	sshow (AExpSimilar x y (Just z)) = sshow x ++ " SIMILAR TO " ++ sshow y ++ " ESCAPE " ++ sshow z
	sshow (AExpSimilar x y Nothing) = sshow x ++ " SIMILAR TO " ++ sshow y
	sshow (AExpNotSimilar x y (Just z)) = sshow x ++ " NOT SIMILAR TO " ++ sshow y ++ " ESCAPE " ++ sshow z
	sshow (AExpNotSimilar x y Nothing) = sshow x ++ " NOT SIMILAR TO " ++ sshow y
	sshow (AExpIsNull x) = sshow x ++ " IS NULL"
	sshow (AExpNotNull x) = sshow x ++ " NOTNULL"
	sshow (AExpIsNotNull x) = sshow x ++ " IS NOT NULL"
	sshow (AExpOverlaps x y) = sshow x ++ " OVERLAPS " ++ sshow y
	sshow (AExpIsTrue x) = sshow x ++ " IS TRUE"
	sshow (AExpIsNotTrue x) = sshow x ++ " IS NOT TRUE"
	sshow (AExpIsFalse x) = sshow x ++ " IS FALSE"
	sshow (AExpIsNotFalse x) = sshow x ++ " IS NOT FALSE"
	sshow (AExpIsUnknown x) = sshow x ++ " IS UNKNOWN"
	sshow (AExpIsNotUnknown x) = sshow x ++ " IS NOT UNKNOWN"
	sshow (AExpIsDistinct x y) = sshow x ++ " IS DISTINCT FROM " ++ sshow y
	sshow (AExpIsOf x y) = sshow x ++ " IS OF (" ++ sshow y ++ ")"
	sshow (AExpIsNotOf x y) = sshow x ++ " IS NOT OF (" ++ sshow y ++ ")"
	sshow (AExpBetween x y z w) = sshow x ++ " BETWEEN " ++ sshow y ++ ' ' : sshow z ++ " AND " ++ sshow w
	sshow (AExpNotBetween x y z w) = sshow x ++ " NOT BETWEEN " ++ sshow y ++ ' ' : sshow z ++ " AND " ++ sshow w
	sshow (AExpBetweenSymmetric x y z) = sshow x ++ " BETWEEN SYMMETRIC " ++ sshow y ++ " AND " ++ sshow z
	sshow (AExpNotBetweenSymmetric x y z) = sshow x ++ " NOT BETWEEN SYMMETRIC " ++ sshow y ++ " AND " ++ sshow z
	sshow (AExpIn x y) = sshow x ++ " IN " ++ sshow y
	sshow (AExprNotIn x y) = sshow x ++ " NOT IN " ++ sshow y
	sshow (AExpSub x y z (Left w)) = sshow x ++ ' ' : sshow y ++ ' ' : sshow z ++ ' ' : sshow w
	sshow (AExpSub x y z (Right w)) = sshow x ++ ' ' : sshow y ++ " (" ++ sshow z ++ ")"
	sshow (AExpUnique x) = "UNIQUE " ++ sshow x

instance SQLShow OptInterval where
	sshow IntervalYear = "YEAR"
	sshow IntervalMonth = "MONTH"
	sshow IntervalDay = "DAY"
	sshow IntervalHour = "HOUR"
	sshow IntervalMinute = "MINUTE"
	sshow IntervalSecond = "SECOND"
	sshow IntervalYearToMonth = "YEAR TO MONTH"
	sshow IntervalDayToHour = "DAY TO HOUR"
	sshow IntervalDayToMinute = "DAY TO MINUTE"
	sshow IntervalDayToSecond = "DAY TO SECOND"
	sshow IntervalHourToMinute = "HOUR TO MINUTE"
	sshow IntrevalHourToSecond = "HOUR TO SECOND"
	sshow IntervalMinuteToSecond = "MINUTE TO SECOND"

instance SQLShow OptTimezone where
	sshow WithTimeZone = "WITH TIME ZONE"
	sshow WithoutTimeZone = "WITHOUT TIME ZONE"
	sshow NoTimeZoneDecl = ""

instance SQLShow ConstDatetime where
	sshow (ConstTimestamp (Just x) y) = "TIMESTAMP (" ++ sshow x ++ ')' : sshow y
	sshow (ConstTimestamp Nothing y) = "TIMESTAMP " ++ sshow y

instance SQLShow OptCharset where
	sshow (OptCharConst (Just x)) = "CHARACTER SET " ++ sshow x
	sshow (OptCharConst Nothing) = ""

instance SQLShow Character2 where
	sshow (Char2Character x) = "CHARACTER " ++ sshow x
	sshow Char2VarChar = "VARCHAR"
	sshow (Char2National x) = "NATIONAL CHARACTER " ++ sshow x

instance SQLShow BitDefinition where
	sshow (BitDef x (Just y)) = "BIT " ++ sshow x ++ '(' : sshow y ++ ")"
	sshow (BitDef x Nothing) = "BIT " ++ sshow x

instance SQLShow OptNumeric where
	sshow (OptNumericConst (Just (x,Just y))) = '(' : sshow x ++ ',' : sshow y ++ ")"
	sshow (OptNumericConst Nothing) = ""
	sshow (OptNumericConst (Just (x,Nothing))) = '(' : sshow x ++ ")"

instance SQLShow OptFloat where
	sshow (OptFloatConst (Just x)) = '(' : sshow x ++ ")"
	sshow (OptFloatConst Nothing) = ""

instance SQLShow Numeric where
	sshow NumInt = "INT"
	sshow NumInteger = "INTEGER"
	sshow NumSmallInt = "SMALLINT"
	sshow NumBigInt = "BIGINT"
	sshow NumReal = "REAL"
	sshow (NumFloat x) = "FLOAT " ++ sshow x
	sshow NumDouble = "DOUBLE PRECISION"
	sshow (NumDecimal x) = "DECIMAL " ++ sshow x
	sshow (NumNumeric x) = "NUMERIC " ++ sshow x
	sshow NumBool = "BOOLEAN"

instance SQLShow ConstTypename where
	sshow (ConstTypeGeneric x) = sshow x
	sshow (ConstTypeNum x) = sshow x
	sshow (ConstTypeBit x) = sshow x
	sshow (ConstTypeChar x) = sshow x
	sshow (ConstTypeDate x) = sshow x

instance SQLShow SimpleTypename where
	sshow (SimpTypeGeneric x) = sshow x
	sshow (SimpTypeNum x) = sshow x
	sshow (SimpTypeBit x) = sshow x
	sshow (SimpTypeChar x) = sshow x
	sshow (SimpTypeDate x) = sshow x
	sshow (SimpTypeInterval x (Just y) z) = sshow x ++ " (" ++ sshow y ++ ") " ++ sshow z
	sshow (SimpTypeInterval x Nothing z) = sshow x ++ ' ' : sshow z
	sshow (SimpType x y) = sshow x ++ ' ' : sshow y

instance SQLShow OptArrayBounds where
	sshow (SingleArray x) = sshow x ++ "[]"
	sshow (IconstArray x y) = sshow x ++ '[' : sshow y ++ "]"
	sshow EmptyArray = ""

instance SQLShow Typename where
	sshow (TypenameConst mb y (Right z)) = sshow mb ++ ' ' : sshow y ++ " ARRAY [" ++ sshow z ++ "]"
	sshow (TypenameConst mb y (Left z)) = sshow mb ++ ' ' : sshow y ++ ' ' : sshow z

instance SQLShow WhereClause where
	sshow (WhereClauseConst (Just x)) = "WHERE " ++ sshow x
	sshow (WhereClauseConst Nothing) = ""

instance SQLShow RelationExpr where
	sshow (RelationExprConst (Just x) y Nothing) = sshow x ++ " (" ++ sshow y ++ ")"
	sshow (RelationExprConst x y z) = sshow x ++ ' ' : sshow y ++ ' ' : sshow z

instance SQLShow JoinQual where
	sshow (JoinQualConst (Left x)) = "USING (" ++ sshow x ++ ")"
	sshow (JoinQualConst (Right x)) = "ON " ++ sshow x

instance SQLShow AliasClause where
	sshow (AliasClauseConst x y (Just z)) = sshow x ++ ' ' : sshow y ++ " (" ++ sshow z ++ ")"
	sshow (AliasClauseConst x y Nothing) = sshow x ++ ' ' : sshow y

instance SQLShow JoinedTable where
	sshow (JoinedTableJoined x) = '(' : sshow x ++ ")"
	sshow (JoinedTableCrossJoin x y) = sshow x ++ " CROSS JOIN " ++ sshow y
	sshow (JoinedTableUnionjoin x y) = sshow x ++ " UNIONJOIN " ++ sshow y
	sshow (JoinedTableJoin x mb z w) = sshow x ++ ' ' : sshow mb ++ " JOIN " ++ sshow z ++ ' ' : sshow w
	sshow (JoinedTableNatural x mb z) = sshow x ++ " NATURAL " ++ sshow mb ++ " JOIN " ++ sshow z

instance SQLShow TableRef where
	sshow (TableRefExpr x mb) = sshow x ++ ' ' : sshow mb
	sshow (TableRefFunc x mb) = sshow x ++ ' ' : sshow mb
	sshow (TableRefFunc2 x oneof y) = sshow x ++ ' ' : sshow oneof ++ " (" ++ sshow y ++ ")"
	sshow (TableRefSelect x mb) = sshow x ++ ' ' : sshow mb
	sshow (TableRefJoin x (Just y)) = '(' : sshow x ++ ") " ++ sshow y
	sshow (TableRefJoin x Nothing) = sshow x

instance SQLShow SortBy where
	sshow (SortByUsing x y) = sshow x ++ " USING " ++ sshow y
	sshow (SortByAsc x) = sshow x ++ " ASC"
	sshow (SortByDesc x) = sshow x ++ " DESC"
	sshow (SortByExp x) = sshow x

instance (SQLShow a) => SQLShow (MyList a) where
	sshow (MyLst l) = let list = foldr (\h r -> sshow h ++ ',' : r) [] l
			  in if (null list) then list else init list

instance SQLShow FromClause where
	sshow (FromClauseConst (Just x)) = "FROM " ++ sshow x
	sshow (FromClauseConst Nothing) = ""

instance SQLShow LockedRelsList where
	sshow (LockedRelsLst (Just x)) = "OF " ++ sshow x
	sshow (LockedRelsLst Nothing) = ""

instance SQLShow ForLockingClause where
	sshow (ForLckUpdate x y) = "FOR UPDATE " ++ sshow x ++ ' ' : sshow y
	sshow (ForLckShare x y) = "FOR SHARE " ++ sshow x ++ ' ' : sshow y
	sshow ForLckReadOnly = "FOR READ ONLY"

instance SQLShow HavingClause where
	sshow (HavClause (Just x)) = "HAVING " ++ sshow x
	sshow (HavClause Nothing) = ""

instance SQLShow GroupClause where
	sshow (GrpClause (Just x)) = "GROUP BY " ++ sshow x
	sshow (GrpClause Nothing) = ""

instance SQLShow SelectLimit where
	sshow (SlctLimitOffset x y) = "LIMIT " ++ sshow x ++ " OFFSET " ++ sshow y
	sshow (SlctLimit x) = "LIMIT " ++ sshow x
	sshow (SlctOffset x) = "OFFSET " ++ sshow x

instance SQLShow SortClause where
	sshow (SortCl x) = "ORDER BY " ++ sshow x

instance SQLShow OptDistinct where
	sshow DisctDistinct = "DISTINCT"
	sshow (DisctDistinctOn x) = "DISTINCT ON (" ++ sshow x ++ ")"
	sshow DisctAll = "ALL"
	sshow NoDisct = ""

instance SQLShow OptTempTableName where
	sshow (TNTemporary x y) = "TEMPORARY " ++ sshow x ++ ' ' : sshow y
	sshow (TNTemp x y) = "TEMP " ++ sshow x ++ ' ' : sshow y
	sshow (TNLocalTemporary x y) = "LOCAL TEMPORARY " ++ sshow x ++ ' ' : sshow y
	sshow (TNLocalTemp x y) = "LOCAL TEMP " ++ sshow x ++ ' ' : sshow y
	sshow (TNGlobalTemporary x y) = "GLOBAL TEMPORARY " ++ sshow x ++ ' ' : sshow y
	sshow (TNGlobalTemp x y) = "GLOBAL TEMP " ++ sshow x ++ ' ' : sshow y
	sshow (TNTable x) = "TABLE " ++ sshow x
	sshow (TN x) = sshow x

instance SQLShow SimpleSelect where
	sshow (SimpSlct a b c d e f g) = "SELECT " ++ sshow a ++ ' ' : sshow b ++ ' ' : sshow c ++ ' ' : sshow d ++ ' ' : sshow e ++ ' ' : sshow f ++ ' ' : sshow g
	sshow (UnionSlct x y z) = sshow x ++ " UNION " ++ sshow y ++ sshow z
	sshow (IntersectSlct x y z) = sshow x ++ " INTERSECT " ++ sshow y ++ sshow z
	sshow (ExceptSlct x y z) = sshow x ++ " EXCEPT " ++ sshow y ++ sshow z

instance SQLShow SelectNoParens where
	sshow (SlctSimple x) = sshow x
	sshow (SlctBin x y) = sshow x ++ ' ' : sshow y
	sshow (SlctMulti1 a b c d) = sshow a ++ ' ' : sshow b ++ ' ' : sshow c ++ ' ' : sshow d
	sshow (SlctMulti2 a b c d) = sshow a ++ ' ' : sshow b ++ ' ' : sshow c ++ ' ' : sshow d

instance SQLShow SelectWithParens where
	sshow (SlctNo x) = '(' : sshow x ++ ")"
	sshow (SlctWith x ) = '(' : sshow x ++ ")"

instance SQLShow OptHold where
	sshow NoHold = ""
	sshow WithHold = "WITH HOLD"
	sshow WithoutHold = " WITHOUT HOLD"

instance SQLShow LockType where
	sshow LckAccessShare = "Access SHARE"
	sshow LckRowShare = "ROW SHARE"
	sshow LckRowExclusive = "ROW EXCLUSIVE"
	sshow LckShareUptateExclusive = "SHARE UPDATE EXCLUSIVE"
	sshow LckShare = "SHARE"
	sshow LckShareRowExclusive = "SHARE ROW EXCLUSIVE"
	sshow LckExclusive = "EXCLUSIVE"
	sshow LckAccessExclusive = "Access EXCLUSIVE"

instance SQLShow OptLock where
	sshow (OptLck (Just x)) = "IN " ++ sshow x ++ " MODE"
	sshow (OptLck Nothing) = ""

instance SQLShow UsingClause where
	sshow (UseClause (Just x)) = "USING " ++ sshow x
	sshow (UseClause Nothing) = ""

instance SQLShow InsertColumnItem where
	sshow (InsColItem x y) = sshow x ++ ' ' : sshow y

instance SQLShow InsertRest where
	sshow (InsValues x) = "VALUES (" ++ sshow x ++ ")"
	sshow InsDefaultValues = "DEFAULT VALUES"
	sshow (InsSelect x) = sshow x
	sshow (InsColValues x y) = '(' : sshow x ++ ") VALUES (" ++ sshow y ++ ")"
	sshow (InsCol x y) = '(' : sshow x ++ ')' : sshow y

instance SQLShow ExecuteParamClause where
	sshow (ExecClause (Just x)) = '(' : sshow x ++ ")"
	sshow (ExecClause Nothing) = ""

instance SQLShow ExecuteStatement where
	sshow (ExecStmtExecute x y) = "EXECUTE " ++ sshow x ++ ' ' : sshow y
	sshow (ExecStmtCreate x y z w s) = "CREATE " ++ sshow x ++ " TABLE " ++ sshow y ++ ' ' : sshow z ++ " AS EXECUTE " ++ sshow w ++ ' ' : sshow s

instance SQLShow PrepTypeClause where
	sshow (PrepTypeClauseConst (Just x)) = '(' : sshow x ++ ")"
	sshow (PrepTypeClauseConst Nothing) = ""

instance SQLShow OptNameList where
	sshow (OptNameLst (Just x)) = '(' : sshow x ++ ")"
	sshow (OptNameLst Nothing) = ""

instance SQLShow AlterdbOptItem where
	sshow (AlterdbItemConst x y) = "CONNECTION LIMIT " ++ sshow x ++ ' ' : sshow y

instance SQLShow CreatedbOptItem where
	sshow (DBItemTablespace eq either) = "TABLESPACE " ++ sshow eq ++ ' ' : sshow either
	sshow (DBItemLocation eq either) = "LOCATION " ++ sshow eq ++ ' ' : sshow either
	sshow (DBItemTemplate eq either) = "TEMPLATE " ++ sshow eq ++ ' ' : sshow either
	sshow (DBItemEncoding eq oneof) = "ENCODING " ++ sshow eq ++ ' ' : sshow oneof
	sshow (DBItemConnection x y) = "CONNECTION LIMIT " ++ sshow x ++ ' ' : sshow y
	sshow (DbItemOwner x y) = "OWNER " ++ sshow x ++ ' ' : sshow y

instance SQLShow TransactionStatement where
	sshow (TransAbort x) = "ABORT " ++ sshow x
	sshow (TransBegin x y) = "BEGIN " ++ sshow x ++ ' ' : sshow y
	sshow (TransStart x) = "START TRANSACTION " ++ sshow x
	sshow (TransCommit x) = "COMMIT " ++ sshow x
	sshow (TransEnd x) = "END " ++ sshow x
	sshow (TransRollback x) = "ROLLBACK " ++ sshow x
	sshow (TransSavepoint x) = "SAVEPOINT " ++ sshow x
	sshow (TransRelease x y) = "RELEASE " ++ sshow x ++ sshow y
	sshow (TransRollbackSave x y z) = "ROLLBACK " ++ sshow x ++ " TO " ++ sshow y ++ ' ' : sshow z
	sshow (TransPrepare x) = "PREPARE TRANSACTION " ++ sshow x
	sshow (TransCommitPreparred x) = "COMMIT PREPARED " ++ sshow x
	sshow (TransRollbackPrepared x) = "ROLLBACK PREPARED " ++ sshow x

instance SQLShow RuleActionList where
	sshow RuleActNothing = "NOTHING"
	sshow (RuleActStmt stmt) = sshow stmt
	sshow (RuleActMulti x) = '(' : sshow x ++ ")"

instance SQLShow AlterOwnerStatement where
	sshow (AltOwnerAggregate x y z) = "AGGREGATE " ++ sshow x ++ '(' : sshow y ++ ") OWNER TO " ++ sshow z
	sshow (AltOwnerConversion x y) = "CONVERSION " ++ sshow x ++ " OWNER TO " ++ sshow y
	sshow (AltOwnerDatabase x y) = "DATABASE " ++ sshow x ++ " OWNER TO " ++ sshow y
	sshow (AltOwnerDomain x y) = "DOMAIN " ++ sshow x ++ " OWNER TO " ++ sshow y
	sshow (AltOwnerFunction x y z) = "FUNCTION " ++ sshow x ++ ' ' : sshow y ++ " OWNER TO " ++ sshow z
	sshow (AltOwnerOperator x y z) = "OPERATOR " ++ sshow x ++ '(' : sshow y ++ ") OWNER TO " ++ sshow z
	sshow (AltOwnerOperatorClass x y z) = "OPERATOR CLASS " ++ sshow x ++ " USING " ++ sshow y ++ " OWNER TO " ++ sshow z
	sshow (AltOwnerSchema x y) = "SCHEMA " ++ sshow x ++ " OWNER TO " ++ sshow y
	sshow (AltOwnerType x y) = "TYPE " ++ sshow x ++ " OWNER TO " ++ sshow y
	sshow (AltOwnerTablespace x y) = "TABLESPACE" ++ sshow x ++ " OWNER TO " ++ sshow y

instance SQLShow AlterObjectSchemaStatement where
	sshow (AltAggregate x y z) = "AGGREGATE " ++ sshow x ++ '(' : sshow y ++ ") SET SCHEMA " ++ sshow z
	sshow (AltDomain x y) = "DOMAIN " ++ sshow x ++ " SET SCHEMA " ++ sshow y
	sshow (AltFunction x y z) = "FUNCTION " ++ sshow x ++ ' ' : sshow y ++ " SET SCHEMA " ++ sshow z
	sshow (AltSequence x y) = "SEQUENCE " ++ sshow x ++ " SET SCHEMA " ++ sshow y
	sshow (AltTable x y) = "TABLE " ++ sshow x ++ " SET SCHEMA " ++ sshow y
	sshow (AltType x y) = "TYPE " ++ sshow x ++ " SET SCHEMA " ++ sshow y

instance SQLShow RenameStatement where
	sshow (RenAggregate x y z) = "AGGREGATE " ++ sshow x ++ '(' : sshow y ++ ") RENAME TO " ++ sshow z
	sshow (RenConversion x y) = "CONVERSION " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenDatabase x y) = "DATABASE " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenFunction x y z) = "FUNCTION " ++ sshow x ++ ' ' : sshow y ++ " RENAME TO " ++ sshow z
	sshow (RenGroup x y) = "GROUP " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenLanguage x y) = "LANGUAGE " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenOperator x y z) = "OPERATOR CLASS " ++ sshow x ++ " USING " ++ sshow y ++ " RENAME TO " ++ sshow z
	sshow (RenSchema x y) = "SCHEMA " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenTable x y) = "TABLE " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenIndex x y) = "INDEX " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenTableRename x y z w) = "TABLE " ++ sshow x ++ " RENAME " ++ sshow y ++ ' ' : sshow z ++ " TO " ++ sshow w
	sshow (RenTrigger x y z) = "TRIGGER " ++ sshow x ++ " ON " ++ sshow y ++ " RENAME TO " ++ sshow z
	sshow (RenRole x y) = "ROLE " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenUser x y) = "USER " ++ sshow x ++ " RENAME TO " ++ sshow y
	sshow (RenTablespace x y) = "TABLESPACE " ++ sshow x ++ " RENAME TO " ++ sshow y

instance SQLShow CastContext where
	sshow AsImplicit = "AS IMPLICIT"
	sshow AsAssignment = "AS ASSIGNMENT"
	sshow NoCastContext = ""

instance SQLShow AnyOperator where
	sshow (AnyOpAll op) = sshow op
	sshow (AnyOpCol c op) = sshow c ++ '.' : sshow op

instance SQLShow OperArgTypes where
	sshow (SingleArg t) = sshow t
	sshow (BinArg t1 t2) = sshow t1 ++ ',' : sshow t2
	sshow (LeftUnaryArg t) = "NONE," ++ sshow t
	sshow (RightUnaryArg t) = sshow t ++ ",NONE"

instance SQLShow Star where
	sshow STAR = "*"

instance SQLShow FuncAs where
	sshow (SingleFuncAs s) = sshow s
	sshow (BinFuncAs s1 s2) = sshow s1 ++ ',' : sshow s2

instance SQLShow CreateFuncOptItem where
	sshow (FuncItemAs f) = "AS " ++ sshow f
	sshow (FuncItemLanguage either) = "LANGUAGE " ++ sshow either
	sshow (FuncItemCommon c) = sshow c

instance SQLShow CommonFuncOptItem where
	sshow CalledOnNullInput = "CALLED ON NULL INPUT"
	sshow ReturnsNullOnNummInput = "RETURNS NULL ON NULL INPUT"
	sshow Strict = "STRICT"
	sshow Immutable = "IMMUTABLE"
	sshow Stable = "STABLE"
	sshow Volatile = "VOLATILE"
	sshow ExternalSecurityDefiner = "EXTERNAL SECURITY DEFINER"
	sshow ExternalSecurityInvoker = "EXTERNAL SECURITY INVOKER"
	sshow SecurityDefiner = "SECURITY DEFINER"
	sshow SecurityInvoker = "SECURITY INVOKER"

instance SQLShow FuncType where
	sshow (FuncTypename t) = sshow t
	sshow (FuncAttrs t a) = sshow t ++ ' ' : sshow a ++ "% TYPE"
	sshow (FuncSetOf t a) = "SETOF " ++ sshow t ++ ' ' : sshow a ++ "% TYPE"

instance SQLShow FuncArgsList where
	sshow (FuncArgsLst l) = let list = foldr (\h r -> sshow h ++ ',' : r) [] l
				in if (null list) then list else init list

instance SQLShow FuncArgs where
	sshow (FArgs (Just x)) = '(' : sshow x ++ ")"
	sshow (FArgs Nothing) = "()"

instance SQLShow OptOrReplace where
	sshow OrReplace = "OR REPLACE"
	sshow NoOrReplace = ""

instance SQLShow IndexElem where
	sshow (IElem x y) = sshow x ++ ' ' : sshow y
	sshow (IFuncElem x y) = sshow x ++ ' ' : sshow y
	sshow (IExprElem x y) = '(' : sshow x ++ ") " ++ sshow y

instance SQLShow IndexParams where
	sshow (IndexParamsConst l) = let list = foldr (\h r -> sshow h ++ ',' : r) [] l
				     in if (null list) then list else init list

instance SQLShow OptGrantedBy where
	sshow (GrantedBy r) = "GRANTED BY " ++ sshow r
	sshow NoGrantedBy = ""

instance SQLShow OptGrantAdminOption where
	sshow WithAdminOption = "WITH ADMIN OPTION"
	sshow NoGrantAdmin = ""

instance SQLShow FunctionWithArgTypes where
	sshow (FuncArgType f fa) = sshow f ++ ' ' : sshow fa

instance SQLShow FunctionWithArgTypesList where
	sshow (FuncLst l) = let list = foldr (\h r -> sshow h ++ ',' : r) [] l
		            in if (null list) then list else init list

instance SQLShow OptGrantGrantOption where
	sshow WithGrantOption = "WITH GRANT OPTION"
	sshow NoGrant = ""

instance SQLShow PrivilegeTarget where
	sshow (PrivName n) = sshow n
	sshow (PrivTable n) = "TABLE " ++ sshow n
	sshow (PrivFunction f) = "FUNCTION " ++ sshow f
	sshow (PrivDatabase n) = "DATABASE " ++ sshow n
	sshow (PrivLanguage n) = "LANGUAGE " ++ sshow n
	sshow (PrivSchema n) = "SHCEMA " ++ sshow n
	sshow (PrivTablespace n) = "TABLESPACE " ++ sshow n

instance SQLShow Privilege where
	sshow PrivSelect = "SELECT"
	sshow PrivReferences = "REFERENCES"
	sshow PrivCreate = "CREATE"
	sshow (PrivColId cid) = sshow cid

instance SQLShow Privileges where
	sshow (Priv (Left plst)) = sshow plst
	sshow (Priv (Right Nothing)) = "ALL"
	sshow (Priv (Right (Just x))) = "ALL " ++ sshow x

instance SQLShow CommentOn where
	sshow (CmtOn x y z) = sshow x ++ ' ' : sshow y ++ " IS " ++ sshow z
	sshow (CmtOnAggregate x y z) = "AGGREGATE " ++ sshow x ++ " (" ++ sshow y ++ ") IS " ++ sshow z
	sshow (CmtOnFunction x y z) = "FUNCTION " ++ sshow x ++ ' ' : sshow y ++ " IS " ++ sshow z
	sshow (CmtOnOperator x y z) = "OPERATOR " ++ sshow x ++ " (" ++ sshow y ++ ") IS " ++ sshow z
	sshow (CmtOnConstraint x y z) = "CONSTRAINT " ++ sshow x ++ " ON " ++ sshow y ++ " IS " ++ sshow z
	sshow (CmtOnRule x (Just y) z) = "RULE " ++ sshow x ++ " ON " ++ sshow y ++ " IS " ++ sshow z
	sshow (CmtOnRule x Nothing z) = "RULE " ++ sshow x ++ " IS " ++ sshow z
	sshow (CmtOnTrigger x y z) = "TRIGGER " ++ sshow x ++ " ON " ++ sshow y ++ " IS " ++ sshow z
	sshow (CmtOnOperatorClass x y z) = "OPERATOR CLASS " ++ sshow z ++ " USING " ++ sshow y ++ " IS " ++ sshow z
	sshow (CmtOnLarge x y) = "LARGE OBJECT " ++ sshow x ++ " IS " ++ sshow y
	sshow (CmtOnCast x y z) = "CAST (" ++ sshow x ++ " AS " ++ sshow y ++ ") IS " ++ sshow z
	sshow (CmtOnLanguage x y z) = sshow x ++ " LANGUAGE " ++ sshow y ++ " IS " ++ sshow z

instance SQLShow Attrs where
	sshow (AttrsCont l) = foldr (\h r -> '.' : sshow h ++ r) [] l

instance SQLShow AnyNameList where
	sshow (AnyNameLst l) = let list = foldr (\h r -> sshow h ++ ',' : r) [] l
			       in if (null list) then list else init list

instance SQLShow DropType where
	sshow DTTable = "TABLE"
	sshow DTSequence = "SEQUENCE"
	sshow DTView = "VIEW"
	sshow DTIndex = "INDEX"
	sshow DTType = "TYPE"
	sshow DTDomain = "DOMAIN"
	sshow DTConversion = "CONVERSION"
	sshow DTSchema = "SCHEMA"

instance SQLShow OpClassItem where
	sshow (OpClassItemOperator i op Nothing chk) = "OPERATOR " ++ sshow i ++ ' ' : sshow op ++ ' ' : sshow chk
	sshow (OpClassItemOperator i op (Just x) chk) = "OPERATOR " ++ sshow i ++ ' ' : sshow op ++ " (" ++ sshow x ++ ") " ++ sshow chk
	sshow (OpClassItemFunction i f args) = "FUNCTION " ++ sshow i ++ ' ' : sshow f ++ ' ' : sshow args
	sshow (OpClassItemStorage t) = "STORAGE " ++ sshow t

instance SQLShow DefArg where
	sshow (DefArgFunc f) = sshow f
	sshow (DefArgOp op) = sshow op
	sshow (DefArgNum num) = sshow num
	sshow (DefArgS s) = sshow s

instance SQLShow DefElem where
	sshow (DefEqual x y) = sshow x ++ '=' : sshow y
	sshow (DefLabel x) = sshow x

instance SQLShow Definition where
	sshow (DefList l) = let list = foldr (\h r -> sshow h ++ ',' : r) [] l
			    in if (null list)
				then "()"
				else '(' : init list ++ ")"

instance SQLShow ConstraintTimeSpec where
	sshow (CnstrTimeSpec either) = "INITIALLY " ++ sshow either

instance SQLShow OptConstrFromTable where
	sshow (OptConstrFromTableConst (Just x)) = "FROM " ++ sshow x
	sshow (OptConstrFromTableConst Nothing) = ""

instance SQLShow TriggerForSpec where
	sshow (TriggerForSpecConst mb) = "FOR " ++ sshow mb

instance SQLShow TriggerEvents where
	sshow (OneTrigger t) = sshow t
	sshow (TwoTriggers t1 t2) = sshow t1 ++ " OR " ++ sshow t2
	sshow (ThreeTriggers t1 t2 t3) = sshow t1 ++ " OR " ++ sshow t2 ++ " OR " ++ sshow t3

instance SQLShow OptTableSpaceOwner where
	sshow (OptTSOwner (Just n)) = "OWNER " ++ sshow n
	sshow (OptTSOwner Nothing) = ""

instance SQLShow OptLancompiler where
	sshow (OptLanComp (Just x)) = "LANCOMPILER " ++ sshow x
	sshow (OptLanComp Nothing) = ""

instance SQLShow OptValidator where
	sshow (OptVal (Just x)) = "VALIDATOR " ++ sshow x
	sshow (OptVal Nothing) = ""

instance SQLShow FloatOnly where
	sshow (FloatConst s) = sshow s
	sshow (MinusFloatConst s) = '-' : sshow s

instance SQLShow OptSeqElem where
	sshow (SeqElemCache num) = "CACHE " ++ sshow num
	sshow SeqElemCycle = "CYCLE"
	sshow SeqElemNoCycle = "NO CYCLE"
	sshow (SeqElemIncrement by num) = "INCREMENT " ++ sshow by ++ ' ' : sshow num
	sshow (SeqElemMaxValue num) = "MAXVALUE " ++ sshow num
	sshow (SeqElemMinValue num) = "MINVALUE " ++ sshow num
	sshow SeqElemNoMaxValue = "NO MAXVALUE"
	sshow SeqElemNoMinValue = "NO MINVALUE"
	sshow (SeqElemStart with num) = "START " ++ sshow with ++ ' ' : sshow num
	sshow (SeqElemRestart with num) = "RESTART " ++ sshow with ++ ' ' : sshow num

instance SQLShow OptCreateAs where
	sshow (OptCreateAsConst (Just x)) = '(' : sshow x ++ ")"
	sshow (OptCreateAsConst Nothing) = ""

instance SQLShow WithOidsAs where
	sshow WOA_WithOidsAs = "WITH OIDS AS"
	sshow WOA_WithoutOidsAs = "WITHOUT OIDS AS"
	sshow WOA_As = "AS"

instance SQLShow OptConsTableSpace where
	sshow (OptConsTableSpaceConst (Just x)) = "USING INDEX TABLESPACE " ++ sshow x
	sshow (OptConsTableSpaceConst Nothing) = ""

instance SQLShow OptTableSpace where
	sshow (OptTableSpaceConst (Just x)) = "TABLESPACE " ++ sshow x
	sshow (OptTableSpaceConst Nothing) = ""

instance SQLShow OnCommitOptionAux where
	sshow OnCommitDrop = "ON COMMIT DROP"
	sshow OnCommitDeleteRows = "ON COMMIT DELETE ROWS"
	sshow OnCommitPreserveRows = "ON COMMIT PRESERVE ROWS"

instance SQLShow OptWithOids where
	sshow WithOids = "WITH OIDS"
	sshow WithoutOids = "WITHOUT OIDS"
	sshow NoOids = ""

instance SQLShow OptInherit where
	sshow (OptInherits (Just x)) = "INHERITS (" ++ sshow x ++ ")"
	sshow (OptInherits Nothing) = ""

instance SQLShow KeyAction where
	sshow KeyNoAction = "NO ACTION"
	sshow KeyRestrict = "RESTRICT"
	sshow KeyCascade = "CASCADE"
	sshow KeySetNull = "SET NULL"
	sshow KeySetDefault = "SET DEFAULT"

instance SQLShow KeyDelete where
	sshow (OnDelete ka) = "ON DELETE " ++ sshow ka

instance SQLShow KeyUpdate where
	sshow (OnUpdate ka) = "ON UPDATE " ++ sshow ka

instance SQLShow KeyMatchAux where
	sshow MatchFull = "MATCH FULL"
	sshow MatchPartial = "MATCH PARTIAL"
	sshow MatchSimple = "MATCH SIMPLE"

instance SQLShow OptColumnList where
	sshow (OptColList (Just x)) = '(' : sshow x ++ ")"
	sshow (OptColList Nothing) = ""

instance SQLShow ConstraintElem where
	sshow (CstrElemCheck aexpr) = "CHECK (" ++ sshow aexpr ++ ")"
	sshow (CstrElemUnique cl cts) = "UNIQUE (" ++ sshow cl ++ ") " ++ sshow cts
	sshow (CstrElemPrimaryKey cl cts) = "PRIMARY KEY (" ++ sshow cl ++ ") " ++ sshow cts
	sshow (CstrElemForeignKey cl qn ocl km ka cas) = "FOREIGN KEY (" ++ sshow cl ++ ") REFERENCES " ++ sshow qn ++ ' ' : sshow ocl ++ ' ' : sshow km ++ ' ' : sshow ka ++ ' ' : sshow cas

instance SQLShow TableConstraint where
	sshow (TConstraint (Just n) celem) = "CONSTRAINT " ++ sshow n ++ ' ' : sshow celem
	sshow (TConstraint Nothing celem) = sshow celem

instance SQLShow LikeIncludingDefaultsAux where
	sshow LikeIncludingDefaults = "INCLUDING DEFAULTS"
	sshow LikeExcludingDefaults = "EXCLUDING DEFAULTS"

instance SQLShow TableLikeClause where
	sshow (TLike x y) = "LIKE " ++ sshow x ++ ' ' : sshow y

instance SQLShow ConstraintAttr where
	sshow CstrAttrDeferrable = "DEFERRABLE"
	sshow CstrAttrNotDeferrable = "NOT DEFERRABLE"
	sshow CstrAttrInitiallyDeferred = "INITIALLY DEFERRED"
	sshow CstrAttrInitiallyImmediate = "INITIALLY IMMEDIATE"

instance SQLShow ColumnDef where
	sshow (CDef cid tn cqlist) = sshow cid ++ ' ' : sshow tn ++ ' ' : sshow cqlist
		
instance SQLShow OptTempAux where
	sshow TmpTemporary = "TEMPORARY"
	sshow TmpTemp = "TEMP"
	sshow TmpLocalTemporary = "LOCAL TEMPORARY"
	sshow TmpLocalTemp = "LOCAL TEMP"
	sshow TmpGlobalTemporary = "GLOBAL TEMPORARY"
	sshow TmpGlobalTemp = "GLOBAL TEMP"

instance SQLShow CopyOptItem where
	sshow CpItemBinary = "BINARY"
	sshow CpItemOids = "OIDS"
	sshow (CpItemDelimiter as s) = "DELIMITER " ++ sshow as ++ ' ' : sshow s
	sshow (CpItemNull as s) = "NULL " ++ sshow as ++ ' ' : sshow s
	sshow CpItemCsv = "CSV"
	sshow CpItemHeader = "HEADER"
	sshow (CpItemQuote as s) = "QUOTE " ++ sshow as ++ ' ' : sshow s
	sshow (CpItemEscape as s) = "ESCAPE " ++ sshow as ++ ' ' : sshow s
	sshow (CpItemForceQuote cl) = "FORCE QUOTE " ++ sshow cl
	sshow (CpItemForceNotNull cl) = "FORCE NOT NULL " ++ sshow cl

instance SQLShow AlterColumnDefault where
	sshow (AlterColumnSetDefault aexpr) = "SET DEFAULT " ++ sshow aexpr
	sshow AlterColumnDropDefault = "DROP DEFAULT"

instance SQLShow AlterRelCmd where
	sshow (AlterRelCmdOwnerTo rid) = "OWNER TO " ++ sshow rid
	sshow (AlterRelCmdSetTablespace n) = "SET TABLESPACE " ++ sshow n

instance SQLShow AlterTableCmd where
	sshow (AlterTableAdd either) = "ADD " ++ sshow either
	sshow (AlterTableAlter col colid oneof4) = "ALTER " ++ sshow col ++ ' ' : sshow colid ++ ' ' : sshow oneof4
	sshow (AlterTableDrop either behavior) = "DROP " ++ sshow either ++ ' ' : sshow behavior
	sshow (AlterTableCreateToastTable) = "CREATE TOAST TABLE"
	sshow (AlterTableClusterOn n) = "CLUSTER ON " ++ sshow n
	sshow (AlterTableSetWithout either) = "SET WITHOUT " ++ sshow either
	sshow (AlterTableEnableTrigger oneof3) = "ENABLE TRIGGER " ++ sshow oneof3
	sshow (AlterTableDisableTrigger oneof3) = "DISABLE TRIGGER " ++ sshow oneof3
	sshow (AlterTableRelCmd rel) = sshow rel

instance (SQLShow a,SQLShow b) => SQLShow (a,b) where
	sshow (x,y) = sshow x ++ ' ' : sshow y

instance (SQLShow a,SQLShow b,SQLShow c) => SQLShow (a,b,c) where
	sshow (x,y,z) = sshow x ++ ' ' : sshow y ++ ' ' : sshow z

instance SQLShow ZoneValue where
	sshow (ZoneSconst s) = sshow s
	sshow (ZoneIdent) = "IDENT"
	sshow (ZoneConstInterval c Nothing s int) = sshow c ++ ' ' : sshow s ++ ' ' : sshow int
	sshow (ZoneConstInterval c (Just x) s int) = sshow c ++ '(' : sshow x ++ ')' : sshow s ++ ' ' : sshow int
	sshow (ZoneNum num) = sshow num
	sshow ZoneDefault = "DEFAULT"
	sshow ZoneLocal = "LOCAL"

instance SQLShow OptBoolean where
	sshow OptBoolTrue = "TRUE"
	sshow OptBoolFalse = "FALSE"
	sshow OptBoolOn = "ON"
	sshow OptBoolOff = "OFF"

instance SQLShow IsoLevel where
	sshow IsoReadUncommited = "READ UNCOMMITED"
	sshow IsoReadCommited = "READ COMMITED"
	sshow IsoRepeatableRead = "REPEATABLE READ"
	sshow IsoSerializable = "SERIALIZABLE"

instance (SQLShow a,SQLShow b,SQLShow c) => SQLShow (OneOf3 a b c) where
	sshow (OneOf3 x) = sshow x
	sshow (TwoOf3 x) = sshow x
	sshow (ThreeOf3 x) = sshow x

instance SQLShow OptRoleElem where
	sshow (RolePassword sconst) = "PASSWORD " ++ sshow sconst
	sshow (RoleEncryptedPassword sconst) = "ENCRYPTED PASSWORD " ++ sshow sconst
	sshow (RoleUnencryptedPassword sconst) = "UNENCRYPTED PASSWORD " ++ sshow sconst
	sshow RoleSuperUser = "SUPERUSER"
	sshow RoleNoSuperUser = "NOSUPERUSER"
	sshow RoleInherit = "INHERIT"
	sshow RoleNoInherit = "NOINHERIT"
	sshow RoleCreateDB = "CREATEDB"
	sshow RoleNoCreateDB = "NOCREATEDB"
	sshow RoleCreateRole = "CREATEROLE"
	sshow RoleNoCreateRole = "NOCREATEROLE"
	sshow RoleCreateUser = "CREATEUSER"
	sshow RoleNoCreateUser = "NOCREATEUSER"
	sshow RoleLogin = "LOGIN"
	sshow RoleNoLogin = "NOLOGIN"
	sshow (RoleConnectionLimit signedIconst) = "CONNECTION LIMIT " ++ sshow signedIconst
	sshow (RoleValidUntil sconst) = "VALID UNTIL " ++ sshow sconst
	sshow (RoleUser n) = "USER " ++ sshow n
	sshow (RoleSysId iconst) = "SYSID " ++ sshow iconst
	sshow (RoleAdmin n) = "ADMIN " ++ sshow n
	sshow (RoleRole n) = "ROLE " ++ sshow n
	sshow (RoleInRole n) = "INROLE " ++ sshow n
	sshow (RoleInGroup n) = "INGROUP " ++ sshow n

instance SQLShow SetRest where
	sshow (SetTo vn either) = sshow vn ++ " TO " ++ sshow either
	sshow (SetEqual vn either) = sshow vn ++ " = " ++ sshow either
	sshow (SetTimeZone tz) = "TIME ZONE " ++ sshow tz
	sshow (SetTransaction (Left x)) = "TRANSACTION " ++ sshow x
	sshow (SetTransaction (Right x)) = "SESSION CHARACTERISTICS AS TRANSACTION " ++ sshow x
	sshow (SetName enc) = "NAMES " ++ sshow enc
	sshow (SetRole either) = "ROLE " ++ sshow either
	sshow (SetSessionAuthorization either) = "SESSION AUTHORIZATION " ++ sshow either

instance SQLShow With where
	sshow WITH = "WITH"

instance (SQLShow a,SQLShow b) => SQLShow (Either a b) where
	sshow (Left x) = sshow x
	sshow (Right x) = sshow x

instance SQLShow Statement where
	sshow (AlterDatabaseStmt x y z) = "ALTER DATABASE " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (AlterDatabaseSetStmt x (Left y)) = "ALTER DATABASE " ++ sshow x ++ " SET " ++ sshow y
	sshow (AlterDatabaseSetStmt x (Right y)) = "ALTER DATABASE " ++ sshow x ++ ' ' : sshow y
	sshow (AlterDomainStmt x (OneOf5 y)) = "ALTER DOMAIN " ++ sshow x ++ ' ' : sshow y
	sshow (AlterDomainStmt x (TwoOf5 ())) = "ALTER DOMAIN " ++ sshow x ++ " DROP NOT NULL"
	sshow (AlterDomainStmt x (ThreeOf5 ())) = "ALTER DOMAIN " ++ sshow x ++ " SET NOT NULL"
	sshow (AlterDomainStmt x (FourOf5 y)) = "ALTER DOMAIN " ++ sshow x ++ " ADD " ++ sshow y
	sshow (AlterDomainStmt x (FiveOf5 (y,z))) = "ALTER DOMAIN " ++ sshow x ++ " DROP CONSTRAINT " ++ sshow y ++ ' ' : sshow z
	sshow (AlterFunctionStmt x y z) = "ALTER FUNCTION " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (AlterGroupStmt rid ag n) = "ALTER GROUP " ++ sshow rid ++ ' ' : sshow ag ++ " USER " ++ sshow n
	sshow (AlterObjectSchemaStmt x) = "ALTER " ++ sshow x
	sshow (AlterOwnerStmt x) = "ALTER " ++ sshow x
	sshow (AlterSeqStmt qn seqlist) = "ALTER SEQUENCE " ++ sshow qn ++ ' ' : sshow seqlist
	sshow (AlterTableStmt rexpr (Left x)) = "ALTER TABLE " ++ sshow rexpr ++ ' ' : sshow x
	sshow (AlterTableStmt rexpr (Right x)) = "ALTER INDEX " ++ sshow rexpr ++ ' ' : sshow x
	sshow (AlterRoleSetStmt rid (Left set)) = "ALTER ROLE " ++ sshow rid ++ " SET " ++ sshow set
	sshow (AlterRoleSetStmt rid (Right x)) = "ALTER ROLE " ++ sshow rid ++ ' ' : sshow x
	sshow (AlterRoleStmt rid with rlist) = "ALTER ROLE " ++ sshow rid ++ ' ' : sshow with ++ ' ' : sshow rlist
	sshow (AlterUserSetStmt rid (Left set)) = "ALTER USER " ++ sshow rid ++ " SET " ++ sshow set
	sshow (AlterUserSetStmt rid (Right x)) = "ALTER USER " ++ sshow rid ++ ' ' : sshow x
	sshow (AlterUserStmt x y z) = "ALTER USER " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (AnalyzeStmt x y mb) = sshow x ++ ' ' : sshow y ++ ' ' : sshow mb
	sshow CheckPointStmt = "CHECKPOINT"
	sshow (ClosePortalStmt n) = "CLOSE " ++ sshow n
	sshow (ClusterStmt Nothing) = "CLUSTER"
	sshow (ClusterStmt (Just (Nothing,x))) = "CLUSTER " ++ sshow x
	sshow (ClusterStmt (Just (Just y,x))) = "CLUSTER " ++ sshow y ++ " ON " ++ sshow x
	sshow (CommentStmt cmt) = "COMMENT ON " ++ sshow cmt
	sshow (ConstraintsSetStmt setlist setmode) = "SET CONSTRAINTS " ++ sshow setlist ++ ' ' : sshow setmode
	sshow (CopyStmt x1 x2 x3 x4 x5 x6 x7 x8 x9) = "COPY " ++ sshow x1 ++ ' ' : sshow x2 ++ ' ' : sshow x3 ++ ' ' : sshow x4 ++ ' ' : sshow x5 ++ ' ' : sshow x6 ++ ' ' : sshow x7 ++ ' ' : sshow x8 ++ ' ' : sshow x9
	sshow (CreateAsStmt a b c d e) = "CREATE " ++ sshow a ++ " TABLE " ++ sshow b ++ ' ' : sshow c ++ ' ' : sshow d ++ ' ' : sshow e
	sshow (CreateAssertStmt n expr c) = "CREATE ASSERTION " ++ sshow n ++ " CHECK (" ++ sshow expr ++ ") " ++ sshow c
	sshow (CreateCastStmt a b (Just c) d) = "CREATE CAST (" ++ sshow a ++ " AS " ++ sshow b ++ ") WITH FUNCTION " ++ sshow c ++ ' ' : sshow d
	sshow (CreateCastStmt a b Nothing d) = "CREATE CAST (" ++ sshow a ++ " AS " ++ sshow b ++ ") WITHOUT FUNCTION " ++ sshow d
	sshow (CreateConversionStmt x y z w s) = "CREATE " ++ sshow x ++ " CONVERSION " ++ sshow y ++ " FOR " ++ sshow z ++ " TO " ++ sshow w ++ " FROM " ++ sshow s
	sshow (CreateDomainStmt x y z w) = "CREATE DOMAIN " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z ++ ' ' : sshow w
	sshow (CreateFunctionStmt a b c (Just d) e f) = "CREATE " ++ sshow a ++ " FUNCTION " ++ sshow b ++ ' ' : sshow c ++ " RETURNS " ++ sshow d ++ ' ' : sshow e ++ ' ' : sshow f
	sshow (CreateFunctionStmt a b c Nothing e f) = "CREATE " ++ sshow a ++ " FUNCTION " ++ sshow b ++ ' ' : sshow c ++ ' ' : sshow e ++ ' ' : sshow f
	sshow (CreateGroupStmt rid with rlist) = "CREATE GROUP " ++ sshow rid ++ ' ' : sshow with ++ ' ' : sshow rlist
	sshow (CreateOpClassStmt a b c d e) = "CREATE OPERATOR CLASS " ++ sshow a ++ ' ' : sshow b ++ " FOR TYPE " ++ sshow c ++ " USING " ++ sshow d ++ " AS " ++ sshow e
	sshow (CreatePLangStmt trusted proc either Nothing) = "CREATE " ++ sshow trusted ++ ' ' : sshow proc ++ " LANGUAGE " ++ sshow either
	sshow (CreatePLangStmt trusted proc either (Just (x,y,z))) = "CREATE " ++ sshow trusted ++ ' ' : sshow proc ++ " LANGUAGE " ++ sshow either ++ " HANDLER " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (CreateSchemaStmt (Just schemaname) rid schemalist) = "CREATE SCHEMA " ++ sshow schemaname ++ " AUTHORIZATION " ++ sshow rid ++ ' ' : sshow schemalist
	sshow (CreateSchemaStmt Nothing rid schemaList) = "CREATE SCHEMA " ++ ' ' : sshow rid ++ ' ' : sshow schemaList

	sshow (CreateSeqStmt x y z) = "CREATE " ++ sshow x ++ " SEQUENCE " ++ sshow y ++ ' ' : sshow z
	sshow (CreateStmt tmp qn (Just mb) tlist Nothing x y z) = "CREATE " ++ sshow tmp ++ " TABLE " ++ sshow qn ++ " OF " ++ sshow mb ++ " (" ++ sshow tlist ++ ") " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (CreateStmt tmp qn Nothing tlist (Just mb) x y z) = "CREATE " ++ sshow tmp ++ " TABLE " ++ sshow qn ++ " (" ++ sshow tlist ++ ") " ++ sshow mb ++ ' ' : sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (CreateTableSpaceStmt n opt s) = "CREATE TABLESPACE " ++ sshow n ++ ' ' : sshow opt ++ " LOCATION " ++ sshow s
	sshow (CreateTrigStmt n (Just ta) te qn (Left ts) func tfuncargs) = "CREATE TRIGGER " ++ sshow n ++ ' ' : sshow ta ++ ' ' : sshow te ++ " ON " ++ sshow qn ++ ' ' : sshow ts ++ " EXECUTE PROCEDURE " ++ sshow func ++ ' ' : " (" ++ sshow tfuncargs ++ ")"
	sshow (CreateTrigStmt n Nothing te qn (Right (x,y)) func tfuncargs) = "CREATE CONSTRAINT TRIGGER " ++ sshow n ++ " AFTER " ++ sshow te ++ " ON " ++ sshow qn ++ ' ' : sshow x ++ ' ' : sshow y ++ " FOR EACH ROW EXECUTE PROCEDURE " ++ sshow func ++ " (" ++ sshow tfuncargs ++ ")"
	sshow (CreateRoleStmt rid with rlist) = "CREATE ROLE " ++ sshow rid ++ ' ' : sshow with ++ ' ' : sshow rlist
	sshow (CreateUserStmt rid with rlist) = "CREATE USER " ++ sshow rid ++ ' ' : sshow with ++ ' ' : sshow rlist
	sshow (CreatedbStmt x y z) = "CREATE DATABASE " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (DeallocateStmt (Left y)) = "DEALLOCATE " ++ sshow y
	sshow (DeallocateStmt (Right y)) = "DEALLOCATE PREPARE " ++ sshow y
	sshow (DeclareCursorStmt a b c d) = "DECLARE " ++ sshow a ++ ' ' : sshow b ++ " CURSOR " ++ sshow c ++ " FOR " ++ sshow d
	sshow (DefineStmt (OneOf3 (f,d))) = "CREATE AGGREGATE " ++ sshow f ++ ' ' : sshow d
	sshow (DefineStmt (TwoOf3 (op,d))) = "CREATE OPERATOR " ++ sshow op ++ ' ' : sshow d
	sshow (DefineStmt (ThreeOf3 (n,Left d))) = "CREATE TYPE " ++ sshow n ++ ' ' : sshow d
	sshow (DefineStmt (ThreeOf3 (n,Right tfunc))) = "CREATE TYPE " ++ sshow n ++ " AS (" ++ sshow tfunc ++ ")"
	sshow (DeleteStmt x y z) = "DELETE FROM " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (DropAssertStmt n b) = "DROP ASSERTION " ++ sshow n ++ ' ' : sshow b
	sshow (DropCastStmt x y z) = "DROP CAST (" ++ sshow x ++ " AS " ++ sshow y ++ ')' : sshow z
	sshow (DropGroupStmt n) = "DROP GROUP " ++ sshow n
	sshow (DropOpClassStmt n am b) = "DROP OPERATOR CLASS " ++ sshow n ++ " USING " ++ sshow am ++ ' ' : sshow b
	sshow (DropPLangStmt x y z) = "DROP " ++ sshow x ++ " LANGUAGE " ++ sshow y ++ ' ' : sshow z
	sshow (DropRuleStmt x y z) = "DROP RULE " ++ sshow x ++ " ON " ++ sshow y ++ ' ' : sshow z
	sshow (DropStmt x y z) = "DROP " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (DropTableSpaceStmt n) = "DROP TABLESPACE " ++ sshow n
	sshow (DropTrigStmt n qn b) = "DROP TRIGGER " ++ sshow n ++ " ON " ++ sshow qn ++ ' ' : sshow b
	sshow (DropRoleStmt n) = "DROP ROLE " ++ sshow n
	sshow (DropUserStmt n) = "DROP USER " ++ sshow n
	sshow (DropdbStmt x) = "DROP DATABASE " ++ sshow x
	sshow (ExecuteStmt x) = sshow x
	sshow (ExplainStmt x y z) = "EXPLAIN " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (FetchStmt either mb n) = sshow either ++ ' ' : sshow mb ++ ' ' : sshow n
	sshow (GrantStmt a b c d) = "GRANT " ++ sshow a ++ " ON " ++ sshow b ++ " TO " ++ sshow c ++ ' ' : sshow d
	sshow (GrantRoleStmt a b c d) = "GRANT " ++ sshow a ++ " TO " ++ sshow b ++ ' ' : sshow c ++ ' ' : sshow d
	sshow (IndexStmt a b c d e f g) = "CREATE " ++ sshow a ++ " INDEX " ++ sshow b ++ " ON " ++ sshow c ++ ' ' : sshow d ++ " (" ++ sshow e ++ ") " ++ sshow f ++ ' ' : sshow g
	sshow (InsertStmt x y) = "INSERT INTO " ++ sshow x ++ ' ' : sshow y
	sshow (ListenStmt n) = "LISTEN " ++ sshow n
	sshow (LoadStmt fn ) = "LOAD " ++ sshow fn
	sshow (LockStmt a b c d) = "LOCK " ++ sshow a ++ ' ' : sshow b ++ ' ' : sshow c ++ ' ' : sshow d
	sshow (NotifyStmt n) = "NOTIFY " ++ sshow n
	sshow (PrepareStmt x y z) = "PREPARE " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (ReindexStmt either x) = "REINDEX " ++ sshow either ++ ' ' : sshow x
	sshow (RemoveAggrStmt x y z) = "DROP AGGREGATE " ++ sshow x ++ " (" ++ sshow y ++ ") " ++ sshow z
	sshow (RemoveFuncStmt x y z) = "DROP FUNCTION " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z
	sshow (RemoveOperStmt x y z) = "DROP OPERATOR " ++ sshow x ++ " (" ++ sshow y ++ ") " ++ sshow z
	sshow (RenameStmt ren) = "ALTER " ++ sshow ren
	sshow (RevokeStmt (Left (ON,a,b,c,d))) = "REVOKE " ++ sshow a ++ " ON " ++ sshow b ++ " FROM " ++ sshow c ++ ' ' : sshow d
	sshow (RevokeStmt (Right (FOR,a,b,c,d))) = "REVOKE GRANT OPTION FOR " ++ sshow a ++ " ON " ++ sshow b ++ " FROM " ++ sshow c ++ ' ' : sshow d
	sshow (RevokeRoleStmt (Left (a,b,c,d))) = "REVOKE " ++ sshow a ++ " FROM " ++ sshow b ++ ' ' : sshow c ++ ' ' : sshow d
	sshow (RevokeRoleStmt (Right (a,b,c,d))) = "REVOKE ADMIN OPTION FOR " ++ sshow a ++ " FROM " ++ sshow b ++ ' ' : sshow c ++ ' ' : sshow d
	sshow (RuleStmt a b c d e f g) = "CREATE " ++ sshow a ++ " RULE " ++ sshow b ++ " AS ON " ++ sshow c ++ " TO " ++ sshow d ++ ' ' : sshow e ++ " DO " ++ sshow f ++ ' ' : sshow g
	sshow (SelectStmt either) = sshow either
	sshow (TransactionStmt t) = sshow t
	sshow (TruncateStmt x y) = "TRUNCATE " ++ sshow x ++ ' ' : sshow y
	sshow (UnlistenStmt either) = "UNLISTEN " ++ sshow either
	sshow (UpdateStmt a b c d) = "UPDATE " ++ sshow a ++ " SET " ++ sshow b ++ ' ' : sshow c ++ ' ' : sshow d
	sshow (VacuumStmt x y z mb) = "VACUUM " ++ sshow x ++ ' ' : sshow y ++ ' ' : sshow z ++ ' ' : sshow mb
	sshow (VariableResetStmt x) = "SHOW " ++ sshow x
	sshow (VariableSetStmt mb setrest) = "SET " ++ sshow mb ++ ' ' : sshow setrest
	sshow (VariableShowStmt x) = "SHOW " ++ sshow x
	sshow (ViewStmt (Just REPLACE) a b c d) = "CREATE OR REPLACE " ++ sshow a ++ " VIEW " ++ sshow b ++ ' ' : sshow c ++ " AS " ++ sshow d
	sshow (ViewStmt Nothing a b c d) = "CREATE " ++ sshow a ++ " VIEW " ++ sshow b ++ ' ' : sshow c ++ " AS " ++ sshow d

instance SQLShow ASTPostgreSQL where
	sshow (AST l) = foldr (\h r -> (sshow h) ++ (";\n" ++ r)) [] l

instance SQLShow a => SQLShow (Maybe a) where
	sshow (Just x) = sshow x
	sshow Nothing = ""

instance SQLShow Char where
	sshow  = (:[])

instance SQLShow a => SQLShow [a] where
	sshow = foldr (\h r -> (sshow h) ++ r) []

