{-# OPTIONS -fglasgow-exts #-}

-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt, alcino@di.uminho.pt
--
-- Example: internet movie database.
--
-----------------------------------------------------------------------------

module Main where

import Data.Transform.Coupled
import qualified Data.Generics as SYB
import Data.Char

-----------------------------------------------------------------------------

data Imdb      = Imdb [Movie] [Actor] deriving (SYB.Typeable, SYB.Data, Show)
data Movie     = Movie Year Title Director [Review] [BoxOffice] deriving (SYB.Typeable, SYB.Data, Show)
data Year      = Year Int deriving (SYB.Typeable, SYB.Data, Show)
data Title     = Title String deriving (SYB.Typeable, SYB.Data, Show)
data Director  = Director String deriving (SYB.Typeable, SYB.Data, Show)
data Review    = Review String deriving (SYB.Typeable, SYB.Data, Show)
data BoxOffice = BoxOffice Country Value deriving (SYB.Typeable, SYB.Data, Show)
data Country   = Country String deriving (SYB.Typeable, SYB.Data, Show)
data Value     = Value Int deriving (SYB.Typeable, SYB.Data, Show)
data Actor     = Actor Name [Played] deriving (SYB.Typeable, SYB.Data, Show)
data Name      = Name String deriving (SYB.Typeable, SYB.Data, Show)
data Played    = Played Year Title Role [Award] deriving (SYB.Typeable, SYB.Data, Show)
data Role      = Role String deriving (SYB.Typeable, SYB.Data, Show)
data Award     = Award String deriving (SYB.Typeable, SYB.Data, Show)

pulpf = Movie (Year 94) (Title "Pulp Fiction") (Director "Quentin Tarentino") revs boxs
    where revs = [ Review "If you think Pulp Fiction is brilliant, you're wrong. It's more than that. It's a milestone in the history of film making.",
		   Review "To put this in context, I am 34 years old and I have to say that this is the best film I have seen without doubt and I don't expect it will be beaten as far as I am concerned." ]
          boxs = [ BoxOffice (Country "USA") (Value 107928762),
		   BoxOffice (Country "UK") (Value 10734320),
		   BoxOffice (Country "Spain") (Value 1117000000) ]

killb1 = Movie (Year 2003) (Title "Kill Bill 1") (Director "Quentin Tarentino") revs boxs
    where revs = [ Review "This film contains extreme violence and sometimes strange dialog coupled with some pretty good acting and directing. If you're not a fan of Tarantino's films, you should pass on this one because it is doesn't stray to far from his other stuff." ]
          boxs = [ BoxOffice (Country "USA") (Value 70098138),
		   BoxOffice (Country "UK") (Value 11453044),
		   BoxOffice (Country "Italy") (Value 5306263) ]

killb2 = Movie (Year 2004) (Title "Kill Bill 2") (Director "Quentin Tarentino") [] []

uma = Actor (Name "Uma Thurman") pld
    where pld = [ Played (Year 94) (Title "Pulp Fiction") (Role "Mia Wallace") [ Award "MTV Movie Award: Best Dance Sequence" ], 
		  Played (Year 2003) (Title "Kill Bill 1") (Role "The Bride") [ Award "Saturn Award: Best Actress", Award "Empire Award: Best Actress", Award "MTV Movie Award: Best Female Performance", Award "MTV Movie Award: Best Fight" ] ]

travolta = Actor (Name "John Travolta") pld
    where pld = [ Played (Year 94) (Title "Pulp Fiction") (Role "Vincent Vega") [ Award "David: Best Foreign Actor", Award "ALFS Award: Actor of the Year", Award "LAFCA Award: Best Actor", Award "MTV Movie Award: Best Dance Sequence" ] ]

imdb1 = Imdb [ pulpf, killb1, killb2 ] [ uma, travolta ]
imdbs n = Imdb (replicate n pulpf) (replicate n uma)

-----------------------------------------------------------------------------

sybtrunc :: SYB.Data a => a -> a
sybtrunc = SYB.everywhere (SYB.mkT (\(Review r) -> Review (take 100 r)))

sybcount :: SYB.Data a => a -> Int
sybcount = SYB.everything (+) (SYB.mkQ 0 (\(Review r) -> length r))

sybtrans :: SYB.Data a => a -> Int
sybtrans = sybcount . sybtrunc

-----------------------------------------------------------------------------
 
instance Typeable Award where
    typeof = Data "Award" (EP to from) String
	where to (Award s) = s
	      from = Award

instance Typeable Role where
    typeof = Data "Role" (EP to from) String
	where to (Role s) = s
	      from = Role

instance Typeable Name where
    typeof = Data "Name" (EP to from) String
	where to (Name s) = s
	      from = Name

instance Typeable Country where
    typeof = Data "Country" (EP to from) String
	where to (Country s) = s
	      from = Country

instance Typeable Review where
    typeof = Data "Review" (EP to from) String
	where to (Review s) = s
	      from = Review

instance Typeable Director where
    typeof = Data "Director" (EP to from) String
	where to (Director s) = s
	      from = Director

instance Typeable Title where
    typeof = Data "Title" (EP to from) String
	where to (Title s) = s
	      from = Title

instance Typeable Year where
    typeof = Data "Year" (EP to from) Int
	where to (Year s) = s
	      from = Year

instance Typeable Value where
    typeof = Data "Value" (EP to from) Int
	where to (Value s) = s
	      from = Value

instance Typeable Played where
    typeof = Data "Played" (EP to from) (Prod typeof (Prod typeof (Prod typeof (List typeof))))
	where to (Played y t r as) = (y,(t,(r,as)))
	      from (y,(t,(r,as))) = Played y t r as

instance Typeable Actor where
    typeof = Data "Actor" (EP to from) (Prod typeof (List typeof))
	where to (Actor n ps) = (n,ps)
	      from (n,ps) = Actor n ps

instance Typeable BoxOffice where
    typeof = Data "BoxOffice" (EP to from) (Prod typeof typeof)
	where to (BoxOffice c v) = (c,v)
	      from (c,v) = BoxOffice c v

instance Typeable Movie where
    typeof = Data "Movie" (EP to from) (Prod typeof (Prod typeof (Prod typeof (Prod (List typeof) (List typeof)))))
	where to (Movie y t d rs bs) = (y,(t,(d,(rs,bs))))
	      from (y,(t,(d,(rs,bs)))) = Movie y t d rs bs

instance Typeable Imdb where
    typeof = Data "Imdb" (EP to from) (Prod (List typeof) (List typeof))
	where to (Imdb ms as) = (ms,as)
	      from (ms,as) = Imdb ms as

-----------------------------------------------------------------------------

take100 (Review r) = Review (take 100 r)
size (Review r) = length r

trunc :: Imdb -> Imdb
trunc = eval typeof $ APPLY typeof (EVERYWHERE (MKT typeof (FUN "take100" take100)))

count :: Imdb -> Int
count = eval typeof $ APPLYQ typeof (EVERYTHING mint (MKQ mint typeof (FUN "size" size)))

imdb f g (Imdb m a) = Imdb (f m) (g a)
movie f g h i j (Movie y t d r b) = Movie (f y) (g t) (h d) (i r) (j b)

trunc' :: Imdb -> Imdb
trunc' = imdb (map (movie id id id (map take100) id)) id

movies :: Imdb -> [Movie]
movies (Imdb m a) = m

reviews :: Movie -> [Review]
reviews (Movie y t d r b) = r

count' :: Imdb -> Int
count' = sum . (map (sum . map size . reviews)) . movies

-----------------------------------------------------------------------------


t1 :: PF (Imdb -> Imdb)
t1 = APPLY typeof (EVERYWHERE (MKT typeof (FUN "take100" take100)))

t2 :: PF (Review -> Review)
t2 = APPLY typeof (EVERYWHERE (MKT typeof (FUN "take100" take100)))

t3 :: PF (Imdb -> Imdb)
t3 = APPLY typeof (EVERYWHERE (MKT typeof (APPLY (typeof::Type Movie) ((EVERYWHERE (MKT typeof (FUN "take 100" take100)))))))

t4 :: PF (Imdb -> Imdb)
t4 = APPLY typeof (EVERYWHERE (MKT typeof (FUN "y2k" aux)))
    where aux (Year y) | y<100 = Year (1900+y)
		       | otherwise = Year y

upper :: Title -> Title
upper (Title t) = Title (map toUpper t)

t5 :: PF (Movie -> Movie)
t5 = APPLY typeof (EVERYWHERE (MKT typeof (FUN "upper" upper)))

t6 :: PF (Actor -> Actor)
t6 = APPLY typeof (EVERYWHERE (MKT typeof (FUN "upper" upper)))

t7 :: PF (Imdb -> Imdb)
t7 = APPLY typeof (EVERYWHERE (SEQ (MKT typeof t5) (MKT typeof t6)))

t8 :: PF (Imdb -> Imdb)
t8 = APPLY typeof (EVERYWHERE (MKT typeof aux))
     where aux :: PF (Actor -> Actor)
	   aux = APPLY typeof (EVERYWHERE (MKT typeof (FUN "upper" upper)))
	   upper :: Award -> Award
	   upper (Award t) = Award (map toUpper t)

sybt t = reduce sybt2pf typeof t

sybtio t = rewrite typeof True sybt2pf t

gsybt :: Typeable a => PF (a -> a) -> IO (PF (a -> a))
gsybt t = rewrite typeof True pf2t (APPLY typeof (MKT typeof t))

q1 :: PF (Imdb -> Int)
q1 = APPLYQ typeof (EVERYTHING mint (MKQ mint typeof (FUN "size" size)))

q2 :: PF (Actor -> Int)
q2 = APPLYQ typeof (EVERYTHING mint (MKQ mint typeof (FUN "size" size)))

sybq t = reduce sybq2pf typeof t

sybqio t = rewrite typeof True sybq2pf t

gsybq :: (Typeable a, Typeable b, Result b) => PF (a -> b) -> IO (PF (a -> b))
gsybq t = rewrite typeof True pf2q (APPLYQ typeof (MKQ monoid typeof t))

--sybtrans' = eval typeof sybcount' . eval typeof sybtrunc'
--pftrans = eval typeof pfcount . eval typeof pftrunc

x1 :: PF (GQ [Dynamic])
x1 = CHILD :/: NAME "Imdb" :/: CHILD :/: NAME "Movie" :/: CHILD :/: NAME "Director"

pfx1 :: PF ([Imdb] -> [Dynamic])
pfx1 = optimizexp typeof x1

x2 :: PF ([Imdb] -> [Dynamic])
x2 = APPLYQ typeof (DESCorSELF :/: CHILD :/: NAME "Movie" :/: CHILD :/: NAME "Director")

x3 :: PF ([Imdb] -> [Dynamic])
x3 = APPLYQ typeof (DESCorSELF :/: CHILD :/: NAME "Director")

x4 :: PF ([Imdb] -> [Dynamic])
x4 = APPLYQ typeof (DESCorSELF :?: (CHILD :/: NAME "Director" :/: NONEMPTY))

x5 :: PF (Imdb -> [Dynamic])
x5 = APPLYQ typeof (DESCorSELF :/: (NAME "Movie" :?: (DESCorSELF :/: NAME "Review" :/: NONEMPTY)))

x6 :: PF ([Imdb] -> [Dynamic])
x6 = APPLYQ typeof (DESCorSELF :/: (NAME "Movie" :?: (NAME "Movie" :/: NONEMPTY)))

x7 :: PF ([Imdb] -> [Dynamic])
x7 = APPLYQ typeof (DESCorSELF :/: NAME "Movie" :/: CHILD :/: NAME "Actor")

x8 :: PF (Imdb -> [Dynamic])
x8 = APPLYQ typeof ((DESCorSELF :/: NAME "Movie") :?: (DESCorSELF :/: NAME "Actor" :/: NONEMPTY))

x9 :: PF (Movie -> [Dynamic])
x9 = APPLYQ typeof (SELF :?: (CHILD :/: NAME "Review" :/: NONEMPTY))

x10 :: PF (Imdb -> [Dynamic])
x10 = APPLYQ typeof (DESCorSELF :/: NAME "Year")

x11 :: PF (Imdb -> [Dynamic])
x11 = APPLYQ typeof (DESCorSELF :/: (NAME "Movie" :?: (((DESCorSELF :/: NAME "BoxOffice") :?: (DESCorSELF :/: NAME "Value" :/: NONEMPTY )) :/: NONEMPTY)))

x12 :: PF (Imdb -> [Dynamic])
x12 = APPLYQ typeof (DESCorSELF :/: (NAME "Movie" :?: ((UNION mlist (DESCorSELF :/: NAME "Actor") (DESCorSELF :/: NAME "Year")) :/: NONEMPTY)))


xp q = reduce xpath typeof q

xpio q = rewrite typeof True xpath q

pf2sxp :: (Typeable a, Typeable b, Result b) => PF (a -> b) -> IO (PF (a -> b))
pf2sxp t = rewrite typeof True pf2xp (APPLYQ typeof (MKQ monoid typeof t))


{-
addMovie m
  = topdown (MKT  (List movie) 
                  (COMP  (Prod movie (List movie)) 
                         CONS 
                         (SPLIT (pnt m) ID)))
                         
addMovieSpider
  = APPLY imdb $ addMovie (El (El "Spider",El 2000))

tst1 
  = rewrite  (Func imdb imdb) True 
             -- syb2pf
             -- (syb2pf Rewriting.>>> Rewriting.tagged optimize) 
             (syb2pf Rewriting.>>> optimize')
             addMovieSpider
getActors
  = collect (AtTag "actor" actor)
-}



-----------------------------------------------------------------------------

