-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt, alcino@di.uminho.pt
--
-- Example: proceedings.
--
-----------------------------------------------------------------------------

module Data.Transform.CoupledProcExample where

import Data.Transform.Coupled
import qualified Data.Transform.Rewriting as Rewriting

-----------------------------------------------------------------------------

proc        = EL "Proceedings"  $  preface .*. Either (List article) (List shortpaper)
preface     = EL "Preface"      $  authors
article     = EL "Article"      $  title .*. authors .*. EL "Abstract" String .*. keywords .*. sections
keywords    = EL "Keywords"     $  List String
shortpaper  = EL "ShortPaper"   $  title .*. authors .*. sections
authors     = EL "Authors"      $  List author
author      = EL "Author"       $  name
name        = EL "Name"         $  EL "Surname" String .*. opt (EL "GivenName" String)
sections    = EL "Sections"     $  List section
section     = EL "Section"      $  title .*. paragraphs
paragraphs  = EL "Paragraphs"   $  List paragraph
paragraph   = EL "Paragraph"    $  title
title       = EL "title"        $  String

(.*.) = Prod

opt a = Either a One

proc0       = El (El (El []), Left [])

-----------------------------------------------------------------------------

-- Find articles about "Data Warehousing"
-- //Article[./Title ftcontains "Data Warehousing" or ./Keywords ftcontains "Data Warehousing"]
q1    =  (etn article `contains` match "Data Warehousing" (etn keywords)) `Union`
         (etn article `contains` match "Data Warehousing" (etn title))

-- Find the paragraphs of the "Introduction" section of each article
-- that has the words "Data Warehousing" in its title.
-- //Article[./Title ftcontains "Data Warehousing]
--     //Section[./Title ftcontains "Data Warehousing"]/Paragraph
q3    =  etn paragraph `containedIn` 
         (etn section `contains` match "Introduction" (etn title)) `containedIn` 
         (etn article `contains` match "Data Warehousing" (etn title))

-- Find all paragraphs containing both "OLAP" and "multidimension" 
-- from either Article or ShortPaper.         
q6    = (  ((etn paragraph `contains` match "OLAP" (etn title)) `containedIn` etn article) `Union`
           ((etn paragraph `contains` match "OLAP" (etn title)) `containedIn` etn shortpaper))
        `Intersect`
        (  ((etn paragraph `contains` match "multidimension" (etn title)) `containedIn` etn article) `Union`
           ((etn paragraph `contains` match "multidimension" (etn title)) `containedIn` etn shortpaper))
q7    = (  ((match "OLAP" (etn title)) `containedIn` etn article) `Union`
           ((match "OLAP" (etn title)) `containedIn` etn shortpaper))
        `Intersect`
        (  ((match "multidimension" (etn title)) `containedIn` etn article) `Union`
           ((match "multidimension" (etn title)) `containedIn` etn shortpaper))
           
tst0 = eval (APPLYQ proc q7) proc0

-----------------------------------------------------------------------------
