{-# OPTIONS -fglasgow-exts #-}
-----------------------------------------------------------------------------
-- |
-- Maintainer  :  joost.visser@di.uminho.pt, alcino@di.uminho.pt
--
-- SYB rewrite rules
--
-----------------------------------------------------------------------------

module Data.Transform.RulesPAT where

import Data.Transform.Type
import Data.Transform.Rewriting
import Control.Monad

-----------------------------------------------------------------------------
-- * Enforcing schema constraints:

e4 :: Rule
e4 _ (MatchContent x p Empty)  = return Empty
e4 _ _                         = mzero

r1 :: Type schema -> Rule
r1 s _ (Etn n t)  = guard (not $ (EL n t) `occursIn` s) >> return Empty
r1 _ _ _          = mzero

r3 :: Rule
r3 _ (MatchContent x p f)     = guard (not $ hascontent x) >> return Empty
r3 _ _                        = mzero 

r4 :: Rule
r4 _ (ContainedIn tf tg f g)  = guard (not $ tf `occursIn` tg) >> return Empty
r4 _ _                        = mzero

r5 :: Rule
r5 _ (Contains tf tg f g)  = guard (not $ tg `occursIn` tf) >> return Empty
r5 _ _                     = mzero

-----------------------------------------------------------------------------
-- * Pulling up union and difference

r6 :: Rule
r6 _ (MatchContent x p (Union f g))  = return $ Union (MatchContent x p f) (MatchContent x p g)
r6 _ _                               = mzero

r7 :: Rule
r7 _ (Intersect f (Union g h))       = return $ Union (Intersect f g) (Intersect f h)
r7 _ _                               = mzero

r8 :: Rule
r8 _ (Intersect (Union f g) h)       = return $ Union (Intersect f h) (Intersect g h)
r8 _ _                               = mzero

r9 :: Rule
r9 _ (Difference (Union f g) h)      = return $ Difference (Intersect f h) (Intersect g h)
r9 _ _                               = mzero

r10 :: Rule
r10 _ (Difference f (Union g h))     = return $ Difference (Difference f g) h
r10 _ _                              = mzero

r11 :: Rule
r11 _ (Contains x y (Union f g) h)     = return $ Union (Contains x y f h) (Contains x y g h)
r11 _ (ContainedIn x y (Union f g) h)  = return $ Union (ContainedIn x y f h) (ContainedIn x y g h)
r11 _ _                                = mzero

r12 :: Rule
r12 _ (Contains x y f (Union g h))     = return $ Union (Contains x y f g) (Contains x y f h)
r12 _ (ContainedIn x y f (Union g h))  = return $ Union (ContainedIn x y f g) (ContainedIn x y f h)
r12 _ _                                = mzero

r13 :: Rule
r13 _ (MatchContent x p (Difference f g))  = return $ Difference (MatchContent x p f) g
r13 _ _                                    = mzero

r14 :: Rule
r14 _ (Intersect (Difference f g) h)   = return $ Difference (Intersect f h) g
r14 _ _                                = mzero

r15 :: Rule
r15 _ (Contains x y (Difference f g) h)     = return $ Difference (Contains x y f h) g
r15 _ (ContainedIn x y (Difference f g) h)  = return $ Difference (ContainedIn x y f h) g
r15 _ _                                     = mzero

r16 :: Rule
r16 _ (Contains x y f (Difference g h))     = return $ Difference (Contains x y f g) (Contains x y f h)
r16 _ (ContainedIn x y f (Difference g h))  = return $ Difference (ContainedIn x y f g) (ContainedIn x y f h)
r16 _ _                                     = mzero

-----------------------------------------------------------------------------
-- * Removing intersection

r17 :: Rule
r17 _ (Intersect (Contains x y f g) h)      = return $ Contains x y (Intersect f h) g
r17 _ (Intersect (ContainedIn x y f g) h)   = return $ ContainedIn x y (Intersect f h) g
r17 _ _                                     = mzero

r18 :: Rule
r18 _ (Intersect f (Contains x y g h))      = return $ Contains x y (Intersect f g) h
r18 _ (Intersect f (ContainedIn x y g h))   = return $ ContainedIn x y (Intersect f g) h
r18 _ _                                     = mzero

r19 :: Rule
r19 _ (Intersect (MatchContent x p f) g)    = return $ MatchContent x p (Intersect f g)
r19 _ _                                     = mzero

r20 :: Rule
r20 _ (Intersect f (MatchContent x p g))    = return $ MatchContent x p (Intersect f g)
r20 _ _                                     = mzero

r21_22_23 :: Rule
r21_22_23 _ (Intersect f Empty)   = return Empty
r21_22_23 _ (Intersect Empty f)   = return Empty
r21_22_23 (TUQ a) (Intersect f g)   = guard (geq (PF (TUQ a)) f g) >> return f
r21_22_23 _ _                     = mzero

-----------------------------------------------------------------------------
-- * Pushing MatchContent down:

r24 :: Rule
r24 _ (MatchContent z p (Contains x y f g))      = return $ Contains x y (MatchContent z p f) g
r24 _ (MatchContent z p (ContainedIn x y f g))   = return $ ContainedIn x y (MatchContent z p f) g
r24 _ _                                          = mzero

-----------------------------------------------------------------------------
-- * Removing contains and contained-in operators

r25 :: Rule
r25 _ (Contains _ z f@(Contains x y e1 e2) e3) = do
  Eq <- teq y z
  guard (geq (PF (TUQ y)) e2 e3)
  return f
r25 _ (ContainedIn _ z f@(ContainedIn x y e1 e2) e3) = do 
  Eq <- teq y z
  guard (geq (PF (TUQ y)) e2 e3)
  return f
r25 _ _ = mzero

r26 :: Rule
r26 _ (Contains _ z f@(Contains x y e1 e2) e3) = do
  Eq <- teq x z
  guard (geq (PF (TUQ x)) e1 e3)
  return f
r26 _ (ContainedIn _ z f@(ContainedIn x _ e1 e2) e3) = do
  Eq <- teq x z
  guard (geq (PF (TUQ x)) e1 e3)
  return f
r26 _ _ = mzero

r27 :: Rule
r27 _ (Contains x y f (MatchContent z p g)) 
          = do Eq <- teq x z; guard (geq (PF (TUQ x)) f g); return $ MatchContent z p g  
r27 _ (ContainedIn x y f (MatchContent z p g)) 
          = do Eq <- teq x z; guard (geq (PF (TUQ x)) f g); return $ MatchContent z p g  
r27 _ _   = mzero

r28 :: Rule
r28 _ (Contains x y (MatchContent z p f) g)
          = do Eq <- teq x y; guard (geq (PF (TUQ x)) f g); return $ MatchContent z p f
r28 _ (ContainedIn x y (MatchContent z p f) g)
          = do Eq <- teq x y; guard (geq (PF (TUQ x)) f g); return $ MatchContent z p f
r28 _ _   = mzero

r29 :: Rule
r29 _ (Contains _ _ f Empty)     = return Empty
r29 _ (ContainedIn _ _ f Empty)  = return Empty
r29 _ _                          = mzero

r30 :: Rule
r30 _ (Contains _ _ Empty f)     = return Empty
r30 _ (ContainedIn _ _ Empty f)  = return Empty
r30 _ _                          = mzero

-----------------------------------------------------------------------------
-- * 

r31 :: Rule
r31 _ (Difference Empty f)  = return Empty
r31 _ _                     = mzero

r32 :: Rule
r32 _ (Difference f Empty)  = return f
r32 _ _                     = mzero

r33 :: Rule
r33 (TUQ a) (Difference f g) = guard (geq (PF (TUQ a)) f g) >> return f
r33 _ _                     = mzero

r34 :: Rule
r34 _ (Union Empty f)       = return f
r34 _ _                     = mzero

r35 :: Rule
r35 (TUQ a) (Union f g)           = guard (geq (PF (TUQ a)) f g) >> return f
r35 _ _                     = mzero

r36 :: Rule
r36 _ (Difference (MatchContent x p f) g)  = guard (geq (PF (TUQ x)) f g) >> return Empty
r36 _ _                                    = mzero

r37 :: Rule
r37 _ (Union (MatchContent x p f) g)   = guard (geq (PF (TUQ x)) f g) >> return f
r37 _ _                                = mzero

r38 :: Rule
r38 _ (Difference (Contains x y f g) h)      = guard (geq (PF (TUQ x)) f h) >> return Empty
r38 _ (Difference (ContainedIn x y f g) h)   = guard (geq (PF (TUQ x)) f h) >> return Empty
r38 _ _                                      = mzero

r39 :: Rule
r39 _ (Union (Contains x y f g) h)      = guard (geq (PF (TUQ x)) f h) >> return f
r39 _ (Union (ContainedIn x y f g) h)   = guard (geq (PF (TUQ x)) f h) >> return f
r39 _ _                                 = mzero

r40 :: Rule
r40 _ (Union f (Contains x y g h)) = 
    do Eq <- teq x y
       guard (geq (PF (TUQ x)) f h) 
       return f 
r40 _ (Union f (ContainedIn x y g h))   = 
    do Eq <- teq x y
       guard (geq (PF (TUQ x)) f h)
       return f 
r40 _ _                                 = mzero

r41 :: Rule
r41 (TUQ a) (Difference (Difference f g) h) = guard (geq (PF (TUQ a)) f h) >> return Empty
r41 _ _                                     = mzero

r42 :: Rule
r42 (TUQ a) (Union (Difference f g) h)  = guard (geq (PF (TUQ a)) f h) >> return f
r42 _ _                                 = mzero

r43 :: Rule
r43 (TUQ a) (Difference (Difference f g) h) = guard (geq (PF (TUQ a)) f h) >> return (Difference f g)
r43 _ _                                     = mzero

r44 :: Rule
r44 (TUQ a) (Union (Difference f g) h)  = guard (geq (PF (TUQ a)) f h) >> return (Union f g)
r44 _ _                                 = mzero

r45 :: Rule
r45 (TUQ a) (Union (Union f g) h)       = guard (geq (PF (TUQ a)) f h) >> return (Union f g)
r45 _ _                                 = mzero

r46 :: Rule
r46 (TUQ a) (Union (Union f g) h)       = guard (geq (PF (TUQ a)) f h) >> return (Union f g)
r46 _ _                                 = mzero

-----------------------------------------------------------------------------
-- * Semantic transformation

-- None. But we should probably still implement |r63| and |r64|.

-----------------------------------------------------------------------------
-- * Exploiting structural information

r67 :: Rule
r67 _ (Contains x _ e1 (Contains z y e3 e2)) 
  | free e3 && entranceLocationFor (qrestype e3) (qrestype e1) (qrestype e2) 
  = return $ Contains x y e1 e2
r67 _ _ = mzero

r68 :: Rule
r68 _ (Contains _ y (Contains x z e1 e3) e2)
  | free e3 && entranceLocationFor (qrestype e3) (qrestype e1) (qrestype e2) 
  = return $ Contains x y e1 e2
r68 _ _ = mzero

r69 :: Rule
r69 _ (Contains _ y (Contains x z e1 e3) e2)
  | free e2 && entranceLocationFor (qrestype e2) (qrestype e1) (qrestype e3) 
  = return $ Contains x z e1 e3
r69 _ _ = mzero

r70 :: Rule
r70 _ (ContainedIn x _ e1 (ContainedIn z y e3 e2)) 
  | free e3 && entranceLocationFor (qrestype e3) (qrestype e1) (qrestype e2) 
  = return $ ContainedIn x y e1 e2
r70 _ _ = mzero

r71 :: Rule
r71 _ (ContainedIn _ y (ContainedIn x z e1 e3) e2)
  | free e3 && entranceLocationFor (qrestype e3) (qrestype e1) (qrestype e2) 
  = return $ ContainedIn x y e1 e2
r71 _ _ = mzero

r72 :: Rule
r72 _ (ContainedIn _ y (ContainedIn x z e1 e3) e2)
  | free e2 && entranceLocationFor (qrestype e2) (qrestype e1) (qrestype e3) 
  = return $ ContainedIn x z e1 e3
r72 _ _ = mzero

r73 :: Rule
r73 _ (ContainedIn x _ e1 (Contains z y e3 e2)) 
  | free e3 && entranceLocationFor (qrestype e1) (qrestype e3) (qrestype e2) 
  = return $ Contains x y e1 e2
r73 _ _ = mzero

r74 :: Rule
r74 _ (Contains x _ e1 (ContainedIn z y e3 e2)) 
  | free e3 && entranceLocationFor (qrestype e1) (qrestype e3) (qrestype e2) 
  = return $ ContainedIn x y e1 e2
r74 _ _ = mzero

-----------------------------------------------------------------------------
-- * Complex transformations

-- | Normalize a PAT-expression, one-step.
pat_normalize1 s
  =  -- e4 
     r1 s ||| r3 ||| r4 ||| r5 |||
     r6 ||| r7 ||| r8 ||| r9 ||| r10 ||| r11 ||| r12 ||| r13 ||| r14 ||| r15 ||| r16 |||
     r17 ||| r18 ||| r19 ||| r20 ||| r21_22_23 |||
     r24 |||
     r25 ||| r26 ||| r27 ||| r28 ||| r29 ||| r30 |||
     r31 ||| r32 ||| r33 ||| r34 ||| r35 ||| r36 ||| r37 ||| r38 ||| r39 ||| r40 ||| 
     r41 ||| r42 ||| r43 ||| r44 ||| r45 ||| r46

-- | Simplify a PAT-expression, one-step
pat_simplify1
  = r67 ||| r68 ||| r69 ||| r70 ||| r71 ||| r72 ||| r73 ||| r74

-- | Normalize (fixpoint) and simplify (fixpoint) a PAT expression.
pat_optimize s
  = innermost (pat_normalize1 s) >>> innermost pat_simplify1


-----------------------------------------------------------------------------


-- * Auxiliary functions

-- | Determine whether a given PAT-algebra expression is free, meaning that
--   it does not constrain its range.
free :: PF (Q b) -> Bool
free Empty                  = False
free (Etn s a)              = True
free (Union f g)            = free f || free g
free (Intersect f g)        = free f && free g
free (Difference f g)       = free f && isEmpty g
free (ContainedIn _ _ f g)  = free f && free g 
free (Contains _ _ f g)     = free f && free g 
free (MatchContent a p f)   = False

-- | Determine whether a given PAT-algebra expression is Empty.
isEmpty :: PF (Q b) -> Bool
isEmpty Empty = True
isEmpty _     = False

-- | Determine whether a give type is an entrance location between two types.
entranceLocationFor :: Type e -> Type x -> Type y -> Bool
entranceLocationFor e x y = not $ or (crushTypeStop aux x)
  where
    aux :: Type a -> [Bool]
    aux z
      | teqBool y z  = [True]    -- found counter example
      | teqBool e z  = [False]   -- no counter example in this branch
      | otherwise    = []        -- keep going down
      
entranceLocationForF :: PF (Q b) -> Type x -> Type y -> Bool
entranceLocationForF (Etn s a) x y  = entranceLocationFor (EL s a) x y
entranceLocationForF _ _ _          = False

-- Equality predicate on types, returning a boolean.
teqBool :: Type a -> Type b -> Bool
teqBool a b = maybe False (const True) (teq a b)

-----------------------------------------------------------------------------

