\documentclass[runningheads]{llncs}

\usepackage{amssymb}
\setcounter{tocdepth}{3}
\usepackage{graphicx}
\usepackage{alltt}
\usepackage{amsmath}
\usepackage{hyperref}
\usepackage{moreverb}

%include polycode.fmt

%------- math -----------------------------------------------------------------%
\def\arrayin#1{\begin{array}{rcl}#1\end{array}}
\def\f{\fun F}
\def\map#1#2{#1\pfunc{#2}}
\def\pfunc#1{\rightharpoonup #1}
\def\Seq#1{[{#1}]}                 % [X]
%\def\Seq#1{{#1}^{\star}}                 % X*
\def\set#1{2^{#1}}                       % 2^X
\def\enset#1{\mathopen{ \{ }#1\mathclose{ \} }} % {a,b,...z}
\def\smp#1{\uppercase{#1}}
\def\ap#1#2{#1\,#2}
\def\ff#1{\ap\f{#1}}
\def\fun#1{{\sf #1}}
\def\dom#1{\unary{dom}{#1}}               % dom x
\def\unary#1#2{\def\arg{#2}\def\omisso{}
    \ifx\arg\omisso{\textsl{#1}}\else\ap{\mbox{\textsl{#1}}}{#2}\fi}
\def\enlist#1{\mathopen{[}#1\mathclose{]}} % [a,b,...z]
\def\enset#1{\mathopen{ \{ }#1\mathclose{ \} }} % {a,b,...z}
\let\iso=\cong
\def\N{I\!\!N}                             % Nat numbers
\def\implies{\mathbin{\Rightarrow}}
\def\biimplies{\mathbin{\Leftrightarrow}}
\def\comp{\mathbin{\cdot}}
\def\rcb#1#2#3#4{\const{#1 \ #2 \ : \ #3 : \ #4}}
\def\Iso#1#2#3#4{{#1}&\iso&{#2}} % backup where arLeq breaks TeX
\def\Leq#1#2#3#4{{#1}&\leq&{#2}} % backup where arLeq breaks TeX
\def\const#1{\mathopen{\langle}#1\mathclose{\rangle}} % <a,b,...z>
%------------------------------------------------------------------------------%


%format dot = "."

%format >>> = "\triangleright"
%format forall a = "\forall " a 
%format ||| = "\oslash"
%format dots = "..."
%format LIST = Listmap
%format MAP = Mapmap
%format ELEMS = Elems
%format LIST2MAP = Fromlist
%format FUN = Fun
%format getArtistsPF = getArtists
%format simplify = rewrite
%%format toRDB = toDB
%format RepPF = Rep
%format to1 = "\Varid{to}_1"
%format to2 = "\Varid{to}_2"
%format from1 = "\Varid{from}_1"
%format from2 = "\Varid{from}_2"

\newcommand{\listmap}{\mathsf{listmap}}
\newcommand{\mapmap}{\mathsf{mapmap}}
\newcommand{\fromlist}{\mathsf{fromlist}}
\newcommand{\elems}{\mathsf{elems}}

\newcommand{\id}{\mathsf{id}}
\newcommand{\fst}{\mathsf{fst}}
\newcommand{\snd}{\mathsf{snd}}
\newcommand{\assocr}{\mathsf{assocr}}
\newcommand{\assocl}{\mathsf{assocl}}
\newcommand{\bang}{\mathsf{bang}}
\newcommand{\after}{\mathbin{\circ}}
\newcommand{\splt}[2]{#1 \mathbin{\vartriangle} #2}

\def\arraystretch{1} % Normal vertical space in tables

\newcommand{\funcdef}[3]{
  \begin{array}{rcl}
    {#1} & : & {#2}\\
    {#1} & = & {#3}
  \end{array}
}

\newenvironment{calculation}[1]{\def\simbolojusti{#1} \left[ \begin{array}{cl}}{\end{array} \right.}
\newcommand{\passo}[1]{& #1}
\newcommand{\justt}[2][\simbolojusti]{\\ #1 & \quad \{ \, \text{#2} \, \} \\}

%format COMP = Comp
%format IDD = Id
%format FST = Fst
%format SND = Snd
%format BANG = Bang
%format SPLIT f g = f "\triangle" g
%format PROD f g = f "\times" g

\newcommand{\myheader}[1]{\textbf{#1}\;}
\newcommand{\ann}[4]{_{#3}#1^{#2}_{#4}}

\begin{document}

\mainmatter  % start of an individual contribution

\title{Constraint-Preserving Two-Level Transformation Rules}
\titlerunning{Constraint-Preserving Two-Level Transformation Rules}

\author{Pablo Berdaguer\and Alcino Cunha\and Joost Visser}
\authorrunning{Berdaguer\and Cunha\and Pacheco\and Visser}

\institute{DI-CCTC, Universidade do Minho, Braga, Portugal}


\toctitle{Constraint-Preserving Two-Level Transformation Rules}
\tocauthor{Berdaguer, Cunha, Pacheco, Visser}
\maketitle

\begin{abstract}
Two-level transformations rules for coupled schema transformation and data donversion for XML and SQL, with annotations for preservation, and propagation of referential constraints.
\end{abstract}

\setlength{\abovedisplayskip}{.3\abovedisplayskip}
\setlength{\belowdisplayskip}{.3\belowdisplayskip}



\section{Introduction}

This document is a literate Haskell script.
\begin{code}
module Data.Transform.RulesWithRefs where

import Data.Transform.Type hiding (from,to)
import Data.Transform.TwoLevelCombinators
import Pointless.Combinators
import Data.Set as Set
import Data.Map as Map
import Control.Monad.State hiding (when)
import Data.List as List hiding (all)
import Data.Char
import Data.Maybe
import Debug.Trace as Trace
\end{code}


\section{Type annotations}

To represent field names and referential constraints, we introduce an annotations mechanisms for types.

\subsection{Representation of field names and referential constraints}

Types are annotated with an optional field name, an optional key identifier, and a possibly empty list of key references.
\begin{spec}
data TypeAnn = TypeAnn {
    fieldName    :: Maybe TagName,
    key          :: Maybe KeyId,
    reference    :: RefType
  } deriving (Show)

type TagName    = String
type KeyId      = (Int,[Int])
type RefName    = String
type RefType    = [(Maybe RefName, KeyId)]
\end{spec}
A new constructor is added to type representations to store annotations:
\begin{spec}
data Type a where
  dots
  Ann :: TypeAnn -> Type a -> Type a
\end{spec}
For convenience, the following getters are defined:
\begin{spec}
getFieldName :: TypeAnn -> Maybe TagName
getFieldName t   = fieldName t

getKey :: TypeAnn -> Maybe KeyId
getKey t         = key t

getReference :: TypeAnn -> RefType
getReference t   = reference t
\end{spec}
Similar functions, but on annotated types, rather than on type annotations:
\begin{code}
getName               :: Type a -> Maybe String
getName (Ann l _)     = getFieldName l
getName _             = Nothing

getKeyType            :: Type a -> Maybe KeyId
getKeyType (Ann l _)  = getKey l
getKeyType _          = Nothing
\end{code}


\subsection{Analysing annotations}

The |getTag| function calculates a name for a structured type, based on the names of its component types.
\begin{code}
getTag :: Type a -> String -> String
getTag (Ann l x) sep     = maybe (getTag x sep) id (getFieldName l)
getTag (Prod a b) sep    = (getTag a sep) ++ sep ++ (getTag b sep)
getTag (Either a b) sep  = (getTag a sep) ++ sep ++ (getTag b sep)
getTag (Map a b) sep     = (getTag a sep) -- ++ (getTag b)
getTag (List a) sep      = getTag a sep
getTag (Set a) sep       = getTag a sep
getTag (Maybe a) sep     = getTag a sep
getTag _ sep             = ""
\end{code}
If a given type has a field name already, this name is used. Otherwise, a name is synthesized from recursive calls on component types. In case of maps, only the key component type is used.

Determine whether at least one key exists inside a type representation:
\begin{code}
isKeyFree :: Type a -> Bool
isKeyFree (Ann l x)
    | (getKey l == Nothing) && (getReference l == []) = isKeyFree x
    | otherwise = False
isKeyFree (Prod a b) = (isKeyFree a) && (isKeyFree b)
isKeyFree (Either a b) = (isKeyFree a) && (isKeyFree b)
isKeyFree (Map a b) = (isKeyFree a) && (isKeyFree b)
isKeyFree (List a) = isKeyFree a
isKeyFree (Set a) = isKeyFree a
isKeyFree (Maybe a) = isKeyFree a
isKeyFree _ = True
\end{code}

\subsection{Changing annotations}

Overwrite the key identifier annotation of a type:
\begin{code}
putKey :: KeyId -> Type a -> Type a
putKey id (Ann l t)  = Ann (l{key=Just id}) t
putKey id t          = Ann (emptyTag{key=Just id}) t
\end{code}
Adds a reference to the annotations of a type.
\begin{code}
addRef :: Maybe RefName -> KeyId -> Type a -> Type a
addRef r id (Ann l t)  = Ann (addReference r id l) t
addRef r id        t   = Ann (addReference r id emptyTag) t
\end{code}
Remove all key annotations from a type representation:
\begin{code}
removeIndexes :: Type a -> Type a
removeIndexes (Ann l a) = Ann l {key = Nothing} (removeIndexes a)
removeIndexes (Prod a b) = Prod (removeIndexes a) (removeIndexes b)
removeIndexes (Either a b) = Either (removeIndexes a) (removeIndexes b)
removeIndexes (Map a b) = Map (removeIndexes a) (removeIndexes b)
removeIndexes (List a) = List (removeIndexes a)
removeIndexes (Set a) = Set (removeIndexes a)
removeIndexes (Maybe a) = Maybe (removeIndexes a)
removeIndexes a = a
\end{code}



\section{Fresh supplies of names and key identifiers}

We use a state monad transformer to keep a fresh supply of names and of key identifiers.
\begin{spec}
type NameSupply m a =  StateT ([String],Int) m a
\end{spec}
This state monad transformer is used to transform the |Maybe| monad in two-level transformation rules:
\begin{spec}
type Rule = forall a . Type a -> NameSupply Maybe (View (Type a))
\end{spec}
First we introduce a function for computing a new name relative to a given supply of names:
\begin{code}
tryName :: [String] -> String -> String
tryName l s
  | ((List.map toUpper s) `elem` l) = tryName' l s 1
  | otherwise = s
  where
    tryName' :: [String] -> String -> Integer -> String
    tryName' l s i
        | (List.map toUpper (s ++ (show i))) `elem` l = tryName' l s (i+1)
        | otherwise = s ++ show i
\end{code}
Using the state monad, we can define monadic functions for computing new names, and such.
\begin{code}
newName :: Monad m => String -> NameSupply m String
newName s = do
    let s' = if (s /= "") then s else "New"
    nn <- nextName s'
    addName nn
    return nn

nextName :: Monad m => String -> NameSupply m String
nextName s = do
    (l,i) <- get
    return $ tryName l s

nextKey :: Monad m => NameSupply m KeyId
nextKey = do
    (l,i) <- get
    put (l,i+1)
    return $ (i+1,[])

addName :: Monad m => String -> NameSupply m ()
addName s = do
    (l,i) <- get
    put (((List.map toUpper s):l),i)
\end{code}
There are other functions to handle names and name-spaces in the type structure.
The function |rename|, rewrites the names of every named field and every reference with a fresh version of the same name in the argument type structure.
\begin{code}
rename :: Monad m => Type a -> NameSupply m (Type a)
rename a = do
    (ns,i) <- get
    let (a',ns') = rename' a ns
    put (ns',i)
    return a'
    where
    rename' :: Type a -> [String] -> (Type a, [String])
    rename' (Ann t a) ns = ((Ann t' a'),ns')
        where
        (t',nl) = renameTag t ns
        (a',ns') = rename' a nl

    rename' (Map a b) ns = ((Map a' b'),ns')
        where
        (a',nn) = rename' a ns
        (b',ns') = rename' b nn
    
    rename' (Prod a b) ns = ((Prod a' b'),ns')
        where
        (a',nn) = rename' a ns
        (b',ns') = rename' b nn
    
    rename' (Either a b) ns = ((Either a' b'),ns')
        where
        (a',nn) = rename' a ns
        (b',ns') = rename' b nn
    
    rename' (List a) ns = ((List a'),ns')
        where
        (a',ns') = rename' a ns
    
    rename' (Set a) ns = ((Set a'),ns')
        where
        (a',ns') = rename' a ns
    
    rename' (Maybe a) ns = ((Maybe a'),ns')
        where
        (a',ns') = rename' a ns
    
    rename' a ns = (a,ns)

    renameTag :: TypeAnn -> [String] -> (TypeAnn,[String])
    renameTag t l
        | getFieldName t /= Nothing = (t {fieldName = jfn', reference = r' }, l')
        | otherwise = (emptyTag {key = getKey t, reference = r' }, l')
        where
        jfn = getFieldName t
        fn' = tryName l (fromMaybe "" jfn)
        jfn' = Just fn'
        (r',l') = renameList (getReference t) (fn':l)

    renameList :: RefType -> [String] -> ( RefType, [String])
    renameList ((rn,kid):t) l = ( ((jrn',kid) : t' ), l')
        where
        Just rnn = rn
        rn' = (tryName l rnn)
        jrn' = Just rn'
        (t', l') = renameList t (l ++ [rn'])
    renameList [] l = ([],l)
\end{code}
And |namesFromType|, returns a list of the names used in a type structure. It's useful to produce the initial state 
\begin{code}
namesFromType :: Type a -> [String]
namesFromType (Ann l a) = (getStrings l) ++ namesFromType a
namesFromType (Map a b) = (namesFromType a) ++ (namesFromType b)
namesFromType (Prod a b) = (namesFromType a) ++ (namesFromType b)
namesFromType (Either a b) = (namesFromType a) ++ (namesFromType b)
namesFromType (List a) = namesFromType a
namesFromType (Set a) = namesFromType a
namesFromType (Maybe a) = namesFromType a
namesFromType _ = []

maxKeyFromType :: Type a -> Int -> Int
maxKeyFromType (Ann t a) m
    | isJust key && kid > m = maxKeyFromType a kid
    | otherwise = maxKeyFromType a m
    where
    key = getKey t
    kid = fst $ fromJust key
maxKeyFromType (Map a b) m
    | maxa > maxb = maxa
    | otherwise = maxb
    where
    maxa = maxKeyFromType a m
    maxb = maxKeyFromType b m
maxKeyFromType (Prod a b) m
    | maxa > maxb = maxa
    | otherwise = maxb
    where
    maxa = maxKeyFromType a m
    maxb = maxKeyFromType b m
maxKeyFromType (Either a b) m
    | maxa > maxb = maxa
    | otherwise = maxb
    where
    maxa = maxKeyFromType a m
    maxb = maxKeyFromType b m
maxKeyFromType (List a) m = maxKeyFromType a m
maxKeyFromType (Set a) m = maxKeyFromType a m
maxKeyFromType (Maybe a) m = maxKeyFromType a m
maxKeyFromType _ m = m

getStrings :: TypeAnn -> [String]
getStrings t = l
    where
    fne tt
        | getFieldName tt /= Nothing = [fn]
        | otherwise = []
        where
        Just fn = getFieldName tt
    rl :: RefType -> [String]
    rl ((rn,kn):rt)
        | rn /= Nothing = (fromMaybe "" rn) : (rl rt)
        | otherwise = rl rt
    rl [] = []
    l = (fne t)++(rl (getReference t))
\end{code}

The function |tailKey| adds an element to the list of integers in the key. This way is kept the order of splitted keys.
\begin{code}
tailAllKeys :: TypeAnn -> Int -> TypeAnn
tailAllKeys t i = t { key = tailKey (getKey t) i, reference = tailAllRefs (getReference t) i}
    where
    tailAllRefs :: RefType -> Int -> RefType
    tailAllRefs ((rn,k):tr) i = (rn,k'):(tailAllRefs tr i)
        where
        Just k' = tailKey (Just k) i
    tailAllRefs [] i = []

tailKey :: Maybe KeyId -> Int -> Maybe KeyId
tailKey (Just (k,l)) i = Just (k, l++[i])
tailKey Nothing _ = Nothing
\end{code}

\section{Well behaved Tag}

The requirements on where annotations occur in types is summarized in Table~\ref{Annotations}.
\begin{table}[t]
\centering
\begin{tabular}{||l||c||c||c||}
\hline 
         & Name & Key/References \\
\hline
basic type    & M & a \\
One           &   &   \\
Prod          &   & a \\
List, Set     & M &   \\
Map           & M &   \\
Either        &   &   \\
Maybe         &   &   \\
\hline
\end{tabular}
\caption{Requirements on where annotations occur in types}
\label{Annotations}
\end{table}

To test if a type structure is ill-annotated we defined:
\begin{code}
traceInv :: Rule
traceInv t
    | checkInv t = return $ View idrep t
    | otherwise = myTrace t

checkInv :: Type a -> Bool
checkInv (Ann t (Prod a b))                        -- In PRODs
    | getFieldName t == Nothing = (checkInv a) && (checkInv b)    -- If there's no name, continue...
    | otherwise = False                        -- else, FALSE

checkInv (Ann t (Either a b))                        -- In EITHERs
    | (isBasicType a) && (isOne b)    = False                -- This should be a Maybe
    | (isOne a) && (isBasicType b)    = False                -- This should be a Maybe
    | t == emptyTag = (checkInv a) && (checkInv b)            -- If there's no name, no key and no reference, continue...
    | otherwise = False                        -- else, FALSE

checkInv (Ann t (Map a b))                        -- In MAPs
    | (getFieldName t /= Nothing)
        && (getKey t == Nothing)
        && (getReference t == []) = (checkInv a) && (checkInv b)    -- If there is name, but no keys/references, continue...
    | otherwise = False                        -- else, FALSE

checkInv (Ann t (Maybe a))                        -- In MAYBEs
    | t == emptyTag = (checkInv a)                    -- If there is no name/keys/references, continue...
    | otherwise = False                        -- else, FALSE

checkInv (Ann t (Set a))                        -- In SETs
    | (getFieldName t /= Nothing)
        && (getKey t == Nothing)
        && (getReference t == []) = (checkInv a)            -- If there is name, but no keys/references, continue...
    | otherwise = False                        -- else, FALSE

checkInv (Ann t (List a))                        -- In LISTs
    | (getFieldName t /= Nothing)
        && (getKey t == Nothing)
        && (getReference t == []) = (checkInv a)            -- If there is name, but no keys/references, continue...
    | otherwise = False                        -- else, FALSE

checkInv (Ann t One)                        -- In ONEs
    |t == emptyTag = True                    -- If there's no name and no keys/references, True!
    | otherwise = False                    -- else, FALSE

checkInv (Ann t a)
    | (isBasicType a)
        && (getFieldName t /= Nothing) = True
    | otherwise = False

checkInv _ = False
\end{code}

The |normalizeTag| rules makes sure that the annotations on a type representation satisfy the requirements.\\
\begin{code}
normalizeTag :: Rule
normalizeTag x = (outerMost (correctTag ||| joinTag)) (tagEverything x)
\end{code}
Normalization is performed by first adding dummy tags to all types that are not tagged yet. Then functions are invoked for adding or removeing information to correct tags on specific types, and for merging possible nested tags.\\
Put empty Ann constructors everywhere:\\
\begin{code}
tagEverything :: Type a -> Type a
tagEverything (Ann l (Prod a b)) = Ann l (Prod (tagEverything a) (tagEverything b))
tagEverything (Prod a b) = (Ann emptyTag (Prod (tagEverything a) (tagEverything b)))
tagEverything (Ann l (Either a b)) = (Ann l (Either (tagEverything a) (tagEverything b)))
tagEverything (Either a b) = Ann emptyTag (Either (tagEverything a) (tagEverything b))
tagEverything (Ann l (Map a b)) = (Ann l (Map (tagEverything a) (tagEverything b)))
tagEverything (Map a b) = Ann emptyTag (Map (tagEverything a) (tagEverything b))
tagEverything (Ann l (List a)) = (Ann l (List (tagEverything a)))
tagEverything (List a) = Ann emptyTag (List (tagEverything a))
tagEverything (Ann l (Set a)) = (Ann l (Set (tagEverything a)))
tagEverything (Set a) = Ann emptyTag (Set (tagEverything a))
tagEverything (Ann l a) = (Ann l a)
tagEverything a = Ann emptyTag a

emptyTag = TypeAnn {fieldName = Nothing, key = Nothing, reference = []}
\end{code}
The |normalTagList| function puts a name on a list type, generated from the name of its element type. For instance, $[\ann{A}{\mathit{movie}}{}{}]$ becomes $\ann{[\ann{A}{\mathit{movie}}{}{}]}{\mathit{movies}}{}{}$.\\
\begin{code}
normalTagList :: Rule
normalTagList t@(List (Ann l a))
    | getFieldName l /= Nothing = do
        nn <- newName ((fromMaybe "" (getFieldName l)) ++ "s")
        return $ View idrep $ Ann l {fieldName = Just nn} t
normalTagList t = return $ View idrep t
\end{code}
Nested Ann merging.\\
\begin{code}
joinTag :: Rule
joinTag (Ann l (Ann m a)) = return $ View idrep (Ann newt a)
  where
    fn = maybe (getFieldName l) (extendFieldName) (getFieldName m)
    extendFieldName m = maybe (Just m) (\l -> Just (l++"$"++m)) (getFieldName l)
    k = concatKeys (getKey l) (getKey m)
    refs = (getReference l) ++ (getReference m)
    newt = TypeAnn {fieldName = fn, key = k, reference = refs}
joinTag _ = mzero

concatKeys :: Maybe KeyId -> Maybe KeyId -> Maybe KeyId
concatKeys x y = fail "We can not join keys yet!"
\end{code}
Note that concatKeys always fails!... So, for now, nested keys must be avoided in the original structure.\\

For specific types, annotations are corrected in different ways to satisfy the requirements.
This function may lose some annotations from the original structure if they are in the wrong places.
\begin{code}

correctTag :: Rule
-- Prod: Doesn't have fieldName, but may have key and refs.
-- Pushes the fieldName down.
correctTag (Ann l (Prod a b)) 
  | getFieldName l == Nothing = mzero
  | otherwise = return $ View idrep (Ann newTag (Prod (Ann tagFN a) (Ann tagFN b))) 
  where
    newTag = l {fieldName = Nothing}
    tagFN = emptyTag {fieldName = getFieldName l}

-- Either: Has a Name Ann, and allows keys and refs
correctTag (Ann l (Either a b))
    | l /= emptyTag = return $ View idrep (Ann emptyTag (Either (Ann tagFNa a) (Ann tagFNb b)))    -- Needs no name!
    | otherwise = mzero
    where
        te = l {fieldName = Nothing}
        tagFNa = emptyTag {fieldName = getFieldName l, key = nka}
        tagFNb = emptyTag {fieldName = getFieldName l, key = nkb}
        kl = getKey l
        nka = tailKey (kl) 0
        nkb = tailKey (kl) 1

-- Map: Has a Name Ann, and dosn't allow keys nor refs
-- The keys are not splitted,they just get lost.
correctTag (Ann l (Map a b))
    | getFieldName l == Nothing = do
        nn <- newName ("Map$"++(getTag a "_"))
        let tm = emptyTag {fieldName = Just nn}
        return $ View idrep (Ann tm (Map a b))
    | getKey l /= Nothing || getReference l /= [] = return $ View idrep (Ann nl (Map a b))
    | otherwise = mzero
    where
    nl = l {key=Nothing, reference=[]}


-- List: Has a Name Ann
correctTag (Ann lt (List (Ann l a)))
    | getFieldName lt == Nothing && getFieldName l == Nothing = mzero
    | getFieldName lt == Nothing = do
        nn <- newName ((fromMaybe "" (getFieldName l)) ++ "s")
        return $ View idrep $ Ann (emptyTag {fieldName = Just nn}) (List (Ann l a))
    | getKey lt /= Nothing = return $ View idrep $ Ann (lt {key = Nothing}) (List (Ann l a))
    | getReference lt /= [] = return $ View idrep $ Ann (lt {reference = []}) (List (Ann l a))
    | otherwise = mzero

-- Set: Has a Name Ann
correctTag (Ann lt (Set (Ann l a)))
    | getFieldName lt == Nothing && getFieldName l == Nothing = mzero
    | getFieldName lt == Nothing = do
        nn <- newName ((fromMaybe "" (getFieldName l)) ++ "s")
        return $ View idrep $ Ann (lt {fieldName = Just nn}) (Set (Ann l a))
    | otherwise = mzero

-- List: Has a Name Ann
correctTag (Ann l (List a))
   | getFieldName l == Nothing = do
        nn <- newName ("List$"++(getTag a "_"))
        let ts =  l {fieldName = Just nn}
        return $ View idrep (Ann ts (List (Ann l a)))
   | otherwise = mzero

-- Set: Has a Name Ann
correctTag (Ann l (Set a))
   | getFieldName l == Nothing = do
        nn <- newName ("Set$"++(getTag a "_"))
        let ts =  l {fieldName = Just nn}
        return $ View idrep (Ann ts (Set (Ann l a)))
   | otherwise = mzero

-- One: Has Nothing! But the empty tag is left to simplify the rules
correctTag (Ann l One)
    | l /= emptyTag = return $ View idrep (Ann emptyTag One)
    | otherwise = mzero

-- Int: Has a Name Ann and keys
correctTag (Ann l Int)
    | getFieldName l == Nothing = do
        nn <- newName "Int"
        let tn =  l {fieldName = Just nn}
        return $ View idrep (Ann tn Int)
    | otherwise = mzero

-- String: Has a Name Ann and keys
correctTag (Ann l String)
    | getFieldName l == Nothing = do
        nn <- newName "String"
        let tn =  l {fieldName = Just nn}
        return $ View idrep (Ann tn String)
    | otherwise = mzero

-- Bool: Has a Name Ann and keys
correctTag (Ann l Bool)
    | getFieldName l == Nothing = do
        nn <- newName "Bool"
        let tn =  l {fieldName = Just nn}
        return $ View idrep (Ann tn Bool)
    | otherwise = mzero

-- Char: Has a Name Ann and keys
correctTag (Ann l Char)
    | getFieldName l == Nothing = do
        nn <- newName "Char"
        let tn =  l {fieldName = Just nn}
        return $ View idrep (Ann tn Char)
    | otherwise = mzero

-- Otherwise
correctTag _ = mzero
\end{code}

The function |replaceDefaultTags| replaces |Either| and |Maybe| based optionals by outermost tagged |Maybe| optionals, used as default in XML to Type and SQL to Type.
\begin{code}
replaceDefaultTags :: Rule
replaceDefaultTags (Ann emptyTag (Either (Ann l a) (Ann _ One))) = return $ View rep $ Ann l $ Maybe a
    where
    rep = Rep { to = either Just (const Nothing), from = maybe (Right ()) Left }
replaceDefaultTags (Ann emptyTag (Either (Ann _ One) (Ann l b))) = return $ View rep $ Ann l $ Maybe b
    where
    rep = Rep { to = either (const Nothing) Just, from = maybe (Left ()) Right }
replaceDefaultTags (Ann _ (Maybe (Ann l a))) = nop $ Ann l $ Maybe a
replaceDefaultTags (Ann _ One) = nop One
replaceDefaultTags _ = mzero
\end{code}

The function |eitherToMaybe| replaces |Either| based optional fields with |Maybe| based optionals. 
\begin{code}

eitherToMaybe :: Rule
eitherToMaybe (Either a (Ann t One))
    | isBasicType a = return $ View auxa (Maybe a)
    | otherwise = nop (Either a (Ann t One))
    where
    auxa :: Rep (Either a ()) (Maybe a)
    auxa = Rep {to = ato, from = afrom}
    ato (Left a) = Just a
    ato (Right _) = Nothing
    afrom (Just a) = Left a
    afrom Nothing = Right ()

eitherToMaybe (Either (Ann _ One) b)
    | isBasicType b = return $ View auxb (Maybe b)
    | otherwise = mzero
    where
    auxb :: Rep (Either () b) (Maybe b)
    auxb = Rep {to = bto, from = bfrom}
    bto (Right b) = Just b
    bto (Left _) = Nothing
    bfrom (Just b) = Right b
    bfrom Nothing = Left ()

eitherToMaybe _ = mzero
\end{code}

\section{Rules involving tags}

\subsection{Associativity and commutativity}

Associate product to the right. One step.
\begin{code}
assocr :: Rule
assocr (Prod (Ann l (Prod a b)) c) = return $ View aux (Prod a (Ann l (Prod b c)))
    where aux = Rep {to = fst.fst /\ snd><id, from = id><fst /\ snd.snd}
assocr (Prod (Prod a b) c) = return $ View aux (Prod a (Prod b c))
    where aux = Rep {to = fst.fst /\ snd><id, from = id><fst /\ snd.snd}
assocr _ = fail "Cannot apply!"

assocl :: Rule
assocl (Prod a (Ann l (Prod b c))) = return $ View aux (Prod (Ann l (Prod a b)) c)
    where aux = Rep {from = fst.fst /\ snd><id, to = id><fst /\ snd.snd}
assocl (Prod a (Prod b c)) = return $ View aux (Prod (Prod a b) c)
    where aux = Rep {from = fst.fst /\ snd><id, to = id><fst /\ snd.snd}
assocl _ = mzero
\end{code}

Put maps to the right and non-maps to the left.
Needs to be redesigned....
\begin{code}
prods :: Rule
prods = {- normalizeTag >>> -} many (many (once Data.Transform.RulesWithRefs.assocr) >>> once swap) {- >>> normalizeTag -} >>> many (once Data.Transform.RulesWithRefs.assocl )
    where
    swap :: Rule
    swap (Prod a (Ann _ (Map b c))) = fail "Cannot apply!"
    swap (Prod a (Ann _ (Prod (Ann l (Map b c)) d))) = fail "Cannot apply!"
    swap (Prod (Ann l (Map a b)) c) = return $ View (Rep {from = snd/\fst, to = snd/\fst}) (Prod c (Ann l (Map a b)))
    swap _ = fail "Cannot apply!"
\end{code}


\subsection{Flattening rules}

The flattening strategy needs to be fixed.
Annotation's restrictions should, now, be preserved by the rules!
\begin{code}
flatten :: Rule
flatten = aux >>> many aux
  where 
    aux = prods >>> flat
    flat = ((once prodone) >>> (myTraceH "prodone")) ||| ((once eitherToMaybe) >>> (myTraceH "eitherToMaybe")) ||| ((once listmap) >>> (myTraceH "listmap")) ||| ((once setmap) >>> (myTraceH "setmap")) ||| ((once summap) >>> (myTraceH "summap")) ||| ((once mapsum) >>> (myTraceH "mapsum")) ||| ((once prodsum) >>> (myTraceH "prodsum")) ||| ((once sumone) >>> (myTraceH "sumone")) ||| ((once mapprodmap) >>> (myTraceH "mapprodmap"))
\end{code}

To start the flattening process, |toRDB| must be used.\\
|toRDB'| makes some changes in the type structure due restrictions on |Type| annotations.\\
\begin{code}
toRDB' = myTraceH "Start" >>> normalizeTag >>> outerMost eitherToMaybe >>> flatten >>>  outerMost mapsum >>> many (once replaceDefaultTags)
toRDB = uncurry evalStateT . (toRDB' /\ initNameSupply)
\end{code}

|NameSupply| is the initial state for the argument type. It contains: the list of names already in the type and SQL reserved words, and the count of keys already present in the type.
\begin{code}
initNameSupply :: Type a -> ([String],Int)
initNameSupply a = (namesFromType a ++ reserved, maxKeyFromType a 0)
    where
    reserved = ["ADD", "ALL", "ALTER", "ANALYZE", "AND", "AS", "ASC", "ASENSITIVE", "BEFORE", "BETWEEN", "BIGINT", "BINARY", "BLOB", "BOTH", "BY", "CALL", "CASCADE", "CASE", "CHANGE", "CHAR", "CHARACTER", "CHECK", "COLLATE", "COLUMN", "CONDITION","CONNECTION", "CONSTRAINT", "CONTINUE", "CONVERT", "CREATE", "CROSS", "CURRENT_DATE", "CURRENT_TIME", "CURRENT_TIMESTAMP", "CURRENT_USER", "CURSOR", "DATABASE", "DATABASES", "DAY_HOUR", "DAY_MICROSECOND","DAY_MINUTE", "DAY_SECOND", "DEC", "DECIMAL", "DECLARE", "DEFAULT", "DELAYED", "DELETE", "DESC","DESCRIBE", "DETERMINISTIC", "DISTINCT", "DISTINCTROW", "DIV", "DOUBLE", "DROP", "DUAL", "EACH", "ELSE", "ELSEIF", "ENCLOSED", "ESCAPED", "EXISTS", "EXIT", "EXPLAIN", "FALSE", "FETCH", "FLOAT", "FLOAT4", "FLOAT8", "FOR", "FORCE", "FOREIGN", "FROM", "FULLTEXT", "GRANT", "GROUP", "HAVING", "HIGH_PRIORITY", "HOUR_MICROSECOND", "HOUR_MINUTE", "HOUR_SECOND", "IF", "IGNORE", "IN", "INDEX", "INFILE", "INNER", "INOUT", "INSENSITIVE", "INSERT", "INT", "INT1", "INT2", "INT3", "INT4", "INT8", "INTEGER", "INTERVAL", "INTO", "IS", "ITERATE", "JOIN", "KEY", "KEYS", "KILL", "LEADING","LEAVE", "LEFT", "LIKE", "LIMIT", "LINES", "LOAD", "LOCALTIME", "LOCALTIMESTAMP", "LOCK", "LONG", "LONGBLOB","LONGTEXT", "LOOP", "LOW_PRIORITY", "MATCH", "MEDIUMBLOB", "MEDIUMINT", "MEDIUMTEXT", "MIDDLEINT", "MINUTE_MICROSECOND", "MINUTE_SECOND", "MOD", "MODIFIES", "NATURAL", "NOT", "NO_WRITE_TO_BINLOG", "NULL", "NUMERIC", "ON", "OPTIMIZE", "OPTION", "OPTIONALLY", "OR", "ORDER", "OUT", "OUTER", "OUTFILE", "PRECISION", "PRIMARY", "PROCEDURE", "PURGE", "RAID0", "READ", "READS", "REAL", "REFERENCES", "REGEXP", "RELEASE", "RENAME", "REPEAT", "REPLACE", "REQUIRE", "RESTRICT", "RETURN", "REVOKE", "RIGHT", "RLIKE", "SCHEMA", "SCHEMAS", "SECOND_MICROSECOND", "SELECT", "SENSITIVE","SEPARATOR", "SET", "SHOW", "SMALLINT", "SONAME", "SPATIAL", "SPECIFIC", "SQL", "SQLEXCEPTION", "SQLSTATE","SQLWARNING", "SQL_BIG_RESULT", "SQL_CALC_FOUND_ROWS", "SQL_SMALL_RESULT", "SSL", "STARTING", "STRAIGHT_JOIN", "TABLE", "TERMINATED", "THEN", "TINYBLOB", "TINYINT", "TINYTEXT", "TO", "TRAILING", "TRIGGER", "TRUE", "UNDO", "UNION", "UNIQUE", "UNLOCK", "UNSIGNED", "UPDATE", "UPGRADE", "USAGE", "USE", "USING", "UTC_DATE", "UTC_TIME", "UTC_TIMESTAMP","VALUES", "VARBINARY", "VARCHAR", "VARCHARACTER", "VARYING", "WHEN", "WHERE", "WHILE", "WITH", "WRITE", "X509", "XOR", "YEAR_MONTH", "ZEROFILL"]
\end{code}


\subsubsection{listmap. }
Eliminate list by refinement to map.\\
* This rule will hold the invariants as long as we keep restrictions on |Map|'s annotations included or equal to restrictions on |List|'s annotations.\\
\begin{center}
$[A] \implies (\ann{int}{\underline{index}}{}{} \pfunc A)$
\end{center}
\begin{code}
listmap :: Rule
listmap (Ann l (List a)) = do
  nn <- newName "index"
  let k = Ann (emptyTag {fieldName = Just nn}) Int
  return $ View rep (Ann l (Map k a)) 
  where
    rep = Rep {to = seq2index, from = list}
    list :: Map Int a -> [a]
    list = Map.elems
    seq2index :: [a] -> Map Int a
    seq2index = Map.fromList . (zip [0..])
listmap _ = mzero
\end{code}
\subsubsection{setmap. }
Eliminate set by refinement to map.\\
* This rule will hold the invariants as long as we keep restrictions on |Map|'s annotations included or equal to restrictions on |Set|'s annotations.\\
\begin{center}
$\{A\} \implies (A \pfunc 1)$
\end{center}
\begin{code}
setmap :: Rule
setmap (Set a) = return $ View rep (Map a (Ann emptyTag One))
  where
    rep :: Rep (Set a) (Map a ())
    rep = Rep {to = set2fm, from = dom}
    set2fm s = Map.fromDistinctAscList (zip (Set.toAscList s) (repeat ()))
    dom = Set.fromDistinctAscList . keys
setmap _ = mzero
\end{code}
\subsubsection{summap. }
Eliminates sums by refinement to prod of two maps.
If the sum has an index, it can't be distributed.\\
\begin{center}
$\ann{(\map{(A+B)}{C})}{n}{}{} \implies (\ann{(\map{A}{C})}{\underline{n}}{}{}) \times (\ann{(\map{B}{C})}{\underline{n}}{}{})$
\end{center}
\begin{code}
summap :: Rule
summap (Ann lm (Map (Ann _ (Either a b)) c)) = do
    l <- newName (fromMaybe "" $ getFieldName lm)
    r <- newName (fromMaybe "" $ getFieldName lm)
    let tml = lm {fieldName = Just l}
    let tmr = lm {fieldName = Just r}
    c' <- rename c
    return $ View eithermap (Ann emptyTag (Prod (Ann tml (Map a c)) (Ann tmr (Map b c'))))
  where
    eithermap :: Rep (Map (Either a b) c) (Map a c, Map b c)
    eithermap = Rep {to = unpeither, from = peither}
    unpeither = ( mapKeysMonotonic (either id undefined) . Map.fromDistinctAscList ><
                  mapKeysMonotonic (either undefined id) . Map.fromDistinctAscList
                ) . List.partition (isLeft . fst) . Map.toAscList
    peither = Map.fromDistinctAscList . uncurry (++) .
              (fmap (Left >< id) . Map.toAscList >< fmap (Right >< id) . Map.toAscList)
summap _ = mzero
\end{code}
\subsubsection{prodsum. }
Distribute product over sum.\\
As it produces a |Sum| of |Prods|, this |Sum| can't be refined as a Maybe... Not directly at least.\\
\begin{center}
$A \times (B+C) \implies ((A \times B) + (A \times C))$
\end{center}
\begin{code}
prodsum :: Rule
prodsum (Ann lp (Prod a (Ann _ (Either b c)))) = do
    a' <- rename a
    return $ View distr (Ann emptyTag (Either (Ann lp0 (Prod a' b)) (Ann lp1 (Prod a c))))
    where
    lp0 = tailAllKeys lp 0
    lp1 = tailAllKeys lp 1
    distr :: Rep (a,Either b c) (Either (a,b) (a,c))
    distr = Rep {from = undistr, to = distr}
        where
        distr (x, Left y) = Left (x,y)
        distr (x, Right y) = Right (x,y)
        undistr (Left (x,y)) = (x, Left y)
        undistr (Right (x,y)) = (x, Right y)
\end{code}
\begin{center}
$(B+C) \times A \implies (B \times A)+(C \times A)$
\end{center}
\begin{code}
prodsum (Ann lp (Prod (Ann _ (Either b c)) a)) = do
    a' <- rename a
    return $ View distl (Ann emptyTag (Either (Ann lp0 (Prod b a)) (Ann lp1 (Prod c a'))))
    where
    lp0 = tailAllKeys lp 0
    lp1 = tailAllKeys lp 1
    distl :: Rep (Either b c, a) (Either (b,a) (c,a))
    distl = Rep {from = undistl, to = distl}
        where
        distl (Left y,x) = Left (y,x)
        distl (Right y,x) = Right (y,x)
        undistl (Left (y,x)) = (Left y,x)
        undistl (Right (y,x)) = (Right y,x)
prodsum _ = mzero
\end{code}
\subsubsection{mapsum. }
Distribute map over sum.\\
There are two ways of flattening\\
\begin{center}
$\ann{(\map{A}{(B+C)})}{n}{}{}$
\end{center}
If there's no reference that points to $A$, that means, $A$ has no key... then, it can be distributed like\\
\begin{center}
$\ann{( \map{ \ann{A}{}{\emptyset}{} } {(B+C)} )}{n}{}{} \implies (\ann{(\map{A}{B})}{\underline{n}}{}{}) \times \ann{(\map{A}{C})}{\underline{n}}{}{}$
\end{center}
If $A$ has a key, each side of the either should be made optional ($\map{A}{\map{B}{1} \times \map{c}{1}}$) and let |mapprodmap| split it to new tables:\\
\begin{center}
$\ann{(\map{\ann{A}{}{k}{}}{(B+C)})}{n}{}{} \implies (\ann{(\map{\ann{A}{}{k}{}}{1})}{\underline{n}}{}{}) \times \ann{(\map{\ann{A}{}{}{k}}{B})}{\underline{n}}{}{} \times \ann{(\map{\ann{A}{}{}{k}}{C})}{\underline{n}}{}{}$
\end{center}

\begin{code}
mapsum :: Rule
mapsum (Ann lm (Map a (Ann _ (Either b c))))
    | isKeyFree a = do
        l <- newName (fn)
        let lml = emptyTag {fieldName = Just l}
        r <- newName (fn)
        let lmr = emptyTag {fieldName = Just r}
        a' <- rename a
        return $ View rep (Ann emptyTag (Prod (Ann lml (Map a b)) (Ann lmr (Map a' c))))
  where
    mapname = getFieldName lm
    Just fn = mapname

    rep = Rep {to = uncojoin, from = cojoin a}
    uncojoin :: Map a (Either b c) -> (Map a b, Map a c)
    uncojoin = (  fmap (either id undefined) . Map.fromDistinctAscList ><
                  fmap (either undefined id) . Map.fromDistinctAscList) .
               List.partition (isLeft . snd) . Map.toAscList
    -- Pre: non duplicate keys
    cojoin :: Type a -> (Map a b, Map a c) -> Map a (Either b c)
    cojoin t = Map.fromDistinctAscList .
               sortBy (\x y -> gcompare t (fst x) (fst y)) .
               uncurry (++) .
               (Map.toAscList . fmap Left >< Map.toAscList . fmap Right)
\end{code}

\begin{code}
mapsum (Ann lm (Map a b)) = do
    View rep' t'<- (once hasEither >>> (once either2opts)) b
    let rep = Rep {to = Map.map (to rep'), from = Map.map (from rep')}
    return $ View rep $ Ann lm $ Map a t'
mapsum _ = mzero
\end{code}
\subsubsection{prodone. }
Eliminates One fields/tables?\\
If the context of $(A \times 1)$ is $((A \times 1) + 1)$, and A is a basic type, the application of this rule ***********
\begin{center}
$ A \times 1 \implies A $
\end{center}
\begin{code}
prodone :: Rule
prodone (Ann _ (Prod a (Ann _ One))) = return $ View rep a
 where
  rep :: Rep (a,()) a
  rep = Rep {to = fst, from = \x -> (x,())}
prodone (Ann _ (Prod (Ann _ One) b)) = return $ View rep b
 where
  rep :: Rep ((),b) b
  rep = Rep {to = snd, from = \x -> ((),x)}
prodone _ = fail "Not applicable"
\end{code}
\subsubsection{sumone. }
Eliminate optional by refinement to map.\\
\begin{center}
$A+1 \implies \map{1}{A}$
\end{center}
\begin{code}
sumone :: Rule
sumone (Ann _ (Either a (Ann _ One)))
    | not (isBasicType a) = do
        mn <- newName (getTag a "_")
        let mt = emptyTag {fieldName = Just mn}
        return $ View plusone (Ann mt (Map (Ann emptyTag One) a))
    | otherwise = fail "(Either BasicType One) found. This should be (Maybe BasicType)." -- HERE WE SHOULD REFINE TO MAYBE A... BUT, ACCORDING TO INVARIANTS, THIS CAN'T HAPPEN ----------------------------------------------
    where
    plusone :: Rep (Either a ()) (Map () a)
    plusone = Rep {to = optintro, from = optelim}
        where
        optintro (Left x) = Map.singleton () x
        optintro (Right ()) = Map.empty
        optelim m
            | Map.member () m = let Just x = Map.lookup () m in Left x
            | otherwise = Right ()
sumone _ = fail "Not applicable"
\end{code}
\subsubsection{mapprodmap. }
Split and join.\\
This rule expands nested maps as a product of maps creating a reference from the inner |Map| to the outer one.\\
If A already has a key, the rules reuses it insted of creating a new one.
\begin{center}
$\ann{(\map{A}{(\ann{(\map{C}{D})}{m}{}{})})}{n}{}{} \implies (\ann{(\map{(\ann{A}{}{\underline{k}}{})}{1})}{n}{}{}) \times (\ann{(\map{(\ann{A}{}{}{k} \times C)}{D})}{\underline{m}}{}{})$
\end{center}
\begin{code}
mapprodmap :: Rule
mapprodmap (Ann lm (Map a (Ann lm1 (Map c d)))) = do
  View rep t <- mapprodmap (Ann lm (Map a (Ann emptyTag (Prod (Ann emptyTag One) (Ann lm1 (Map c d))))))
  return $ View (comprep aux rep) t
  where aux = Rep {to = Map.map (\x -> ((),x)) , from = Map.map snd}
\end{code}
\begin{center}
$ \ann{(\map{A}{(B \times \ann{(\map{C}{D})}{m}{}{})})}{n}{}{} \implies (\ann{(\map{(\ann{A}{}{\underline{k}}{})}{B})}{n}{}{}) \times (\ann{(\map{((\ann{A}{}{}{k}) \times C)}{D})}{m}{}{}) $
\end{center}
\begin{code}
mapprodmap (Ann lm (Map (Ann ta a) (Ann _ (Prod b (Ann lm1 (Map c d)))))) = do
    nk <- nextKey
    let Just nin = if (getKey ta) == Nothing
                   then Just nk
                   else getKey ta
    let (nkn,ni)=nin
    fkn <- newName ("FK"++show nkn)
    lm1n' <- newName (fromMaybe "" (getFieldName lm1))
    let lm1n = emptyTag {fieldName = Just lm1n'}
    let lfk = emptyTag {fieldName = getFieldName ta, reference = [(Just fkn,nin)]}
    let li = ta {key = Just nin}
    na <- rename a
    let a' = removeIndexes na    -- Careful!
    let resultType = if (getKeyType a) /= Nothing
                     then (Ann emptyTag (Prod (Ann lmn (Map (Ann ta a) b)) (Ann lm1n (Map (Ann emptyTag (Prod (Ann lfk a') c)) d))))
                     else (Ann emptyTag (Prod (Ann lmn (Map (Ann li a) b)) (Ann lm1n (Map (Ann emptyTag (Prod (Ann lfk a') c)) d))))
    return $ View (mapcomposite a) resultType
 where
  lmn = emptyTag {fieldName = getFieldName lm}
  Just (kid,l)=getKeyType a
  -- Precondition to from: all keys a in second table must exist in the first
  mapcomposite :: Type a -> Rep (Map a (b, Map c d)) (Map a b, Map (a,c) d)
  mapcomposite t = Rep {to = unnjoin, from = njoin}
    where unnjoin = (Map.map fst /\ to (mapcomp t) . Map.map snd)
          njoin = Map.fromDistinctAscList . uncurry myzip . (Map.toAscList >< Map.toAscList . from (mapcomp t))
          myzip l [] = fmap (id >< (id /\ const Map.empty)) l
          myzip ((x,y):l) ((z,w):m) | gcompare t x z == EQ = (x,(y,w)) : myzip l m
                                    | otherwise = (x,(y,Map.empty)) : myzip l ((z,w):m)
  unnjoin :: Map a (Map b c) -> Map (a,b) c
  unnjoin =  Map.fromDistinctAscList . concat .
             List.map ((\ (x,l) -> List.map ((const x /\ id) >< id) l) .
             (id >< Map.toAscList)) .
             Map.toAscList
  njoin :: Type a -> Map (a,b) c -> Map a (Map b c)
  njoin t = Map.fromDistinctAscList .
            fmap (fst.fst.head /\ Map.fromDistinctAscList . fmap (snd >< id)) .
            groupBy aux . Map.toAscList
    where aux x y = gcompare t (fst (fst x)) (fst (fst y)) == EQ
  mapcomp :: Type a -> Rep (Map a (Map c d)) (Map (a,c) d)
  mapcomp t = Rep {to = unnjoin, from = njoin t}
mapprodmap _ = mzero
\end{code}
\section{Other flattening functions}
Returns it's argument if it's an Either, or mzero otherwise
\begin{code}
hasEither :: Rule
hasEither t@(Either a b)
    | isBasicType a && isOne b = mzero
    | isBasicType b && isOne a = mzero
    | otherwise = nop t
hasEither _ = mzero
\end{code}
Rewrites Either branches as optional Prod branches
\begin{code}
either2opts :: Rule
either2opts = (outerMost eitherToMaybe) >>> either2opts' >>> (outerMost eitherToMaybe)
    where
    either2opts' :: Rule
    either2opts' (Either (Ann l a) (Ann j b))
        | isOne b = mzero
        | otherwise = do
		nn1 <- newName "OptA"
		let nl = if (fromMaybe "" (getFieldName l) /= "")
			then getFieldName l
			else Just nn1
		nn2 <- newName "OptB"
		let nj = if (fromMaybe "" (getFieldName j) /= "")
			then getFieldName j
			else Just nn2
		return $ View rep (Prod (Ann emptyTag {fieldName = nl} (Map (Ann emptyTag One) (Ann l a))) (Ann emptyTag {fieldName=nj} (Map (Ann emptyTag One) (Ann j b))))
        where
        rep = Rep {to = map_opt, from = opt_map}
        map_opt = either (\x -> (Map.singleton () x,Map.empty)) (\x -> (Map.empty,Map.singleton () x))
        opt_map (x,y) | not (Map.null x) && Map.null y = Left $ head $ Map.elems x
        opt_map (x,y) | not (Map.null y) && Map.null x = Right $ head $ Map.elems y
    either2opts' (Either (a) (b))
        | isOne b = mzero
        | otherwise = return $ View rep (Prod ((Map (Ann emptyTag One) a)) ((Map (Ann emptyTag One) b)))
        where
        rep = Rep {to = map_opt, from = opt_map}
        map_opt = either (\x -> (Map.singleton () x,Map.empty)) (\x -> (Map.empty,Map.singleton () x))
        opt_map (x,y) | not (Map.null x) && Map.null y = Left $ head $ Map.elems x
        opt_map (x,y) | not (Map.null y) && Map.null x = Right $ head $ Map.elems y
    either2opts' _ = mzero
\end{code}


\section{Error reporting functions}
These functions were written to report errors in the flattening process. They were defined using |isRelationalDB_Type| as pattern.
The first one, |whereIsntRelationalDB_Type|, returns the string representation of the argument type structure but replaces every non-relational sub-structure with an error message.

\begin{code}
whereIsntRelationalDB_Type :: Type a -> String
whereIsntRelationalDB_Type = wir
    where
    wir :: Type a -> String
    wir (Ann l (Prod a b))
        | l==emptyTag = " (Ann "++(show l)++" (Prod" ++ (wir a) ++ (wir b) ++ ") "
        | otherwise = "   Error: Table-level Prods should have empty Anns!   "

    wir (Ann l (Map a b))
        | fn l /= Nothing
        && getKey l == Nothing
        && getReference l == [] = " (Ann "++(show l)++" (Map" ++ (wir' a) ++ (wir' b) ++ ") "
        | otherwise = "   Error Unnamed Map   "
    
    wir _ = "   Error   "
    
    wir' :: Type a -> String
    
    wir' t@(Ann l One)
        | l == emptyTag = show t
        | otherwise = "   Error Unnamed Int   "

    wir' t@(Ann l Int)
        | fn l /= Nothing = show t
        | otherwise = "   Error Unnamed Int   "
    
    wir' t@(Ann l Bool)
        | fn l /= Nothing = show t
        | otherwise = "   Error Unnamed Bool   "
    
    wir' t@(Ann l Char)
        | fn l /= Nothing = show t
        | otherwise = "   Error Unnamed Char   "
    
    wir' t@(Ann l String)
        | fn l /= Nothing = show t
        | otherwise = "   Error Unnamed String   "
    
    wir' t@(Ann l (Prod x y))
        | fn l == Nothing = show t
        | otherwise = "   Error: Named Prod!   "
    
    wir' t@(Ann l (Either x y))
        | isOne x && l == emptyTag && isBasic y = show t
        | isOne y && l == emptyTag && isBasic x = show t
        | otherwise = "   Error Nullable Field   "
    
    wir' t@(Ann l (Maybe x))
        | l == emptyTag && isBasic x = show t
        | isJust (getFieldName l) && isBasic x = show t
        | otherwise = "   Error Nullable Field   "
    
    wir' _ = "   Error   "

    fn = getFieldName

    isBasic = isBasicType
\end{code}


To show the non-relational sub-structures in a Type structure, we defined a functions that returns the string representation of every one of them sepaired by two new-line characters.
\begin{code}
whatIsntRelationalDB_Type :: Type a -> String
whatIsntRelationalDB_Type = wnr
    where
    wnr :: Type a -> String
    wnr (Ann l (Prod a b))
        | l == emptyTag = (wnr a) ++ (wnr b)
        | otherwise = "\n\n"++show (Ann l (Prod a b))
    
    wnr (Ann l (Map a b))
        | getFieldName l /= Nothing
            && getKey l == Nothing
            && getReference l == [] = (wnr' a) ++ (wnr' b)
        | otherwise = "\n\n"++show (Ann l (Map a b))
    
    wnr a = "\n\n" ++ show a
    
    wnr' :: Type a -> String
    wnr' t@(Ann l One)
        | l == emptyTag = ""
        | otherwise = "\n\n" ++ show t
    
    wnr' t@(Ann l Int)
        | getFieldName l /= Nothing = ""
        | otherwise = "\n\n" ++ show t
    
    wnr' t@(Ann l Bool)
        | getFieldName l /= Nothing = ""
        | otherwise = "\n\n" ++ show t
    
    wnr' t@(Ann l Char)
        | getFieldName l /= Nothing = ""
        | otherwise = "\n\n" ++ show t
    
    wnr' t@(Ann l String)
        | getFieldName l /= Nothing = ""
        | otherwise = "\n\n" ++ show t
    
    wnr' t@(Ann l (Prod x y))
        | getFieldName l == Nothing = (wnr' x) ++ (wnr' y)
        | otherwise = "\n\n" ++ show t

    wnr' t@(Ann l (Either x y))
        | isOne x && l == emptyTag && isBasicType y = ""
        | isOne y && l == emptyTag && isBasicType x = ""
        | otherwise = "\n\n" ++ show t

    wnr' (Ann l (Maybe a))
        | l == emptyTag && isBasicType a = ""
        | isJust (getFieldName l) && isBasicType a = ""
        | otherwise = "\n\n"++show (Ann l (Maybe a))

    wnr' a = "\n\n" ++ show a
\end{code}

\appendix

\section{TODO: place and talk about new required functions (hugo)}

\begin{code}
-- * Functions and Instances over the TypeAnn

isAtt :: TypeAnn -> Bool
isAtt = maybe False (isPrefixOf "att$") . getFieldName

getAttFieldName :: TypeAnn -> Maybe TagName
getAttFieldName = liftM (drop 4) . getFieldName

addReference :: Maybe RefName -> KeyId -> TypeAnn -> TypeAnn
addReference  n k tag = tag {reference = (n,k) : getReference tag}
\end{code}

\begin{code}
{- * Generated by DrIFT : Look, but Don't Touch. * -}
instance Read TypeAnn where
    readsPrec d input =
              readParen (d > 9)
              (\ inp ->
               [((TypeAnn aa ab ac) , rest) | ("TypeAnn" , inp) <- lex inp ,
                ("{" , inp) <- lex inp , ("fieldName" , inp) <- lex inp ,
                ("=" , inp) <- lex inp , (aa , inp) <- readsPrec 10 inp ,
                ("," , inp) <- lex inp , ("key" , inp) <- lex inp ,
                ("=" , inp) <- lex inp , (ab , inp) <- readsPrec 10 inp ,
                ("," , inp) <- lex inp , ("reference" , inp) <- lex inp ,
                ("=" , inp) <- lex inp , (ac , inp) <- readsPrec 10 inp ,
                ("}" , rest) <- lex inp])
              input
\end{code}

\begin{code}
-- * Relational Database to XML conversion

-- Converts a relational type to a xml schema type representation.
toXML a = evalStateT (toXML' a) ([],0)

-- Eiliminates maps from xml schemas.
toXML' :: Rule
toXML' = many (once map2xsd)

-- Eliminates a map from xml schemas through several refinements.
map2xsd :: Rule
-- refinement to boolean
map2xsd (Ann l (Map One One)) = return $ View rep $ Ann l Bool
    where
    rep = Rep {to = fm2bool, from = bool2fm}
    fm2bool m = if (Map.null m) then False else True
    bool2fm b = if b then (Map.singleton () ()) else Map.empty
-- refinement to either
map2xsd (Ann l (Map (Ann _ One) b)) = return $ View rep $ Ann l $ Either b (Ann emptyTag One)
    where
    rep = Rep {to = untotOne, from = totOne}
    untotOne m = if (Map.null m) then (Right ()) else (Left $ head $ Map.elems m)
    totOne =  either (Map.singleton ()) (const Map.empty)
-- refinement to set
{-
map2xsd (Ann l (Map a One)) = return (View rep (Ann l (Set a)))
  where
    rep = Rep {to = dom, from = set2fm}
    set2fm s = Map.fromDistinctAscList (zip (Set.toAscList s) (repeat ()))
    dom = Set.fromDistinctAscList . keys
-}
-- refinement to list of products
map2xsd (Ann l (Map a b)) = return $ View rep $ Ann l $ List $ Prod a b
    where
    rep = Rep {to = Map.toAscList, from = Map.fromDistinctAscList}
map2xsd _ = mzero
\end{code}

\begin{code}
-- ** Enrichment and removal

--  Add a field (to the right).
--   The forward value-level function inserts a 
--   constant value for the new field.
addfield :: Type b -> b -> Rule
addfield b x a = return $ View rep (Prod a b)
    where rep = Rep {to = \y -> (y,x), from = fst}

--   Add a field (to the left).
--   The forward value-level function inserts a 
--   constant value for the new field.
addfieldl :: Type b -> b -> Rule
addfieldl b x a = return $ View rep (Prod b a)
    where rep = Rep {to = \y -> (x,y), from = snd}

--   Type of generic value-level queries.
type Query b = forall a . Type a -> a -> b

--   Add a field.
--   The forward value-level function inserts a field
--   determined by a query on its input value.
addinfo :: Type b -> Query b -> Rule
addinfo b f a = return $ View rep (Prod a b)
    where rep = Rep {to = \y -> (y,f a y), from = fst} 
\end{code}

\begin{code}
--   Adds an entry to a list.
-- The function only succeeds if the input list type matches the rule type.
-- Execute the backward leve function only when the values where transformed with the forward value function function, otherwise the first element of the list is dropped.
addlistval :: Type [b] -> b -> Rule
addlistval b v a =
    case (teq' a b) of
        Just Eq -> return $ View (addlistentry v) a
        Nothing -> mzero

\end{code}

\begin{code}
-- ** Generalization and restriction

--   Add alternative.
--   The backward value level function fails if the added 
--   alternative is present.
addalt :: Type b -> Rule
addalt b a = return $ View rep (Either a b)
    where rep = Rep {to = Left, from = \(Left x) -> x}

--   Add alternative somewhere inside a list.
-- The backward value leve function is complete, since it removes all the values for the added alternative.
addaltlist :: (forall a . Type a -> Bool)        -- the lookup function (see 'dowhen')
       -> Type b            -- the alternative to add
       -> Rule            -- the generated rule
addaltlist f b t@(List _) = do
        (View rep c) <- once (when f (addalt b)) t
        let newrep = addaltlistrep b c rep
        return $ View newrep c
addaltlist _ _ _ = mzero

--   Allow repetition.
--   Maps optional to a list.
allowRep :: Rule
allowRep (Either a One) = return $ (View (Rep opt2seq seq2opt) (List a))
  where 
    opt2seq (Left x) = [x]
    opt2seq (Right ()) = []
    seq2opt []  = Right ()
    seq2opt [x] = Left x
    seq2opt _ = error "Too many elements in list for conversion to optional"
allowRep _ = mzero

--   Lifted version of 'allowRep', works on functors.
allowRep1 :: Rule
allowRep1 a = return $ (View rep (Mu ((K a) :+: ((K a) :*: Id))))
  where 
    rep = Rep singleton (nehead . out)
    singleton x = In $ Inl $ Const x
    nehead (Inl x) = unConst x
    nehead (Inr y) = error "Too many elements in list+"

\end{code}

\begin{code}
-- ** Focus on tags

--   Perform the argument rule inside the given tag.
inside :: String -> Rule -> Rule
inside n r (Ann l a) = liftM (==n) (mb2mplus (getFieldName l)) >>= bool2mplus >> do
        (View r b) <- r a
        return $ (View r (Ann l b))
inside _ _ _ = mzero

--   Perform the argument rule on the given tag.
atTag :: String -> Rule -> Rule
atTag n r x@(Ann l a) = liftM (==n) (mb2mplus (getFieldName l)) >>= bool2mplus >> r x
atTag _ _ _ = mzero

alterTag :: (TypeAnn -> TypeAnn) -> Type a -> Type a
alterTag f (Ann l t) = Ann (f l) t

--   Perform the argument rule on the elements with the given name of a list.
atListOf :: String -> Rule -> Rule
atListOf n r x@(List (Ann l a)) = liftM (==n) (mb2mplus (getFieldName l)) >>= bool2mplus >> r x
atListOf _ _ _ = mzero

onMapRange :: Rule -> Rule
onMapRange r (Map a b) = do
  (View (Rep to fro) b') <- r b
  return $ (View (Rep (Map.map to) (Map.map fro)) (Map a b'))
onMapRange _ _ = mzero

\end{code}

\begin{code}
-- ** Value-level combinators used to define two-level combinators.

--   Adds the argument entry to the a list.
-- Execute the from function only when the values where evoluted with the to function, otherwise the first element of the list is dropped.
addlistentry :: a -> Rep [a] [a]
addlistentry e = Rep {to = (e:), from = tail}

--   Value level function for adding an alternative somewhere in a list.
-- The backward function is complete.
-- Used by 'addaltlist'.
addaltlistrep :: Type c -> Type b -> Rep a b -> Rep a b
addaltlistrep alt (List t) rep = Rep {to = to rep, from = from rep . concat . List.map (removeAltRight alt t)}
    where
    findAltRight :: Type b -> Type a -> a -> Bool
    findAltRight alt (Either a _) (Left v) = findAltRight alt a v
    findAltRight alt (Either _ b) (Right v) = case (teq alt b) of
        Just Eq -> True
        Nothing -> findAltRight alt b v
    findAltRight alt (Prod a b) (x,y) = findAltRight alt a x || findAltRight alt b y
    findAltRight alt (List a) l = or $ List.map (findAltRight alt a) l
    findAltRight alt (Set a) s = or $ Set.fold (\h r -> findAltRight alt a h : r) [] s
    findAltRight alt (Map a b) m = or $ List.map (findAltRight alt a) (Map.keys m) ++ List.map (findAltRight alt b) (Map.elems m)
    findAltRight alt (Ann _ a) v = findAltRight alt a v
    findAltRight _ _ _ = False

    removeAltRight :: Type b -> Type a -> a -> [a]
    removeAltRight alt t v = if (findAltRight alt t v) then [] else [v]
addaltlistrep _ _ rep = rep

\end{code}

\begin{code}
-- ** Generic lookup

--   Perform the argument rule when the test function succeeds.
when :: (forall a . Type a -> Bool) -> Rule -> Rule
when f r t = if (f t)
        then (r t)
        else mzero

--   Remove the name from the current type.
untag :: Rule
untag (Ann l a) = return $ View idrep $ Ann (l {fieldName = Nothing}) a
untag _ = mzero

--  
tag :: String -> Rule
tag name (Ann t a) = return $ View idrep $ Ann t' a
    where
    t' = t { fieldName = Just name}
tag name a = return $ View idrep $ Ann t' a
    where
    t' = emptyTag { fieldName = Just name }

--   
changeTag :: String -> String -> Rule
changeTag old new = atTag old (untag >>> tag new)

\end{code}

\begin{code}


-- Lifts a function over a Maybe, returning a MonadPlus monad.
liftMb :: MonadPlus m => (a -> b) -> Maybe a -> m b
liftMb f mb = case mb of
    (Just x) -> return $ f x
    Nothing  -> mzero

mb2mplus :: MonadPlus m => Maybe a -> m a
mb2mplus (Just x) = return x
mb2mplus Nothing = mzero

bool2mplus :: MonadPlus m => Bool -> m ()
bool2mplus True = return ()
bool2mplus False = mzero

liftMbf :: MonadPlus m => (a -> b -> c) -> Maybe a -> (b -> m c)
liftMbf f mb = case mb of
    (Just x) -> return . f x
    Nothing  -> const mzero
    
\end{code}

\begin{code}
-- |isBasicType| returns true whenever the argument is a basic Type.
isBasicType :: Type a -> Bool
isBasicType Int = True
isBasicType Bool = True
isBasicType Char = True
isBasicType String = True
isBasicType One = True
isBasicType (Ann _ a) = isBasicType a
isBasicType _ = False

-- Returns true when the parameter is type One
isOne :: Type a -> Bool
isOne (Ann l a) = isOne a
isOne One = True
isOne _ = False


myTrace :: Rule
myTrace t = Trace.trace ("\n\nTrace: "++show t) (nop t)

myTraceH :: String -> Rule
myTraceH s t = Trace.trace ("\n\nTrace ["++s++"]: "++show t) (nop t)

\end{code}

\end{document}
