{-# OPTIONS -fglasgow-exts #-}
{-# OPTIONS -fallow-undecidable-instances #-}

-----------------------------------------------------------------------------
-- |
-- Maintainer  :  pabloberdaguer@alfa.di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- See
--
--     Type-safe Two-level Data Transformation to XML structures
--
-- by
--
--     Alcino Cunha, Jose Nuno Oliveira, Joost Visser
--
--
--
-----------------------------------------------------------------------------

module Data.Transform.TwoLevelXML where

import Data.Transform.TwoLevelMisc
import Control.Monad.State
import Data.Map as Map
import Data.List as List
--import Data.Set as Set

-- | Converts a relational type to a xml schema type representation.
toXML a = evalStateT (toXML' a) []

-- | Eiliminates maps from xml schemas.
toXML' :: Rule
toXML' = many (once map2xsd)

-- | Eliminates a map from xml schemas through several refinements.
map2xsd :: Rule
-- refinement to boolean
map2xsd (Tags l (Map One One)) = return $ View rep $ Tags l Bool
	where
	rep = Rep {to = fm2bool, from = bool2fm}
	fm2bool m = if (Map.null m) then False else True
	bool2fm b = if b then (Map.singleton () ()) else Map.empty
-- refinement to either
map2xsd (Tags l (Map One b)) = return $ View rep $ Tags l $ Either b One
	where
	rep = Rep {to = untotOne, from = totOne}
	untotOne m = if (Map.null m) then (Right ()) else (Left $ head $ Map.elems m)
	totOne =  either (Map.singleton ()) (const Map.empty)
-- refinement to set
--map2xsd (Tags l (Map a One)) = return $ View rep $ Tags l $ Set a
--	where
--	rep = Rep {to = dom, from = set2fm}	
--	set2fm s = Map.fromDistinctAscList (zip (Set.toAscList s) (repeat ()))
--	dom = Set.fromDistinctAscList . keys
-- refinement to list of products
map2xsd (Tags l (Map a b)) = return $ View rep $ Tags l $ List $ Prod a b
	where
	rep = Rep {to = Map.toAscList, from = Map.fromDistinctAscList}
		
map2xsd _ = mzero

