module Data.Transform.TwoLevelXmlSql 
{-
(
  module Data.Transform.TwoLevelXmlSql,
  module Data.Transform.TwoLevelFunctions,
  module Language.XMLSchema.Parse
 )
-}
  where

import Data.Transform.Type
import Data.Transform.TwoLevelCombinators
import Data.Transform.TwoLevelFunctions
import Data.Transform.CommandLine
import Language.XMLSchema.Parse
import Language.XMLSchema.NameSpaces
import Data.SQL.DBaseTS (DDL)
import Text.XML.HaXml.Pretty
import Text.XML.HaXml.Types
import Control.Monad.State hiding (when)
import Data.XML.TypetoXML		  -- Type to XML conversion
import Data.Transform.RulesWithRefs
import Text.XML.XsdAliases
import Data.SQL.DBase as DB		  -- Our database non-type-safe representation



applyRule :: Rule -> Type a -> Maybe (View (Type a))
applyRule r t = evalStateT (r t) (initNameSupply t)

parseXmlFile = xmlparse

evolve :: Rule
evolve = makeMaps
	 >>> evolution
    

fromJust (Just x) = x

makeMaps = once (movieAsMap)
	 >>> once (actorAsMap)

-- mytrace s x = trace (show s ++ " * " ++ show x) x

-- | Functions refeared to the examples

evolution :: Rule
evolution = once (changeTag "movie" "show") >>> once (when isMovie (tag "movie" >>> addalt series))
	where
	isMovie :: Type a -> Bool
	isMovie (Prod (Ann l1 _) (List (Ann l2 _))) = (fromJust (getFieldName l2) == "box_office") && (fromJust (getFieldName l1) == "director")
	isMovie _ = False
	series = Ann (emptyTag {fieldName = Just "series"}) (Map year episodes)
	year = Ann (emptyTag {fieldName = Just "year"}) Int
	episodes = Ann (emptyTag {fieldName = Just "episodes"}) (List (Prod name guestdir))
	name = Ann (emptyTag {fieldName = Just "name"}) String
	guestdir = Ann (emptyTag {fieldName = Just "guestdirector"}) String



movieAsMap = assocYearTitle >>> putYearTitleRef

actorAsMap t = do
  k   <- nextKey
  putNameRef k t

assocYearTitle = everywhere (try (assocl2 "title" "year"))

putNameRef k = once (when isActorList (list2map k))

putYearTitleRef :: Rule
putYearTitleRef t = do
  k   <- nextKey
  putYearTitleRef' k t
  
putYearTitleRef' :: KeyId -> Rule
putYearTitleRef' k =
     (once (when isMovieList (list2map k)))
     >>>
     (once (inside "actor" (everywhere (try (putRefs "title" "year" k)))))

isMovieList :: Type a -> Bool
isMovieList (List a)
  | getName a == Just "movie"  = True
  | otherwise                  = False
isMovieList _ = False

isActorList :: Type a -> Bool
isActorList (List a)
  | getName a == Just "actor"  = True
  | otherwise                  = False
isActorList _ = False



loadFromDB file = do
-- 	DynT t <- parsetype dml
	sqlData <- (flip fromRelationalDB) [file]
	return sqlData


instance Show Document where
  show x = show (document x)
