
-----------------------------------------------------------------------------
-- |
-- Maintainer  :  dtlapa@yahoo.com, flavioxavier@gmail.com, hpacheco@gmail.com
-- Stability   :  stable
-- Portability :  portable
--
-- See
--
--     Parsec Common Combinators
-- 
-- by
--
--     Digo Lapa, Flavio Ferreira and Hugo Pacheco
--
-----------------------------------------------------------------------------

module Text.SQL.ParsecCombinators where


import Text.ParserCombinators.Parsec

infixl 1 <!>
infixl 2 <*>
infixl 3 <$>

-- * Parser Combinators

-- | 'GenParser' and 'GenParser'
(<*>) :: GenParser tok st (a -> r) -> GenParser tok st a -> GenParser tok st r
p <*> q = do    f <- p
                v <- q
                return (f v)
-- ^ Applies the parser in left to right order

-- | parser cast
(<$>) :: (a -> r) -> GenParser tok st a -> GenParser tok st r
f <$> p = do    r <- p
                return (f r)
-- ^ Applies a function to the parser result

-- | 'GenParser' or 'GenParser'
(<!>) :: GenParser tok st a -> GenParser tok st a -> GenParser tok st a
p <!> q = try p <|> q
-- ^ Applies either of the two parsers (the left one first)
-- My desired version of '<|>', in a way that if the first parser fails it rewinds the state

-- | succeed
succeed :: a -> GenParser tok st a
succeed a = do return a
-- ^ Creates a parser that always succeeds returning a

-- | zero or more 'GenParser'
zeroOrMore :: GenParser tok st [a] -> GenParser tok st [a]
zeroOrMore p = (++) <$> p <*> zeroOrMore p <!> succeed []
-- ^ Succeeds if the parser can be applied zero or more times

-- | one or more 'GenParser'
oneOrMore :: GenParser tok st [a] -> GenParser tok st [a]
oneOrMore p = (++) <$> p <*> oneOrMore p <!> p
-- ^ Succeeds if the parser can be applied one or more times

