
-----------------------------------------------------------------------------
-- |
-- Maintainer  :  dtlapa@yahoo.com, flavioxavier@gmail.com, hpacheco@gmail.com
-- Stability   :  experimental
-- Portability :  portable
--
-- See
--
--     SQL Lexer Tokens
-- 
-- by
--
--     Digo Lapa, Flavio Ferreira and Hugo Pacheco
--
-----------------------------------------------------------------------------

module Text.SQL.Tokens where

import Char
import Text.ParserCombinators.Parsec.Pos

-- * Data Types

-- | Token Definition
data Token = Token TokenType SourcePos
	deriving Show

-- | Token Type Definition
data TokenType
	= TokKeyword String
	| TokTypecast
	| TokIdent String
	| TokFconst String	-- decimal,real
	| TokSconst String	-- quoted, dollar quoted strings
	| TokBconst String	-- binary string
	| TokXconst String	-- hexadecimal string
	| TokOp String
	| TokIconst Int		-- integer
	| TokParam Int		-- param
	| TokSymbol Char
	deriving Show

scanKeyword :: String -> SourcePos -> Maybe Token
scanKeyword str pos = case (map toLower str) of
			"abort" -> Just $ Token (TokKeyword "ABORT") pos
			"absolute" -> Just $ Token (TokKeyword "ABSOLUTE") pos
			"access" -> Just $ Token (TokKeyword "ACCESS") pos
			"action" -> Just $ Token (TokKeyword "ACTION") pos
			"add" -> Just $ Token (TokKeyword "ADD") pos
			"admin" -> Just $ Token (TokKeyword "ADMIN") pos
			"after" -> Just $ Token (TokKeyword "AFTER") pos
			"aggregate" -> Just $ Token (TokKeyword "AGGREGATE") pos
			"all" -> Just $ Token (TokKeyword "ALL") pos
			"also" -> Just $ Token (TokKeyword "ALSO") pos
			"alter" -> Just $ Token (TokKeyword "ALTER") pos
			"analyse" -> Just $ Token (TokKeyword "ANALYSE") pos
			"analyze" -> Just $ Token (TokKeyword "ANALYZE") pos
			"and" -> Just $ Token (TokKeyword "AND") pos
			"any" -> Just $ Token (TokKeyword "ANY") pos
			"array" -> Just $ Token (TokKeyword "ARRAY") pos
			"as" -> Just $ Token (TokKeyword "AS") pos
			"asc" -> Just $ Token (TokKeyword "ASC") pos
			"assertion" -> Just $ Token (TokKeyword "ASSERTION") pos
			"assignment" -> Just $ Token (TokKeyword "ASSIGNMENT") pos
			"asymmetric" -> Just $ Token (TokKeyword "ASYMMETRIC") pos
			"at" -> Just $ Token (TokKeyword "AT") pos
			"authorization" -> Just $ Token (TokKeyword "AUTHORIZATION") pos
			"backward" -> Just $ Token (TokKeyword "BACKWARD") pos
			"before" -> Just $ Token (TokKeyword "BEFORE") pos
			"begin" -> Just $ Token (TokKeyword "BEGIN") pos
			"between" -> Just $ Token (TokKeyword "BETWEEN") pos
			"bigint" -> Just $ Token (TokKeyword "BIGINT") pos
			"binary" -> Just $ Token (TokKeyword "BINARY") pos
			"bit" -> Just $ Token (TokKeyword "BIT") pos
			"boolean" -> Just $ Token (TokKeyword "BOOLEAN") pos
			"both" -> Just $ Token (TokKeyword "BOTH") pos
			"by" -> Just $ Token (TokKeyword "BY") pos
			"cache" -> Just $ Token (TokKeyword "CACHE") pos
			"called" -> Just $ Token (TokKeyword "CALLED") pos
			"cascade" -> Just $ Token (TokKeyword "CASCADE") pos
			"case" -> Just $ Token (TokKeyword "CASE") pos
			"cast" -> Just $ Token (TokKeyword "CAST") pos
			"chain" -> Just $ Token (TokKeyword "CHAIN") pos
			"char" -> Just $ Token (TokKeyword "CHAR") pos
			"character" -> Just $ Token (TokKeyword "CHARACTER") pos
			"characteristics" -> Just $ Token (TokKeyword "CHARACTERISTICS") pos
			"check" -> Just $ Token (TokKeyword "CHECK") pos
			"checkpoint" -> Just $ Token (TokKeyword "CHECKPOINT") pos
			"class" -> Just $ Token (TokKeyword "CLASS") pos
			"close" -> Just $ Token (TokKeyword "CLOSE") pos
			"cluster" -> Just $ Token (TokKeyword "CLUSTER") pos
			"coalesce" -> Just $ Token (TokKeyword "COALESCE") pos
			"collate" -> Just $ Token (TokKeyword "COLLATE") pos
			"column" -> Just $ Token (TokKeyword "COLUMN") pos
			"comment" -> Just $ Token (TokKeyword "COMMENT") pos
			"commit" -> Just $ Token (TokKeyword "COMMIT") pos
			"committed" -> Just $ Token (TokKeyword "COMMITTED") pos
			"connection" -> Just $ Token (TokKeyword "CONNECTION") pos
			"constraint" -> Just $ Token (TokKeyword "CONSTRAINT") pos
			"constraints" -> Just $ Token (TokKeyword "CONSTRAINTS") pos
			"conversion" -> Just $ Token (TokKeyword "CONVERSION") pos
			"convert" -> Just $ Token (TokKeyword "CONVERT") pos
			"copy" -> Just $ Token (TokKeyword "COPY") pos
			"create" -> Just $ Token (TokKeyword "CREATE") pos
			"createdb" -> Just $ Token (TokKeyword "CREATEDB") pos
			"createrole" -> Just $ Token (TokKeyword "CREATEROLE") pos
			"createuser" -> Just $ Token (TokKeyword "CREATEUSER") pos
			"cross" -> Just $ Token (TokKeyword "CROSS") pos
			"csv" -> Just $ Token (TokKeyword "CSV") pos
			"current_date" -> Just $ Token (TokKeyword "CURRENT_DATE") pos
			"current_role" -> Just $ Token (TokKeyword "CURRENT_ROLE") pos
			"current_time" -> Just $ Token (TokKeyword "CURRENT_TIME") pos
			"current_timestamp" -> Just $ Token (TokKeyword "CURRENT_TIMESTAMP") pos
			"current_user" -> Just $ Token (TokKeyword "CURRENT_USER") pos
			"cursor" -> Just $ Token (TokKeyword "CURSOR") pos
			"cycle" -> Just $ Token (TokKeyword "CYCLE") pos
			"database" -> Just $ Token (TokKeyword "DATABASE") pos
			"day" -> Just $ Token (TokKeyword "DAY") pos
			"deallocate" -> Just $ Token (TokKeyword "DEALLOCATE") pos
			"dec" -> Just $ Token (TokKeyword "DEC") pos
			"decimal" -> Just $ Token (TokKeyword "DECIMAL") pos
			"declare" -> Just $ Token (TokKeyword "DECLARE") pos
			"default" -> Just $ Token (TokKeyword "DEFAULT") pos
			"defaults" -> Just $ Token (TokKeyword "DEFAULTS") pos
			"deferrable" -> Just $ Token (TokKeyword "DEFERRABLE") pos
			"deferred" -> Just $ Token (TokKeyword "DEFERRED") pos
			"definer" -> Just $ Token (TokKeyword "DEFINER") pos
			"delete" -> Just $ Token (TokKeyword "DELETE") pos
			"delimiter" -> Just $ Token (TokKeyword "DELIMITER") pos
			"delimiters" -> Just $ Token (TokKeyword "DELIMITERS") pos
			"desc" -> Just $ Token (TokKeyword "DESC") pos
			"disable" -> Just $ Token (TokKeyword "DISABLE") pos
			"distinct" -> Just $ Token (TokKeyword "DISTINCT") pos
			"do" -> Just $ Token (TokKeyword "DO") pos
			"domain" -> Just $ Token (TokKeyword "DOMAIN") pos
			"double" -> Just $ Token (TokKeyword "DOUBLE") pos
			"drop" -> Just $ Token (TokKeyword "DROP") pos
			"each" -> Just $ Token (TokKeyword "EACH") pos
			"else" -> Just $ Token (TokKeyword "ELSE") pos
			"enable" -> Just $ Token (TokKeyword "ENABLE") pos
			"encoding" -> Just $ Token (TokKeyword "ENCODING") pos
			"encrypted" -> Just $ Token (TokKeyword "ENCRYPTED") pos
			"end" -> Just $ Token (TokKeyword "END") pos
			"escape" -> Just $ Token (TokKeyword "ESCAPE") pos
			"except" -> Just $ Token (TokKeyword "EXCEPT") pos
			"excluding" -> Just $ Token (TokKeyword "EXCLUDING") pos
			"exclusive" -> Just $ Token (TokKeyword "EXCLUSIVE") pos
			"execute" -> Just $ Token (TokKeyword "EXECUTE") pos
			"exists" -> Just $ Token (TokKeyword "EXISTS") pos
			"explain" -> Just $ Token (TokKeyword "EXPLAIN") pos
			"external" -> Just $ Token (TokKeyword "EXTERNAL") pos
			"extract" -> Just $ Token (TokKeyword "EXTRACT") pos
			"false" -> Just $ Token (TokKeyword "FALSE") pos
			"fetch" -> Just $ Token (TokKeyword "FETCH") pos
			"first" -> Just $ Token (TokKeyword "FIRST") pos
			"float" -> Just $ Token (TokKeyword "FLOAT") pos
			"for" -> Just $ Token (TokKeyword "FOR") pos
			"force" -> Just $ Token (TokKeyword "FORCE") pos
			"foreign" -> Just $ Token (TokKeyword "FOREIGN") pos
			"forward" -> Just $ Token (TokKeyword "FORWARD") pos
			"freeze" -> Just $ Token (TokKeyword "FREEZE") pos
			"from" -> Just $ Token (TokKeyword "FROM") pos
			"full" -> Just $ Token (TokKeyword "FULL") pos
			"function" -> Just $ Token (TokKeyword "FUNCTION") pos
			"global" -> Just $ Token (TokKeyword "GLOBAL") pos
			"grant" -> Just $ Token (TokKeyword "GRANT") pos
			"granted" -> Just $ Token (TokKeyword "GRANTED") pos
			"greatest" -> Just $ Token (TokKeyword "GREATEST") pos
			"group" -> Just $ Token (TokKeyword "GROUP") pos
			"handler" -> Just $ Token (TokKeyword "HANDLER") pos
			"having" -> Just $ Token (TokKeyword "HAVING") pos
			"header" -> Just $ Token (TokKeyword "HEADER") pos
			"hold" -> Just $ Token (TokKeyword "HOLD") pos
			"hour" -> Just $ Token (TokKeyword "HOUR") pos
			"ilike" -> Just $ Token (TokKeyword "ILIKE") pos
			"immediate" -> Just $ Token (TokKeyword "IMMEDIATE") pos
			"immutable" -> Just $ Token (TokKeyword "IMMUTABLE") pos
			"implicit" -> Just $ Token (TokKeyword "IMPLICIT") pos
			"in" -> Just $ Token (TokKeyword "IN") pos
			"including" -> Just $ Token (TokKeyword "INCLUDING") pos
			"increment" -> Just $ Token (TokKeyword "INCREMENT") pos
			"index" -> Just $ Token (TokKeyword "INDEX") pos
			"inherit" -> Just $ Token (TokKeyword "INHERIT") pos
			"inherits" -> Just $ Token (TokKeyword "INHERITS") pos
			"initially" -> Just $ Token (TokKeyword "INITIALLY") pos
			"inner" -> Just $ Token (TokKeyword "INNER") pos
			"inout" -> Just $ Token (TokKeyword "INOUT") pos
			"input" -> Just $ Token (TokKeyword "INPUT") pos
			"insensitive" -> Just $ Token (TokKeyword "INSENSITIVE") pos
			"insert" -> Just $ Token (TokKeyword "INSERT") pos
			"instead" -> Just $ Token (TokKeyword "INSTEAD") pos
			"int" -> Just $ Token (TokKeyword "INT") pos
			"integer" -> Just $ Token (TokKeyword "INTEGER") pos
			"intersect" -> Just $ Token (TokKeyword "INTERSECT") pos
			"interval" -> Just $ Token (TokKeyword "INTERVAL") pos
			"into" -> Just $ Token (TokKeyword "INTO") pos
			"invoker" -> Just $ Token (TokKeyword "INVOKER") pos
			"is" -> Just $ Token (TokKeyword "IS") pos
			"isnull" -> Just $ Token (TokKeyword "ISNULL") pos
			"isolation" -> Just $ Token (TokKeyword "ISOLATION") pos
			"join" -> Just $ Token (TokKeyword "JOIN") pos
			"key" -> Just $ Token (TokKeyword "KEY") pos
			"lancompiler" -> Just $ Token (TokKeyword "LANCOMPILER") pos
			"language" -> Just $ Token (TokKeyword "LANGUAGE") pos
			"large" -> Just $ Token (TokKeyword "LARGE") pos
			"last" -> Just $ Token (TokKeyword "LAST") pos
			"leading" -> Just $ Token (TokKeyword "LEADING") pos
			"least" -> Just $ Token (TokKeyword "LEAST") pos
			"left" -> Just $ Token (TokKeyword "LEFT") pos
			"level" -> Just $ Token (TokKeyword "LEVEL") pos
			"like" -> Just $ Token (TokKeyword "LIKE") pos
			"limit" -> Just $ Token (TokKeyword "LIMIT") pos
			"listen" -> Just $ Token (TokKeyword "LISTEN") pos
			"load" -> Just $ Token (TokKeyword "LOAD") pos
			"local" -> Just $ Token (TokKeyword "LOCAL") pos
			"localtime" -> Just $ Token (TokKeyword "LOCALTIME") pos
			"localtimestamp" -> Just $ Token (TokKeyword "LOCALTIMESTAMP") pos
			"location" -> Just $ Token (TokKeyword "LOCATION") pos
			"lock" -> Just $ Token (TokKeyword "LOCK") pos
			"login" -> Just $ Token (TokKeyword "LOGIN") pos
			"match" -> Just $ Token (TokKeyword "MATCH") pos
			"maxvalue" -> Just $ Token (TokKeyword "MAXVALUE") pos
			"minute" -> Just $ Token (TokKeyword "MINUTE") pos
			"minvalue" -> Just $ Token (TokKeyword "MINVALUE") pos
			"mode" -> Just $ Token (TokKeyword "MODE") pos
			"month" -> Just $ Token (TokKeyword "MONTH") pos
			"move" -> Just $ Token (TokKeyword "MOVE") pos
			"names" -> Just $ Token (TokKeyword "NAMES") pos
			"national" -> Just $ Token (TokKeyword "NATIONAL") pos
			"natural" -> Just $ Token (TokKeyword "NATURAL") pos
			"nchar" -> Just $ Token (TokKeyword "NCHAR") pos
			"new" -> Just $ Token (TokKeyword "NEW") pos
			"next" -> Just $ Token (TokKeyword "NEXT") pos
			"no" -> Just $ Token (TokKeyword "NO") pos
			"nocreatedb" -> Just $ Token (TokKeyword "NOCREATEDB") pos
			"nocreaterole" -> Just $ Token (TokKeyword "NOCREATEROLE") pos
			"nocreateuser" -> Just $ Token (TokKeyword "NOCREATEUSER") pos
			"noinherit" -> Just $ Token (TokKeyword "NOINHERIT") pos
			"nologin" -> Just $ Token (TokKeyword "NOLOGIN") pos
			"none" -> Just $ Token (TokKeyword "NONE") pos
			"nosuperuser" -> Just $ Token (TokKeyword "NOSUPERUSER") pos
			"not" -> Just $ Token (TokKeyword "NOT") pos
			"nothing" -> Just $ Token (TokKeyword "NOTHING") pos
			"notify" -> Just $ Token (TokKeyword "NOTIFY") pos
			"notnull" -> Just $ Token (TokKeyword "NOTNULL") pos
			"nowait" -> Just $ Token (TokKeyword "NOWAIT") pos
			"null" -> Just $ Token (TokKeyword "NULL") pos
			"nullif" -> Just $ Token (TokKeyword "NULLIF") pos
			"numeric" -> Just $ Token (TokKeyword "NUMERIC") pos
			"object" -> Just $ Token (TokKeyword "OBJECT") pos
			"of" -> Just $ Token (TokKeyword "OF") pos
			"off" -> Just $ Token (TokKeyword "OFF") pos
			"offset" -> Just $ Token (TokKeyword "OFFSET") pos
			"oids" -> Just $ Token (TokKeyword "OIDS") pos
			"old" -> Just $ Token (TokKeyword "OLD") pos
			"on" -> Just $ Token (TokKeyword "ON") pos
			"only" -> Just $ Token (TokKeyword "ONLY") pos
			"operator" -> Just $ Token (TokKeyword "OPERATOR") pos
			"option" -> Just $ Token (TokKeyword "OPTION") pos
			"or" -> Just $ Token (TokKeyword "OR") pos
			"order" -> Just $ Token (TokKeyword "ORDER") pos
			"out" -> Just $ Token (TokKeyword "OUT") pos
			"outer" -> Just $ Token (TokKeyword "OUTER") pos
			"overlaps" -> Just $ Token (TokKeyword "OVERLAPS") pos
			"overlay" -> Just $ Token (TokKeyword "OVERLAY") pos
			"owner" -> Just $ Token (TokKeyword "OWNER") pos
			"partial" -> Just $ Token (TokKeyword "PARTIAL") pos
			"password" -> Just $ Token (TokKeyword "PASSWORD") pos
			"placing" -> Just $ Token (TokKeyword "PLACING") pos
			"position" -> Just $ Token (TokKeyword "POSITION") pos
			"precision" -> Just $ Token (TokKeyword "PRECISION") pos
			"prepare" -> Just $ Token (TokKeyword "PREPARE") pos
			"prepared" -> Just $ Token (TokKeyword "PREPARED") pos
			"preserve" -> Just $ Token (TokKeyword "PRESERVE") pos
			"primary" -> Just $ Token (TokKeyword "PRIMARY") pos
			"prior" -> Just $ Token (TokKeyword "PRIOR") pos
			"privileges" -> Just $ Token (TokKeyword "PRIVILEGES") pos
			"procedural" -> Just $ Token (TokKeyword "PROCEDURAL") pos
			"procedure" -> Just $ Token (TokKeyword "PROCEDURE") pos
			"quote" -> Just $ Token (TokKeyword "QUOTE") pos
			"read" -> Just $ Token (TokKeyword "READ") pos
			"real" -> Just $ Token (TokKeyword "REAL") pos
			"recheck" -> Just $ Token (TokKeyword "RECHECK") pos
			"references" -> Just $ Token (TokKeyword "REFERENCES") pos
			"reindex" -> Just $ Token (TokKeyword "REINDEX") pos
			"relative" -> Just $ Token (TokKeyword "RELATIVE") pos
			"release" -> Just $ Token (TokKeyword "RELEASE") pos
			"rename" -> Just $ Token (TokKeyword "RENAME") pos
			"repeatable" -> Just $ Token (TokKeyword "REPEATABLE") pos
			"replace" -> Just $ Token (TokKeyword "REPLACE") pos
			"reset" -> Just $ Token (TokKeyword "RESET") pos
			"restart" -> Just $ Token (TokKeyword "RESTART") pos
			"restrict" -> Just $ Token (TokKeyword "RESTRICT") pos
			"returns" -> Just $ Token (TokKeyword "RETURNS") pos
			"revoke" -> Just $ Token (TokKeyword "REVOKE") pos
			"right" -> Just $ Token (TokKeyword "RIGHT") pos
			"role" -> Just $ Token (TokKeyword "ROLE") pos
			"rollback" -> Just $ Token (TokKeyword "ROLLBACK") pos
			"row" -> Just $ Token (TokKeyword "ROW") pos
			"rows" -> Just $ Token (TokKeyword "ROWS") pos
			"rule" -> Just $ Token (TokKeyword "RULE") pos
			"savepoint" -> Just $ Token (TokKeyword "SAVEPOINT") pos
			"schema" -> Just $ Token (TokKeyword "SCHEMA") pos
			"scroll" -> Just $ Token (TokKeyword "SCROLL") pos
			"second" -> Just $ Token (TokKeyword "SECOND") pos
			"security" -> Just $ Token (TokKeyword "SECURITY") pos
			"select" -> Just $ Token (TokKeyword "SELECT") pos
			"sequence" -> Just $ Token (TokKeyword "SEQUENCE") pos
			"serializable" -> Just $ Token (TokKeyword "SERIALIZABLE") pos
			"session" -> Just $ Token (TokKeyword "SESSION") pos
			"session_user" -> Just $ Token (TokKeyword "SESSION_USER") pos
			"set" -> Just $ Token (TokKeyword "SET") pos
			"setof" -> Just $ Token (TokKeyword "SETOF") pos
			"share" -> Just $ Token (TokKeyword "SHARE") pos
			"show" -> Just $ Token (TokKeyword "SHOW") pos
			"similar" -> Just $ Token (TokKeyword "SIMILAR") pos
			"simple" -> Just $ Token (TokKeyword "SIMPLE") pos
			"smallint" -> Just $ Token (TokKeyword "SMALLINT") pos
			"some" -> Just $ Token (TokKeyword "SOME") pos
			"stable" -> Just $ Token (TokKeyword "STABLE") pos
			"start" -> Just $ Token (TokKeyword "START") pos
			"statement" -> Just $ Token (TokKeyword "STATEMENT") pos
			"statistics" -> Just $ Token (TokKeyword "STATISTICS") pos
			"stdin" -> Just $ Token (TokKeyword "STDIN") pos
			"stdout" -> Just $ Token (TokKeyword "STDOUT") pos
			"storage" -> Just $ Token (TokKeyword "STORAGE") pos
			"strict" -> Just $ Token (TokKeyword "STRICT") pos
			"substring" -> Just $ Token (TokKeyword "SUBSTRING") pos
			"superuser" -> Just $ Token (TokKeyword "SUPERUSER") pos
			"symmetric" -> Just $ Token (TokKeyword "SYMMETRIC") pos
			"sysid" -> Just $ Token (TokKeyword "SYSID") pos
			"system" -> Just $ Token (TokKeyword "SYSTEM") pos
			"table" -> Just $ Token (TokKeyword "TABLE") pos
			"tablespace" -> Just $ Token (TokKeyword "TABLESPACE") pos
			"temp" -> Just $ Token (TokKeyword "TEMP") pos
			"template" -> Just $ Token (TokKeyword "TEMPLATE") pos
			"temporary" -> Just $ Token (TokKeyword "TEMPORARY") pos
			"then" -> Just $ Token (TokKeyword "THEN") pos
			"time" -> Just $ Token (TokKeyword "TIME") pos
			"timestamp" -> Just $ Token (TokKeyword "TIMESTAMP") pos
			"to" -> Just $ Token (TokKeyword "TO") pos
			"toast" -> Just $ Token (TokKeyword "TOAST") pos
			"trailing" -> Just $ Token (TokKeyword "TRAILING") pos
			"transaction" -> Just $ Token (TokKeyword "TRANSACTION") pos
			"treat" -> Just $ Token (TokKeyword "TREAT") pos
			"trigger" -> Just $ Token (TokKeyword "TRIGGER") pos
			"trim" -> Just $ Token (TokKeyword "TRIM") pos
			"true" -> Just $ Token (TokKeyword "TRUE") pos
			"truncate" -> Just $ Token (TokKeyword "TRUNCATE") pos
			"trusted" -> Just $ Token (TokKeyword "TRUSTED") pos
			"type" -> Just $ Token (TokKeyword "TYPE") pos
			"uncommitted" -> Just $ Token (TokKeyword "UNCOMMITTED") pos
			"unencrypted" -> Just $ Token (TokKeyword "UNENCRYPTED") pos
			"union" -> Just $ Token (TokKeyword "UNION") pos
			"unique" -> Just $ Token (TokKeyword "UNIQUE") pos
			"unknown" -> Just $ Token (TokKeyword "UNKNOWN") pos
			"unlisten" -> Just $ Token (TokKeyword "UNLISTEN") pos
			"until" -> Just $ Token (TokKeyword "UNTIL") pos
			"update" -> Just $ Token (TokKeyword "UPDATE") pos
			"user" -> Just $ Token (TokKeyword "USER") pos
			"using" -> Just $ Token (TokKeyword "USING") pos
			"vacuum" -> Just $ Token (TokKeyword "VACUUM") pos
			"valid" -> Just $ Token (TokKeyword "VALID") pos
			"validator" -> Just $ Token (TokKeyword "VALIDATOR") pos
			"values" -> Just $ Token (TokKeyword "VALUES") pos
			"varchar" -> Just $ Token (TokKeyword "VARCHAR") pos
			"varying" -> Just $ Token (TokKeyword "VARYING") pos
			"verbose" -> Just $ Token (TokKeyword "VERBOSE") pos
			"view" -> Just $ Token (TokKeyword "VIEW") pos
			"volatile" -> Just $ Token (TokKeyword "VOLATILE") pos
			"when" -> Just $ Token (TokKeyword "WHEN") pos
			"where" -> Just $ Token (TokKeyword "WHERE") pos
			"with" -> Just $ Token (TokKeyword "WITH") pos
			"without" -> Just $ Token (TokKeyword "WITHOUT") pos
			"work" -> Just $ Token (TokKeyword "WORK") pos
			"write" -> Just $ Token (TokKeyword "WRITE") pos
			"year" -> Just $ Token (TokKeyword "YEAR") pos
			"zone" -> Just $ Token (TokKeyword "ZONE") pos
			otherwise -> Nothing
-- ^ returns the keyword 'Token' for the given string
			