{-# OPTIONS -fglasgow-exts -fallow-undecidable-instances -fallow-overlapping-instances #-}
-----------------------------------------------------------------------------
-- |
-- Maintainer  :  pabloberdaguer@di.uminho.pt
-- Stability   :  experimental
-- Portability :  portable
--
-- See
--
--     Two Level Transformation: XML <-> SQL
--
-- by
--
--     Diogo Lapa, Flavio Ferreira and Hugo Pacheco
--
-----------------------------------------------------------------------------


module Main where


import Data.SQL.DBase as DB
import Text.SQL.HSQLAliases

import Text.SQL.PrettyPrintDBaseTS
import Data.Transform.CommandLine
import System.IO

import Data.Transform.Type
import Data.Transform.TwoLevelFunctions
import Data.Transform.TwoLevelCombinators
import Data.Transform.RulesWithRefs

import Maybe

-------------------------------------------------------------------------------------
-- | Test functions

-- Main function
main :: IO ()
main = transformWithExternalOptions Main.evolve noevolve

-- | Test HSQL binding
{-testc :: FilePath -> FilePath -> IO ()
testc xsd xml = do
	schema <- xsdparse xsd
	doc <- xmlparse xml
	let (Just (ddl,to,from)) = transform schema (evolve >>> toRDB')
	let (Just dml) = to doc
	let conn = connect "localhost" "test2" "postgresql" "pacheco"
	bracket conn disconnect $ \c ->
		inTransaction c $ \c -> do
				createDB conn ddl
				loadDB conn dml
				dml' <- dumpDB c ddl
				print dml'
-}
-- | Test legrandefinale
test :: FilePath -> FilePath -> IO ()
test xsd sql = do
	schema <- xsdparse xsd
	dml1 <- sql2dbase [sql] (DB.DB [])
	let Just (sql1,sql2,to,from) = legrandefinale schema noevolve
	putStrLn "\nsql1:\n"
	putStrLn $ Text.SQL.PrettyPrintDBaseTS.sshow sql1
	putStrLn "\nsql2:\n"
	putStrLn $ Text.SQL.PrettyPrintDBaseTS.sshow sql2
	putStrLn "\nsql1 to sql2:\n"
	let (Just x) = to dml1
	putStrLn $ Text.SQL.PrettyPrintDBaseTS.sshow x

noevolve :: Rule
noevolve = movieAsMap >>> actorAsMap

evolve :: Rule
evolve = movieAsMap >>> actorAsMap
	 >>> evolution


-- mytrace s x = trace (show s ++ " * " ++ show x) x

-- | Functions refeared to the examples

evolution :: Rule
evolution = once (changeTag "movie" "show") >>> once (when isMovie (tag "movie" >>> addalt series))
	where
	isMovie :: Type a -> Bool
	isMovie (Prod (Ann l1 _) (List (Ann l2 _))) = (fromJust (getFieldName l2) == "box_office") && (fromJust (getFieldName l1) == "director")
	isMovie _ = False
	series = Ann (emptyTag {fieldName = Just "series"}) (Map year episodes)
	year = Ann (emptyTag {fieldName = Just "year"}) Int
	episodes = Ann (emptyTag {fieldName = Just "episodes"}) (List (Prod name guestdir))
	name = Ann (emptyTag {fieldName = Just "name"}) String
	guestdir = Ann (emptyTag {fieldName = Just "guestdirector"}) String


movieAsMap = assocYearTitle >>> putYearTitleRef

actorAsMap t = do
  k   <- nextKey
  putNameRef k t

assocYearTitle = everywhere (try (assocl2 "title" "year"))

putNameRef k = once (when isActorList (list2map k))

putYearTitleRef :: Rule
putYearTitleRef t = do
  k   <- nextKey
  putYearTitleRef' k t
  
putYearTitleRef' :: KeyId -> Rule
putYearTitleRef' k =
     (once (when isMovieList (list2map k)))
     >>>
     (once (inside "actor" (everywhere (try (putRefs "title" "year" k)))))

isMovieList :: Type a -> Bool
isMovieList (List a)
  | getName a == Just "movie"  = True
  | otherwise                  = False
isMovieList _ = False

isActorList :: Type a -> Bool
isActorList (List a)
  | getName a == Just "actor"  = True
  | otherwise                  = False
isActorList _ = False


